@@ -26,8 +26,8 @@ from senlin.objects import base as obj_base
 def policy_enforce(handler):
     """Decorator that enforces policies.
 
-    Checks the path matches the request context and enforce policy defined in
-    policy.json.
+    Check the path matches the request context and enforce policy defined in
+    policy file and policies in code.
 
     This is a handler method decorator.
     """
@@ -45,10 +45,10 @@ def policy_enforce(handler):
 
 
 def parse_request(name, req, body, key=None):
-    """Formalize an API request and validates it.
+    """Formalize an API request and validate it.
 
     :param name: The name for a versioned request object.
-    :param req: Reference to a WSGI reqeust object.
+    :param req: Reference to a WSGI request object.
     :param body: The JSON body (if any) that accompanies a request. Could be
                  augmented by controller before getting passed here.
     :param key: An optional key indicating the inner object for a request.
@@ -77,6 +77,7 @@ def parse_request(name, req, body, key=None):
     # Do version coversion if necessary
     if obj is not None and version != req_cls.VERSION:
         obj.obj_make_compatible(primitive, version)
+        return req_cls.obj_from_primitive(primitive)
 
     return obj
 
