@@ -11,17 +11,16 @@
 # under the License.
 
 from oslo_log import log as logging
-from oslo_serialization import jsonutils
 import six
 from six.moves.urllib import parse as urlparse
 
+from senlin.api.common import util
+from senlin.api.common import wsgi
 from senlin.common import context
 from senlin.common import exception as exc
 from senlin.common.i18n import _
-from senlin.common import utils
-from senlin.common import wsgi
 from senlin.drivers import base as driver_base
-from senlin.engine import webhook as webhook_mod
+from senlin.rpc import client as rpc
 
 LOG = logging.getLogger(__name__)
 
@@ -38,69 +37,57 @@ class WebhookMiddleware(wsgi.Middleware):
         if req.method != 'POST':
             return
 
-        # Extract project, webhook ID and key
+        # Extract webhook (receiver) ID and params
         results = self._parse_url(req.url)
         if not results:
             return
 
-        (project, webhook_id, key) = results
+        (receiver_id, params) = results
 
-        credential = self._get_credential(project, webhook_id, key)
-        if not credential:
-            return
+        api_version = str(req.version_request)
+        ctx = context.RequestContext(is_admin=True, api_version=api_version)
+        req.context = ctx
 
-        svc_ctx = context.get_service_context()
+        obj = util.parse_request(
+            'ReceiverGetRequest', req, {'identity': receiver_id})
+        rpcc = rpc.EngineClient()
+        receiver = rpcc.call(ctx, 'receiver_get', obj)
+
+        svc_ctx = context.get_service_credentials()
         kwargs = {
             'auth_url': svc_ctx['auth_url'],
             'username': svc_ctx['username'],
             'user_domain_name': svc_ctx['user_domain_name'],
             'password': svc_ctx['password']
         }
-        kwargs.update(credential)
+        kwargs.update(receiver['actor'])
 
         # Get token and fill it into the request header
         token = self._get_token(**kwargs)
         req.headers['X-Auth-Token'] = token
 
     def _parse_url(self, url):
-        """Extract webhook ID from the request URL.
+        """Extract receiver ID from the request URL.
 
+        Parse a URL of format: http://host:port/webhooks/id/trigger?V=1&k=v
         :param url: The URL from which the request is received.
         """
         parts = urlparse.urlparse(url)
-        components = parts.path.split('/')
-        if len(components) < 6:
+        p = parts.path.split('/')
+
+        # expected: ['', 'v1', 'webhooks', 'webhook-id', 'trigger']
+        if len(p) != 5:
             return None
 
-        if any((components[0] != '', components[3] != 'webhooks',
-                components[5] != 'trigger')):
+        if any((p[0] != '', p[2] != 'webhooks', p[4] != 'trigger')):
             return None
 
         qs = urlparse.parse_qs(parts.query)
-        if 'key' not in qs:
+        if 'V' not in qs:
             return None
-
-        return components[2], components[4], qs['key'][0]
-
-    def _get_credential(self, project, webhook_id, key):
-        """Get credential for the given webhook using the provided key.
-
-        :param webhook_id: ID of the webhook.
-        :param key: The key string to be used for decryption.
-        """
-        # Build a dummy RequestContext for DB APIs
-        dbctx = context.RequestContext(is_admin=True, project=project)
-        webhook = webhook_mod.Webhook.load(dbctx, webhook_id)
-        credential = webhook.credential
-        # Decrypt the credential using provided key
-        try:
-            cdata = utils.decrypt(webhook.credential, key)
-            credential = jsonutils.loads(cdata)
-        except Exception as ex:
-            LOG.exception(six.text_type(ex))
-            raise exc.Forbidden()
-
-        return credential
+        qs.pop('V')
+        params = dict((k, v[0]) for k, v in qs.items())
+        return p[3], params
 
     def _get_token(self, **kwargs):
         """Get a valid token based on the credential provided.
