@@ -26,17 +26,18 @@ from senlin.common.i18n import _
 class ClusterController(wsgi.Controller):
     """WSGI controller for clusters resource in Senlin v1 API."""
 
-    # Define request scope (must match what is in policy.json)
+    # Define request scope
+    # (must match what is in policy file and policies in code.)
     REQUEST_SCOPE = 'clusters'
 
     SUPPORTED_ACTIONS = (
         ADD_NODES, DEL_NODES, SCALE_OUT, SCALE_IN, RESIZE,
         POLICY_ATTACH, POLICY_DETACH, POLICY_UPDATE,
-        CHECK, RECOVER, REPLACE_NODES
+        CHECK, RECOVER, REPLACE_NODES, COMPLETE_LIFECYCLE
     ) = (
         'add_nodes', 'del_nodes', 'scale_out', 'scale_in', 'resize',
         'policy_attach', 'policy_detach', 'policy_update',
-        'check', 'recover', 'replace_nodes'
+        'check', 'recover', 'replace_nodes', 'complete_lifecycle'
     )
 
     @util.policy_enforce
@@ -227,6 +228,17 @@ class ClusterController(wsgi.Controller):
         obj = util.parse_request('ClusterRecoverRequest', req, params)
         return self.rpc_client.call(req.context, 'cluster_recover', obj)
 
+    @wsgi.Controller.api_version('1.9')
+    def _do_complete_lifecycle(self, req, cid, data):
+        lifecycle_action_token = data.get('lifecycle_action_token', None)
+
+        params = {'identity': cid,
+                  'lifecycle_action_token': lifecycle_action_token}
+        obj = util.parse_request('ClusterCompleteLifecycleRequest', req,
+                                 params)
+        return self.rpc_client.call(req.context, 'cluster_complete_lifecycle',
+                                    obj)
+
     @util.policy_enforce
     def action(self, req, cluster_id, body=None):
         """Perform specified action on a cluster."""
@@ -300,8 +312,18 @@ class ClusterController(wsgi.Controller):
         return res
 
     @util.policy_enforce
-    def delete(self, req, cluster_id):
-        params = {'identity': cluster_id}
+    def delete(self, req, cluster_id, body=None):
+        if req.params.get('force') is not None:
+            force = util.parse_bool_param(consts.CLUSTER_DELETE_FORCE,
+                                          req.params.get('force'))
+        elif body:
+            force = body.get('force')
+            if force is None:
+                force = False
+        else:
+            force = False
+
+        params = {'identity': cluster_id, 'force': force}
         obj = util.parse_request('ClusterDeleteRequest', req, params)
         res = self.rpc_client.call(req.context, 'cluster_delete', obj)
 
