@@ -119,8 +119,16 @@ class NodeController(wsgi.Controller):
         return result
 
     @util.policy_enforce
-    def delete(self, req, node_id):
-        params = {'identity': node_id}
+    def delete(self, req, node_id, body=None):
+        if body:
+            force = body.get('force')
+        else:
+            force = False
+
+        if force is not None:
+            force = util.parse_bool_param(consts.NODE_DELETE_FORCE, force)
+
+        params = {'identity': node_id, 'force': force}
 
         obj = util.parse_request('NodeDeleteRequest', req, params)
         res = self.rpc_client.call(req.context, 'node_delete', obj)
