@@ -25,6 +25,7 @@ from senlin.api.openstack.v1 import policy_types
 from senlin.api.openstack.v1 import profile_types
 from senlin.api.openstack.v1 import profiles
 from senlin.api.openstack.v1 import receivers
+from senlin.api.openstack.v1 import services
 from senlin.api.openstack.v1 import version
 from senlin.api.openstack.v1 import webhooks
 
@@ -190,6 +191,14 @@ class API(wsgi.Router):
                                action="create",
                                conditions={'method': 'POST'},
                                success=202)
+            sub_mapper.connect("node_adopt",
+                               "/nodes/adopt",
+                               action="adopt",
+                               conditions={'method': 'POST'})
+            sub_mapper.connect("node_adopt_preview",
+                               "/nodes/adopt-preview",
+                               action="adopt_preview",
+                               conditions={'method': 'POST'})
             sub_mapper.connect("node_get",
                                "/nodes/{node_id}",
                                action="get",
@@ -264,6 +273,10 @@ class API(wsgi.Router):
                                "/receivers/{receiver_id}",
                                action="get",
                                conditions={'method': 'GET'})
+            sub_mapper.connect("receiver_update",
+                               "/receivers/{receiver_id}",
+                               action="update",
+                               conditions={'method': 'PATCH'})
             sub_mapper.connect("receiver_delete",
                                "/receivers/{receiver_id}",
                                action="delete",
@@ -306,3 +319,14 @@ class API(wsgi.Router):
                                conditions={'method': 'GET'})
 
         super(API, self).__init__(mapper)
+
+        # Services
+        res = wsgi.Resource(services.ServiceController(conf))
+        with mapper.submapper(controller=res) as sub_mapper:
+
+            sub_mapper.connect("service_index",
+                               "/services",
+                               action="index",
+                               conditions={'method': 'GET'})
+
+        super(API, self).__init__(mapper)
