@@ -15,14 +15,20 @@
 Controller that returns information on the senlin API versions
 """
 
-import json
+from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
 from six.moves import http_client
 import webob.dec
 
+from senlin.api.openstack.v1 import version as v1_controller
+
 
 class Controller(object):
-    '''A controller that produces information on the senlin API versions.'''
+    """A controller that produces information on the senlin API versions."""
+
+    Controllers = {
+        '1.0': v1_controller.VersionController,
+    }
 
     def __init__(self, conf):
         self.conf = conf
@@ -30,18 +36,12 @@ class Controller(object):
     @webob.dec.wsgify
     def __call__(self, req):
         """Respond to a request for all OpenStack API versions."""
-        version_objs = [
-            {
-                "id": "v1.0",
-                "status": "CURRENT",
-                "links": [
-                    {
-                        "rel": "self",
-                        "href": self.get_href(req)
-                    }]
-            }]
 
-        body = json.dumps(dict(versions=version_objs))
+        versions = []
+        for ver, vc in self.Controllers.items():
+            versions.append(vc.version_info())
+
+        body = jsonutils.dumps(dict(versions=versions))
 
         response = webob.Response(request=req,
                                   status=http_client.MULTIPLE_CHOICES,
@@ -50,5 +50,10 @@ class Controller(object):
 
         return response
 
-    def get_href(self, req):
-        return "%s/v1/" % req.host_url
+    def get_controller(self, version):
+        """Return the version specific controller.
+
+        :param version: The version string for mapping.
+        :returns: A version controller instance or ``None``.
+        """
+        return self.Controllers.get(version, None)
