@@ -24,7 +24,7 @@ from oslo_service import systemd
 import six
 
 from senlin.api.common import wsgi
-from senlin.common.i18n import _LI
+from senlin.common import config
 from senlin.common import messaging
 from senlin.common import profiler
 from senlin import objects
@@ -40,6 +40,7 @@ def main():
         logging.register_options(cfg.CONF)
         cfg.CONF(project='senlin', prog='senlin-api',
                  version=version.version_info.version_string())
+        config.set_config_defaults()
         logging.setup(cfg.CONF, 'senlin-api')
         objects.register_all()
         messaging.setup()
@@ -48,7 +49,7 @@ def main():
 
         host = cfg.CONF.senlin_api.bind_host
         port = cfg.CONF.senlin_api.bind_port
-        LOG.info(_LI('Starting Senlin API on %(host)s:%(port)s'),
+        LOG.info('Starting Senlin API on %(host)s:%(port)s',
                  {'host': host, 'port': port})
         profiler.setup('senlin-api', host)
         server = wsgi.Server('senlin-api', cfg.CONF.senlin_api)
