@@ -17,6 +17,7 @@ import socket
 
 from keystoneauth1 import loading as ks_loading
 from oslo_config import cfg
+from oslo_middleware import cors
 from osprofiler import opts as profiler
 
 from senlin.api.common import wsgi
@@ -41,12 +42,15 @@ engine_opts = [
                help=_('Seconds between running periodic tasks.')),
     cfg.IntOpt('periodic_interval_max',
                default=120,
-               help='Maximum seconds between periodic tasks to be called'),
+               help=_('Maximum seconds between periodic tasks to be called.')),
+    cfg.IntOpt('check_interval_max',
+               default=3600,
+               help=_('Maximum seconds between cluster check to be called.')),
     cfg.IntOpt('periodic_fuzzy_delay',
                default=10,
-               help='Range of seconds to randomly delay when starting the'
-                    ' periodic task scheduler to reduce stampeding.'
-                    ' (Disable by setting to 0)'),
+               help=_('Range of seconds to randomly delay when starting the '
+                      'periodic task scheduler to reduce stampeding. '
+                      '(Disable by setting to 0)')),
     cfg.IntOpt('num_engine_workers',
                default=1,
                help=_('Number of senlin-engine processes to fork and run.')),
@@ -86,16 +90,20 @@ engine_opts = [
                 default=False,
                 help=_('Flag to indicate whether to enforce unique names for '
                        'Senlin objects belonging to the same project.')),
+    cfg.IntOpt('service_down_time',
+               default=60,
+               help=_('Maximum time since last check-in for a service to be '
+                      'considered up.')),
 ]
 cfg.CONF.register_opts(engine_opts)
 
 # DEFAULT, host
 rpc_opts = [
-    cfg.StrOpt('host',
-               default=socket.gethostname(),
-               help=_('Name of the engine node. This can be an opaque '
-                      'identifier. It is not necessarily a hostname, FQDN, '
-                      'or IP address.'))
+    cfg.HostAddressOpt('host',
+                       default=socket.gethostname(),
+                       help=_('Name of the engine node. This can be an opaque '
+                              'identifier. It is not necessarily a hostname, '
+                              'FQDN or IP address.'))
 ]
 cfg.CONF.register_opts(rpc_opts)
 
@@ -109,7 +117,7 @@ cfg.CONF.register_opts(cloud_backend_opts)
 # DEFAULT, event dispatchers
 event_opts = [
     cfg.MultiStrOpt("event_dispatchers", default=['database'],
-                    help=_("Event dispatchers to enable"))]
+                    help=_("Event dispatchers to enable."))]
 cfg.CONF.register_opts(event_opts)
 
 # Dispatcher section
@@ -130,7 +138,7 @@ authentication_opts = [
     cfg.StrOpt('auth_url', default='',
                help=_('Complete public identity V3 API endpoint.')),
     cfg.StrOpt('service_username', default='senlin',
-               help=_('Senlin service user name')),
+               help=_('Senlin service user name.')),
     cfg.StrOpt('service_password', default='', secret=True,
                help=_('Password specified for the Senlin service user.')),
     cfg.StrOpt('service_project_name', default='service',
@@ -147,9 +155,11 @@ cfg.CONF.register_opts(authentication_opts, group=authentication_group)
 healthmgr_group = cfg.OptGroup('health_manager')
 healthmgr_opts = [
     cfg.StrOpt('nova_control_exchange', default='nova',
-               help=_("Exchange name for nova notifications")),
+               help=_("Exchange name for nova notifications.")),
     cfg.StrOpt('heat_control_exchange', default='heat',
-               help=_("Exchange name for heat notifications")),
+               help=_("Exchange name for heat notifications.")),
+    cfg.MultiStrOpt("enabled_endpoints", default=['nova', 'heat'],
+                    help=_("Notification endpoints to enable.")),
 ]
 cfg.CONF.register_group(healthmgr_group)
 cfg.CONF.register_opts(healthmgr_opts, group=healthmgr_group)
@@ -176,13 +186,26 @@ receiver_opts = [
                 help=_('The port for notifying and triggering receivers. '
                        'It is useful for case Senlin API service is running '
                        'behind a proxy.')),
-    cfg.PortOpt('max_message_size', default=65535,
-                help=_('The max size(bytes) of message can be posted to '
-                       'receiver queue.'))
+    cfg.IntOpt('max_message_size', default=65535,
+               help=_('The max size(bytes) of message can be posted to '
+                      'receiver queue.'))
 ]
 cfg.CONF.register_group(receiver_group)
 cfg.CONF.register_opts(receiver_opts, group=receiver_group)
 
+# Notification group
+notification_group = cfg.OptGroup('notification')
+notification_opts = [
+    cfg.IntOpt('max_message_size', default=65535,
+               help=_('The max size(bytes) of message can be posted to '
+                      'notification queue.')),
+    cfg.IntOpt('ttl', default=300,
+               help=_('The ttl in seconds of a message posted to '
+                      'notification queue.'))
+]
+cfg.CONF.register_group(notification_group)
+cfg.CONF.register_opts(notification_opts, group=notification_group)
+
 # Zaqar group
 zaqar_group = cfg.OptGroup(
     'zaqar', title='Zaqar Options',
@@ -233,3 +256,24 @@ def list_opts():
     yield receiver_group.name, receiver_opts
     yield zaqar_group.name, zaqar_opts
     yield profiler.list_opts()[0]
+
+
+def set_config_defaults():
+    """Update default configuration options for oslo.middleware."""
+    cors.set_defaults(
+        allow_headers=['X-Auth-Token',
+                       'X-Identity-Status',
+                       'X-Roles',
+                       'X-Service-Catalog',
+                       'X-User-Id',
+                       'X-Tenant-Id',
+                       'X-OpenStack-Request-ID'],
+        expose_headers=['X-Auth-Token',
+                        'X-Subject-Token',
+                        'X-Service-Token',
+                        'X-OpenStack-Request-ID'],
+        allow_methods=['GET',
+                       'PUT',
+                       'POST',
+                       'DELETE',
+                       'PATCH'])
