@@ -41,10 +41,11 @@ SUPPORT_STATUSES = (
 )
 
 ACTION_CAUSES = (
-    CAUSE_RPC, CAUSE_DERIVED,
+    CAUSE_RPC, CAUSE_DERIVED, CAUSE_DERIVED_LCH
 ) = (
     'RPC Request',
     'Derived Action',
+    'Derived Action with Lifecycle Hook'
 )
 
 CLUSTER_ACTION_NAMES = (
@@ -93,14 +94,20 @@ CLUSTER_ATTRS = (
     CLUSTER_DOMAIN, CLUSTER_PROJECT, CLUSTER_USER,
     CLUSTER_INIT_AT, CLUSTER_CREATED_AT, CLUSTER_UPDATED_AT,
     CLUSTER_STATUS, CLUSTER_STATUS_REASON, CLUSTER_TIMEOUT,
-    CLUSTER_METADATA,
+    CLUSTER_METADATA, CLUSTER_CONFIG,
 ) = (
     'name', 'profile_id', 'desired_capacity',
     'min_size', 'max_size', 'id',
     'domain', 'project', 'user',
     'init_at', 'created_at', 'updated_at',
     'status', 'status_reason', 'timeout',
-    'metadata',
+    'metadata', 'config',
+)
+
+CLUSTER_PARAMS = (
+    CLUSTER_PROFILE_ONLY, CLUSTER_DELETE_FORCE
+) = (
+    'profile_only', 'force',
 )
 
 CLUSTER_STATUSES = (
@@ -141,6 +148,12 @@ NODE_SORT_KEYS = [
     NODE_INIT_AT, NODE_CREATED_AT, NODE_UPDATED_AT,
 ]
 
+NODE_PARAMS = (
+    NODE_DELETE_FORCE,
+) = (
+    'force',
+)
+
 PROFILE_ATTRS = (
     PROFILE_ID, PROFILE_NAME, PROFILE_TYPE,
     PROFILE_CREATED_AT, PROFILE_UPDATED_AT,
@@ -245,16 +258,17 @@ CLUSTER_DEFAULT_VALUE = (
 # senlin.engine.actions.base module.
 ACTION_STATUSES = (
     ACTION_INIT, ACTION_WAITING, ACTION_READY, ACTION_RUNNING,
-    ACTION_SUCCEEDED, ACTION_FAILED, ACTION_CANCELLED
+    ACTION_SUCCEEDED, ACTION_FAILED, ACTION_CANCELLED,
+    ACTION_WAITING_LIFECYCLE_COMPLETION,
 ) = (
     'INIT', 'WAITING', 'READY', 'RUNNING',
-    'SUCCEEDED', 'FAILED', 'CANCELLED',
+    'SUCCEEDED', 'FAILED', 'CANCELLED', 'WAITING_LIFECYCLE_COMPLETION'
 )
 
 EVENT_LEVELS = {
     'CRITICAL': logging.CRITICAL,
     'ERROR': logging.ERROR,
-    'WARNING': logging.WARNING,
+    'WARN': logging.WARNING,
     'INFO': logging.INFO,
     'DEBUG': logging.DEBUG,
 }
@@ -285,3 +299,9 @@ NOTIFICATION_PHASES = (
 ) = (
     'start', 'end', 'error',
 )
+
+LIFECYCLE_TRANSITION_TYPE = (
+    LIFECYCLE_NODE_TERMINATION,
+) = (
+    'termination',
+)
