@@ -256,7 +256,9 @@ class String(PropertySchema):
 
     def to_schema_type(self, value):
         try:
-            return str(value)
+            if isinstance(value, six.string_types):
+                return value
+            return str(value) if value is not None else None
         except Exception:
             raise
 
