@@ -368,6 +368,10 @@ def action_mark_succeeded(context, action_id, timestamp):
     return IMPL.action_mark_succeeded(context, action_id, timestamp)
 
 
+def action_mark_ready(context, action_id, timestamp):
+    return IMPL.action_mark_ready(context, action_id, timestamp)
+
+
 def action_mark_failed(context, action_id, timestamp, reason=None):
     return IMPL.action_mark_failed(context, action_id, timestamp, reason)
 
@@ -384,8 +388,12 @@ def action_acquire_random_ready(context, owner, timestamp):
     return IMPL.action_acquire_random_ready(context, owner, timestamp)
 
 
-def action_abandon(context, action_id):
-    return IMPL.action_abandon(context, action_id)
+def action_acquire_first_ready(context, owner, timestamp):
+    return IMPL.action_acquire_first_ready(context, owner, timestamp)
+
+
+def action_abandon(context, action_id, values=None):
+    return IMPL.action_abandon(context, action_id, values)
 
 
 def action_lock_check(context, action_id, owner=None):
@@ -486,6 +494,10 @@ def registry_get(context, cluster_id):
     return IMPL.registry_get(context, cluster_id)
 
 
+def registry_get_by_param(context, params):
+    return IMPL.registry_get_by_param(context, params)
+
+
 def db_sync(engine, version=None):
     """Migrate the database to `version` or the most recent version."""
     return IMPL.db_sync(engine, version=version)
