@@ -34,7 +34,6 @@ from sqlalchemy.sql.expression import func
 
 from senlin.common import consts
 from senlin.common import exception
-from senlin.common.i18n import _
 from senlin.db.sqlalchemy import migration
 from senlin.db.sqlalchemy import models
 from senlin.db.sqlalchemy import utils
@@ -85,7 +84,7 @@ def query_by_short_id(context, model, short_id, project_safe=True):
     q = q.filter(model.id.like('%s%%' % short_id))
 
     if project_safe:
-        q = q.filter_by(project=context.project)
+        q = q.filter_by(project=context.project_id)
 
     if q.count() == 1:
         return q.first()
@@ -100,7 +99,7 @@ def query_by_name(context, model, name, project_safe=True):
     q = q.filter_by(name=name)
 
     if project_safe:
-        q = q.filter_by(project=context.project)
+        q = q.filter_by(project=context.project_id)
 
     if q.count() == 1:
         return q.first()
@@ -126,7 +125,7 @@ def cluster_get(context, cluster_id, project_safe=True):
         return None
 
     if project_safe:
-        if context.project != cluster.project:
+        if context.project_id != cluster.project:
             return None
     return cluster
 
@@ -145,7 +144,7 @@ def _query_cluster_get_all(context, project_safe=True):
     query = model_query(context, models.Cluster)
 
     if project_safe:
-        query = query.filter_by(project=context.project)
+        query = query.filter_by(project=context.project_id)
     return query
 
 
@@ -230,7 +229,7 @@ def node_get(context, node_id, project_safe=True):
         return None
 
     if project_safe:
-        if context.project != node.project:
+        if context.project_id != node.project:
             return None
 
     return node
@@ -252,7 +251,7 @@ def _query_node_get_all(context, project_safe=True, cluster_id=None):
         query = query.filter_by(cluster_id=cluster_id)
 
     if project_safe:
-        query = query.filter_by(project=context.project)
+        query = query.filter_by(project=context.project_id)
 
     return query
 
@@ -299,7 +298,7 @@ def node_count_by_cluster(context, cluster_id, **kwargs):
     query = query.filter_by(cluster_id=cluster_id)
     query = query.filter_by(**kwargs)
     if project_safe:
-        query = query.filter_by(project=context.project)
+        query = query.filter_by(project=context.project_id)
 
     return query.count()
 
@@ -324,7 +323,7 @@ def node_update(context, node_id, values):
                 if values['status'] == 'ERROR':
                     cluster.status = 'WARNING'
                 if 'status_reason' in values:
-                    cluster.status_reason = _('Node %(node)s: %(reason)s') % {
+                    cluster.status_reason = 'Node %(node)s: %(reason)s' % {
                         'node': node.name, 'reason': values['status_reason']}
                 cluster.save(session)
 
@@ -422,7 +421,8 @@ def cluster_lock_acquire(cluster_id, action_id, scope):
     :return: A list of action IDs that currently works on the cluster.
     '''
     with session_for_write() as session:
-        lock = session.query(models.ClusterLock).get(cluster_id)
+        query = session.query(models.ClusterLock).with_lockmode('update')
+        lock = query.get(cluster_id)
         if lock is not None:
             if scope == 1 and lock.semaphore > 0:
                 if action_id not in lock.action_ids:
@@ -539,7 +539,7 @@ def policy_get(context, policy_id, project_safe=True):
         return None
 
     if project_safe:
-        if context.project != policy.project:
+        if context.project_id != policy.project:
             return None
 
     return policy
@@ -560,7 +560,7 @@ def policy_get_all(context, limit=None, marker=None, sort=None, filters=None,
     query = model_query(context, models.Policy)
 
     if project_safe:
-        query = query.filter_by(project=context.project)
+        query = query.filter_by(project=context.project_id)
 
     if filters:
         query = utils.exact_filter(query, models.Policy, filters)
@@ -767,7 +767,7 @@ def profile_get(context, profile_id, project_safe=True):
         return None
 
     if project_safe:
-        if context.project != profile.project:
+        if context.project_id != profile.project:
             return None
 
     return profile
@@ -788,7 +788,7 @@ def profile_get_all(context, limit=None, marker=None, sort=None, filters=None,
     query = model_query(context, models.Profile)
 
     if project_safe:
-        query = query.filter_by(project=context.project)
+        query = query.filter_by(project=context.project_id)
 
     if filters:
         query = utils.exact_filter(query, models.Profile, filters)
@@ -880,7 +880,7 @@ def event_create(context, values):
 def event_get(context, event_id, project_safe=True):
     event = model_query(context, models.Event).get(event_id)
     if project_safe and event is not None:
-        if event.project != context.project:
+        if event.project != context.project_id:
             return None
 
     return event
@@ -907,7 +907,7 @@ def event_get_all(context, limit=None, marker=None, sort=None, filters=None,
                   project_safe=True):
     query = model_query(context, models.Event)
     if project_safe:
-        query = query.filter_by(project=context.project)
+        query = query.filter_by(project=context.project_id)
 
     return _event_filter_paginate_query(context, query, filters=filters,
                                         limit=limit, marker=marker, sort=sort)
@@ -917,7 +917,7 @@ def event_count_by_cluster(context, cluster_id, project_safe=True):
     query = model_query(context, models.Event)
 
     if project_safe:
-        query = query.filter_by(project=context.project)
+        query = query.filter_by(project=context.project_id)
     count = query.filter_by(cluster_id=cluster_id).count()
 
     return count
@@ -929,7 +929,7 @@ def event_get_all_by_cluster(context, cluster_id, limit=None, marker=None,
     query = query.filter_by(cluster_id=cluster_id)
 
     if project_safe:
-        query = query.filter_by(project=context.project)
+        query = query.filter_by(project=context.project_id)
 
     return _event_filter_paginate_query(context, query, filters=filters,
                                         limit=limit, marker=marker, sort=sort)
@@ -940,7 +940,7 @@ def event_prune(context, cluster_id, project_safe=True):
         query = session.query(models.Event).with_for_update()
         query = query.filter_by(cluster_id=cluster_id)
         if project_safe:
-            query = query.filter_by(project=context.project)
+            query = query.filter_by(project=context.project_id)
 
         return query.delete(synchronize_session='fetch')
 
@@ -989,7 +989,7 @@ def action_get(context, action_id, project_safe=True, refresh=False):
             return None
 
         if project_safe:
-            if action.project != context.project:
+            if action.project != context.project_id:
                 return None
 
         session.refresh(action)
@@ -1017,7 +1017,7 @@ def action_get_all(context, filters=None, limit=None, marker=None, sort=None,
 
     query = model_query(context, models.Action)
     if project_safe:
-        query = query.filter_by(project=context.project)
+        query = query.filter_by(project=context.project_id)
 
     if filters:
         query = utils.exact_filter(query, models.Action, filters)
@@ -1039,7 +1039,7 @@ def action_check_status(context, action_id, timestamp):
         action = session.query(models.Action).get(action_id)
         if action.status == consts.ACTION_WAITING:
             action.status = consts.ACTION_READY
-            action.status_reason = _('All depended actions completed.')
+            action.status_reason = 'All depended actions completed.'
             action.end_time = timestamp
             action.save(session)
 
@@ -1065,7 +1065,7 @@ def dependency_get_dependents(context, action_id):
 def dependency_add(context, depended, dependent):
     if isinstance(depended, list) and isinstance(dependent, list):
         raise exception.NotSupport(
-            _('Multiple dependencies between lists not support'))
+            'Multiple dependencies between lists not support')
 
     with session_for_write() as session:
         if isinstance(depended, list):   # e.g. D depends on A,B,C
@@ -1075,7 +1075,7 @@ def dependency_add(context, depended, dependent):
 
             query = session.query(models.Action).filter_by(id=dependent)
             query.update({'status': consts.ACTION_WAITING,
-                          'status_reason': _('Waiting for depended actions.')},
+                          'status_reason': 'Waiting for depended actions.'},
                          synchronize_session='fetch')
             return
 
@@ -1093,7 +1093,7 @@ def dependency_add(context, depended, dependent):
         q = session.query(models.Action).filter(
             models.Action.id.in_(dependents))
         q.update({'status': consts.ACTION_WAITING,
-                  'status_reason': _('Waiting for depended actions.')},
+                  'status_reason': 'Waiting for depended actions.'},
                  synchronize_session='fetch')
 
 
@@ -1106,7 +1106,7 @@ def action_mark_succeeded(context, action_id, timestamp):
         values = {
             'owner': None,
             'status': consts.ACTION_SUCCEEDED,
-            'status_reason': _('Action completed successfully.'),
+            'status_reason': 'Action completed successfully.',
             'end_time': timestamp,
         }
         query.update(values, synchronize_session=False)
@@ -1116,6 +1116,21 @@ def action_mark_succeeded(context, action_id, timestamp):
         subquery.delete(synchronize_session='fetch')
 
 
+@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
+                           retry_interval=0.5, inc_retry_interval=True)
+def action_mark_ready(context, action_id, timestamp):
+    with session_for_write() as session:
+
+        query = session.query(models.Action).filter_by(id=action_id)
+        values = {
+            'owner': None,
+            'status': consts.ACTION_READY,
+            'status_reason': 'Lifecycle timeout.',
+            'end_time': timestamp,
+        }
+        query.update(values, synchronize_session=False)
+
+
 def _mark_failed(session, action_id, timestamp, reason=None):
     # mark myself as failed
     query = session.query(models.Action).filter_by(id=action_id)
@@ -1123,7 +1138,7 @@ def _mark_failed(session, action_id, timestamp, reason=None):
         'owner': None,
         'status': consts.ACTION_FAILED,
         'status_reason': (six.text_type(reason) if reason else
-                          _('Action execution failed')),
+                          'Action execution failed'),
         'end_time': timestamp,
     }
     query.update(values, synchronize_session=False)
@@ -1150,7 +1165,7 @@ def _mark_cancelled(session, action_id, timestamp, reason=None):
         'owner': None,
         'status': consts.ACTION_CANCELLED,
         'status_reason': (six.text_type(reason) if reason else
-                          _('Action execution failed')),
+                          'Action execution failed'),
         'end_time': timestamp,
     }
     query.update(values, synchronize_session=False)
@@ -1184,14 +1199,13 @@ def action_acquire(context, action_id, owner, timestamp):
             return None
 
         if action.status != consts.ACTION_READY:
-            msg = _('The action is not in an executable status: '
-                    '%s') % action.status
+            msg = 'The action is not executable: %s' % action.status
             LOG.warning(msg)
             return None
         action.owner = owner
         action.start_time = timestamp
         action.status = consts.ACTION_RUNNING
-        action.status_reason = _('The action is being processed.')
+        action.status_reason = 'The action is being processed.'
         action.save(session)
 
         return action
@@ -1211,13 +1225,38 @@ def action_acquire_random_ready(context, owner, timestamp):
             action.owner = owner
             action.start_time = timestamp
             action.status = consts.ACTION_RUNNING
-            action.status_reason = _('The action is being processed.')
+            action.status_reason = 'The action is being processed.'
             action.save(session)
 
             return action
 
 
-def action_abandon(context, action_id):
+def _action_acquire_ready(session, owner, timestamp, order=None):
+    action = session.query(models.Action).\
+        filter_by(status=consts.ACTION_READY).\
+        filter_by(owner=None).\
+        order_by(order or func.random()).\
+        with_for_update().first()
+
+    if action:
+        action.owner = owner
+        action.start_time = timestamp
+        action.status = consts.ACTION_RUNNING
+        action.status_reason = 'The action is being processed.'
+        action.save(session)
+
+    return action
+
+
+@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
+                           retry_interval=0.5, inc_retry_interval=True)
+def action_acquire_first_ready(context, owner, timestamp):
+    with session_for_write() as session:
+        return _action_acquire_ready(session, owner, timestamp,
+                                     consts.ACTION_CREATED_AT)
+
+
+def action_abandon(context, action_id, values=None):
     '''Abandon an action for other workers to execute again.
 
     This API is always called with the action locked by the current
@@ -1229,7 +1268,9 @@ def action_abandon(context, action_id):
         action.owner = None
         action.start_time = None
         action.status = consts.ACTION_READY
-        action.status_reason = _('The action was abandoned.')
+        action.status_reason = 'The action was abandoned.'
+        if values:
+            action.update(values)
         action.save(session)
         return action
 
@@ -1279,8 +1320,7 @@ def action_delete_by_target(context, target, action=None,
                             action_excluded=None, status=None,
                             project_safe=True):
     if action and action_excluded:
-        msg = _("action and action_excluded cannot be specified "
-                "both.")
+        msg = "action and action_excluded cannot be both specified."
         LOG.warning(msg)
         return None
 
@@ -1289,7 +1329,7 @@ def action_delete_by_target(context, target, action=None,
             filter_by(target=target)
 
         if project_safe:
-            q = q.filter_by(project=context.project)
+            q = q.filter_by(project=context.project_id)
 
         if action:
             q = q.filter(models.Action.action.in_(action))
@@ -1315,7 +1355,7 @@ def receiver_get(context, receiver_id, project_safe=True):
         return None
 
     if project_safe:
-        if context.project != receiver.project:
+        if context.project_id != receiver.project:
             return None
 
     return receiver
@@ -1325,7 +1365,7 @@ def receiver_get_all(context, limit=None, marker=None, filters=None, sort=None,
                      project_safe=True):
     query = model_query(context, models.Receiver)
     if project_safe:
-        query = query.filter_by(project=context.project)
+        query = query.filter_by(project=context.project_id)
 
     if filters:
         query = utils.exact_filter(query, models.Receiver, filters)
@@ -1407,6 +1447,54 @@ def service_get_all():
         return session.query(models.Service).all()
 
 
+def _mark_engine_failed(session, action_id, timestamp, reason=None):
+    query = session.query(models.ActionDependency)
+    # process cluster actions
+    d_query = query.filter_by(dependent=action_id)
+    dependents = [d.depended for d in d_query.all()]
+    if dependents:
+        for d in dependents:
+            _mark_engine_failed(session, d, timestamp, reason)
+    else:
+        depended = query.filter_by(depended=action_id)
+        depended.delete(synchronize_session=False)
+
+    # TODO(anyone): this will mark all depended actions' status to 'FAILED'
+    # even the action belong to other engines and the action is running
+    # mark myself as failed
+    action = session.query(models.Action).filter_by(id=action_id).first()
+    values = {
+        'owner': None,
+        'status': consts.ACTION_FAILED,
+        'status_reason': (six.text_type(reason) if reason else
+                          'Action execution failed'),
+        'end_time': timestamp,
+    }
+    action.update(values)
+    action.save(session)
+
+
+@oslo_db_api.wrap_db_retry(max_retries=3, retry_on_deadlock=True,
+                           retry_interval=0.5, inc_retry_interval=True)
+def dummy_gc(engine_id):
+    with session_for_write() as session:
+        q_actions = session.query(models.Action).filter_by(owner=engine_id)
+        timestamp = time.time()
+        for action in q_actions.all():
+            _mark_engine_failed(session, action.id, timestamp,
+                                reason='Engine failure')
+            # Release all node locks
+            query = session.query(models.NodeLock).\
+                filter_by(action_id=action.id)
+            query.delete(synchronize_session=False)
+
+            # Release all cluster locks
+            for clock in session.query(models.ClusterLock).all():
+                res = _release_cluster_lock(session, clock, action.id, -1)
+                if not res:
+                    _release_cluster_lock(session, clock, action.id, 1)
+
+
 def gc_by_engine(engine_id):
     # Get all actions locked by an engine
     with session_for_write() as session:
@@ -1444,7 +1532,7 @@ def registry_create(context, cluster_id, check_type, interval, params,
 
 def registry_update(context, cluster_id, values):
     with session_for_write() as session:
-        query = session.query(models.HealthRegistry)
+        query = session.query(models.HealthRegistry).with_lockmode('update')
         registry = query.filter_by(cluster_id=cluster_id).first()
         if registry:
             registry.update(values)
@@ -1455,7 +1543,7 @@ def registry_claim(context, engine_id):
     with session_for_write() as session:
         engines = session.query(models.Service).all()
         svc_ids = [e.id for e in engines if not utils.is_service_dead(e)]
-        q_reg = session.query(models.HealthRegistry)
+        q_reg = session.query(models.HealthRegistry).with_lockmode('update')
         if svc_ids:
             q_reg = q_reg.filter(
                 models.HealthRegistry.engine_id.notin_(svc_ids))
@@ -1483,6 +1571,12 @@ def registry_get(context, cluster_id):
         return registry
 
 
+def registry_get_by_param(context, params):
+    query = model_query(context, models.HealthRegistry)
+    obj = utils.exact_filter(query, models.HealthRegistry, params).first()
+    return obj
+
+
 # Utils
 def db_sync(engine, version=None):
     """Migrate the database to `version` or the most recent version."""
