@@ -39,8 +39,8 @@ class SenlinDriver(object):
         self.compute = backend.compute
         self.loadbalancing = backend.loadbalancing
         self.network = backend.network
+        self.octavia = backend.octavia
         self.orchestration = backend.orchestration
-        self.telemetry = backend.telemetry
         self.identity = backend.identity
         self.message = backend.message
         self.workflow = backend.workflow
