@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import eventlet
 import six
 import time
 
@@ -38,8 +39,10 @@ class Action(object):
 
     RETURNS = (
         RES_OK, RES_ERROR, RES_RETRY, RES_CANCEL, RES_TIMEOUT,
+        RES_LIFECYCLE_COMPLETE, RES_LIFECYCLE_HOOK_TIMEOUT,
     ) = (
-        'OK', 'ERROR', 'RETRY', 'CANCEL', 'TIMEOUT',
+        'OK', 'ERROR', 'RETRY', 'CANCEL', 'TIMEOUT', 'LIFECYCLE_COMPLETE',
+        'LIFECYCLE_HOOK_TIMEOUT'
     )
 
     # Action status definitions:
@@ -52,10 +55,10 @@ class Action(object):
     #  CANCELLED: Action cancelled because worker thread was cancelled.
     STATUSES = (
         INIT, WAITING, READY, RUNNING, SUSPENDED,
-        SUCCEEDED, FAILED, CANCELLED
+        SUCCEEDED, FAILED, CANCELLED, WAITING_LIFECYCLE_COMPLETION
     ) = (
         'INIT', 'WAITING', 'READY', 'RUNNING', 'SUSPENDED',
-        'SUCCEEDED', 'FAILED', 'CANCELLED',
+        'SUCCEEDED', 'FAILED', 'CANCELLED', 'WAITING_LIFECYCLE_COMPLETION'
     )
 
     # Signal commands
@@ -65,7 +68,7 @@ class Action(object):
         'CANCEL', 'SUSPEND', 'RESUME',
     )
 
-    def __new__(cls, target, action, context, **kwargs):
+    def __new__(cls, target, action, ctx, **kwargs):
         if (cls != Action):
             return super(Action, cls).__new__(cls)
 
@@ -82,17 +85,17 @@ class Action(object):
 
         return super(Action, cls).__new__(ActionClass)
 
-    def __init__(self, target, action, context, **kwargs):
+    def __init__(self, target, action, ctx, **kwargs):
         # context will be persisted into database so that any worker thread
         # can pick the action up and execute it on behalf of the initiator
 
         self.id = kwargs.get('id', None)
         self.name = kwargs.get('name', '')
 
-        self.context = context
-        self.user = context.user
-        self.project = context.project
-        self.domain = context.domain
+        self.context = ctx
+        self.user = ctx.user_id
+        self.project = ctx.project_id
+        self.domain = ctx.domain_id
 
         self.action = action
         self.target = target
@@ -135,10 +138,10 @@ class Action(object):
 
         self.data = kwargs.get('data', {})
 
-    def store(self, context):
+    def store(self, ctx):
         """Store the action record into database table.
 
-        :param context: An instance of the request context.
+        :param ctx: An instance of the request context.
         :return: The ID of the stored object.
         """
 
@@ -170,11 +173,11 @@ class Action(object):
         if self.id:
             self.updated_at = timestamp
             values['updated_at'] = timestamp
-            ao.Action.update(context, self.id, values)
+            ao.Action.update(ctx, self.id, values)
         else:
             self.created_at = timestamp
             values['created_at'] = timestamp
-            action = ao.Action.create(context, values)
+            action = ao.Action.create(ctx, values)
             self.id = action.id
 
         return self.id
@@ -186,7 +189,7 @@ class Action(object):
         :param obj: a DB action object that contains all fields.
         :return: An `Action` object deserialized from the DB action object.
         """
-        context = req_context.RequestContext.from_dict(obj.context)
+        ctx = req_context.RequestContext.from_dict(obj.context)
         kwargs = {
             'id': obj.id,
             'name': obj.name,
@@ -205,19 +208,19 @@ class Action(object):
             'data': obj.data,
         }
 
-        return cls(obj.target, obj.action, context, **kwargs)
+        return cls(obj.target, obj.action, ctx, **kwargs)
 
     @classmethod
-    def load(cls, context, action_id=None, db_action=None, project_safe=True):
+    def load(cls, ctx, action_id=None, db_action=None, project_safe=True):
         """Retrieve an action from database.
 
-        :param context: Instance of request context.
+        :param ctx: Instance of request context.
         :param action_id: An UUID for the action to deserialize.
         :param db_action: An action object for the action to deserialize.
         :return: A `Action` object instance.
         """
         if db_action is None:
-            db_action = ao.Action.get(context, action_id,
+            db_action = ao.Action.get(ctx, action_id,
                                       project_safe=project_safe)
             if db_action is None:
                 raise exception.ResourceNotFound(type='action', id=action_id)
@@ -225,36 +228,36 @@ class Action(object):
         return cls._from_object(db_action)
 
     @classmethod
-    def create(cls, context, target, action, **kwargs):
+    def create(cls, ctx, target, action, **kwargs):
         """Create an action object.
 
-        :param context: The requesting context.
+        :param ctx: The requesting context.
         :param target: The ID of the target cluster/node.
         :param action: Name of the action.
         :param dict kwargs: Other keyword arguments for the action.
         :return: ID of the action created.
         """
         params = {
-            'user': context.user,
-            'project': context.project,
-            'domain': context.domain,
-            'is_admin': context.is_admin,
-            'request_id': context.request_id,
-            'trusts': context.trusts,
+            'user_id': ctx.user_id,
+            'project_id': ctx.project_id,
+            'domain_id': ctx.domain_id,
+            'is_admin': ctx.is_admin,
+            'request_id': ctx.request_id,
+            'trusts': ctx.trusts,
         }
-        ctx = req_context.RequestContext.from_dict(params)
-        obj = cls(target, action, ctx, **kwargs)
-        return obj.store(context)
+        c = req_context.RequestContext.from_dict(params)
+        obj = cls(target, action, c, **kwargs)
+        return obj.store(ctx)
 
     @classmethod
-    def delete(cls, context, action_id):
+    def delete(cls, ctx, action_id):
         """Delete an action from database.
 
-        :param context: An instance of the request context.
+        :param ctx: An instance of the request context.
         :param action_id: The UUID of the target action to be deleted.
         :return: Nothing.
         """
-        ao.Action.delete(context, action_id)
+        ao.Action.delete(ctx, action_id)
 
     def signal(self, cmd):
         """Send a signal to the action.
@@ -314,12 +317,28 @@ class Action(object):
             status = self.CANCELLED
             ao.Action.mark_cancelled(self.context, self.id, timestamp)
 
+        elif result == self.RES_LIFECYCLE_COMPLETE:
+            status = self.SUCCEEDED
+            ao.Action.mark_ready(self.context, self.id, timestamp)
+
         else:  # result == self.RES_RETRY:
-            status = self.READY
+            retries = self.data.get('retries', 0)
             # Action failed at the moment, but can be retried
-            # We abandon it and then notify other dispatchers to execute it
-            ao.Action.abandon(self.context, self.id)
-            dispatcher.start_action()
+            # retries time is configurable
+            if retries < cfg.CONF.lock_retry_times:
+                status = self.READY
+                retries += 1
+
+                self.data.update({'retries': retries})
+                ao.Action.abandon(self.context, self.id, {'data': self.data})
+                # sleep for a while
+                eventlet.sleep(cfg.CONF.lock_retry_interval)
+                dispatcher.start_action(self.id)
+            else:
+                status = self.RES_ERROR
+                reason = reason or _('Exceeded maximum number of retries (%d)'
+                                     ) % cfg.CONF.lock_retry_times
+                ao.Action.mark_failed(self.context, self.id, timestamp, reason)
 
         if status == self.SUCCEEDED:
             EVENT.info(self, consts.PHASE_END, reason or 'SUCCEEDED')
@@ -337,9 +356,11 @@ class Action(object):
         self.status = status
         return status
 
-    def is_timeout(self):
+    def is_timeout(self, timeout=None):
+        if timeout is None:
+            timeout = self.timeout
         time_elapse = wallclock() - self.start_time
-        return time_elapse > self.timeout
+        return time_elapse > timeout
 
     def _check_signal(self):
         # Check timeout first, if true, return timeout message
@@ -460,16 +481,16 @@ class Action(object):
         return action_dict
 
 
-def ActionProc(context, action_id):
+def ActionProc(ctx, action_id):
     '''Action process.'''
 
     # Step 1: materialize the action object
-    action = Action.load(context, action_id=action_id, project_safe=False)
+    action = Action.load(ctx, action_id=action_id, project_safe=False)
     if action is None:
         LOG.error('Action "%s" could not be found.', action_id)
         return False
 
-    EVENT.info(action, consts.PHASE_START)
+    EVENT.info(action, consts.PHASE_START, action_id[:8])
 
     reason = 'Action completed'
     success = True
