@@ -18,6 +18,7 @@ from oslo_utils import timeutils
 from osprofiler import profiler
 
 from senlin.common import consts
+from senlin.common import exception
 from senlin.common.i18n import _
 from senlin.common import scaleutils
 from senlin.common import utils
@@ -25,6 +26,7 @@ from senlin.engine.actions import base
 from senlin.engine import cluster as cluster_mod
 from senlin.engine import dispatcher
 from senlin.engine import node as node_mod
+from senlin.engine.notifications import message as msg
 from senlin.engine import scheduler
 from senlin.engine import senlin_lock
 from senlin.objects import action as ao
@@ -51,6 +53,7 @@ class ClusterAction(base.Action):
 
         try:
             self.entity = cluster_mod.Cluster.load(self.context, self.target)
+            self.timeout = self.entity.timeout
         except Exception:
             self.entity = None
 
@@ -58,7 +61,7 @@ class ClusterAction(base.Action):
         if period:
             eventlet.sleep(period)
 
-    def _wait_for_dependents(self):
+    def _wait_for_dependents(self, lifecycle_hook_timeout=None):
         """Wait for dependent actions to complete.
 
         :returns: A tuple containing the result and the corresponding reason.
@@ -67,7 +70,7 @@ class ClusterAction(base.Action):
         reason = ''
         while status != self.READY:
             if status == self.FAILED:
-                reason = _('%(action)s [%(id)s] failed') % {
+                reason = ('%(action)s [%(id)s] failed') % {
                     'action': self.action, 'id': self.id[:8]}
                 LOG.debug(reason)
                 return self.RES_ERROR, reason
@@ -75,18 +78,27 @@ class ClusterAction(base.Action):
             if self.is_cancelled():
                 # During this period, if cancel request comes, cancel this
                 # operation immediately, then release the cluster lock
-                reason = _('%(action)s [%(id)s] cancelled') % {
+                reason = ('%(action)s [%(id)s] cancelled') % {
                     'action': self.action, 'id': self.id[:8]}
                 LOG.debug(reason)
                 return self.RES_CANCEL, reason
 
             if self.is_timeout():
                 # Action timeout, return
-                reason = _('%(action)s [%(id)s] timeout') % {
+                reason = ('%(action)s [%(id)s] timeout') % {
                     'action': self.action, 'id': self.id[:8]}
                 LOG.debug(reason)
                 return self.RES_TIMEOUT, reason
 
+            if (lifecycle_hook_timeout is not None and
+                    self.is_timeout(lifecycle_hook_timeout)):
+                # if lifecycle hook timeout is specified and Lifecycle hook
+                # timeout is reached, return
+                reason = _('%(action)s [%(id)s] lifecycle hook timeout') % {
+                    'action': self.action, 'id': self.id[:8]}
+                LOG.debug(reason)
+                return self.RES_LIFECYCLE_HOOK_TIMEOUT, reason
+
             # Continue waiting (with reschedule)
             scheduler.reschedule(self.id, 3)
             status = self.get_status()
@@ -121,7 +133,8 @@ class ClusterAction(base.Action):
                 # We assume placement is a list
                 kwargs['data'] = {'placement': placement['placements'][m]}
 
-            name = 'node-%s-%003d' % (self.entity.id[:8], index)
+            name_format = self.entity.config.get("node.name.format", "")
+            name = utils.format_node_name(name_format, self.entity, index)
             node = node_mod.Node(name, self.entity.profile_id,
                                  self.entity.id, context=self.context,
                                  **kwargs)
@@ -155,7 +168,7 @@ class ClusterAction(base.Action):
             for node in nodes:
                 self.entity.add_node(node)
         else:
-            reason = _('Failed in creating nodes.')
+            reason = 'Failed in creating nodes.'
 
         return res, reason
 
@@ -168,7 +181,7 @@ class ClusterAction(base.Action):
         res = self.entity.do_create(self.context)
 
         if not res:
-            reason = _('Cluster creation failed.')
+            reason = 'Cluster creation failed.'
             self.entity.set_status(self.context, consts.CS_ERROR, reason)
             return self.RES_ERROR, reason
 
@@ -176,7 +189,7 @@ class ClusterAction(base.Action):
 
         params = {}
         if result == self.RES_OK:
-            reason = _('Cluster creation succeeded.')
+            reason = 'Cluster creation succeeded.'
             params = {'created_at': timeutils.utcnow(True)}
         self.entity.eval_status(self.context, consts.CLUSTER_CREATE, **params)
 
@@ -184,7 +197,7 @@ class ClusterAction(base.Action):
 
     def _update_nodes(self, profile_id, nodes_obj):
         # Get batching policy data if any
-        fmt = _("Updating cluster '%(cluster)s': profile='%(profile)s'.")
+        fmt = "Updating cluster '%(cluster)s': profile='%(profile)s'."
         LOG.info(fmt, {'cluster': self.entity.id, 'profile': profile_id})
         plan = []
 
@@ -230,7 +243,7 @@ class ClusterAction(base.Action):
                 if result != self.RES_OK:
                     self.entity.eval_status(self.context,
                                             consts.CLUSTER_UPDATE)
-                    return result, _('Failed in updating nodes.')
+                    return result, 'Failed in updating nodes.'
                 # pause time
                 if pause_time != 0:
                     self._sleep(pause_time)
@@ -249,7 +262,7 @@ class ClusterAction(base.Action):
         """
         res = self.entity.do_update(self.context)
         if not res:
-            reason = _('Cluster update failed.')
+            reason = 'Cluster update failed.'
             self.entity.set_status(self.context, consts.CS_ERROR, reason)
             return self.RES_ERROR, reason
 
@@ -270,7 +283,7 @@ class ClusterAction(base.Action):
             self.entity.timeout = timeout
         self.entity.store(self.context)
 
-        reason = _('Cluster update completed.')
+        reason = 'Cluster update completed.'
         if profile_id is None:
             self.entity.eval_status(self.context, consts.CLUSTER_UPDATE,
                                     updated_at=timeutils.utcnow(True))
@@ -288,6 +301,13 @@ class ClusterAction(base.Action):
         result, reason = self._update_nodes(profile_id, self.entity.nodes)
         return result, reason
 
+    def _handle_lifecycle_timeout(self, child):
+        for action_id, node_id in child:
+            status = ao.Action.check_status(self.context, action_id, 0)
+            if (status == consts.ACTION_WAITING_LIFECYCLE_COMPLETION):
+                ao.Action.update(self.context, action_id,
+                                 {'status': base.Action.READY})
+
     def _delete_nodes(self, node_ids):
         action_name = consts.NODE_DELETE
 
@@ -297,31 +317,81 @@ class ClusterAction(base.Action):
             if destroy is False:
                 action_name = consts.NODE_LEAVE
 
+        # get lifecycle hook properties if specified
+        lifecycle_hook = self.data.get('hooks')
+        lifecycle_hook_timeout = None
+        if lifecycle_hook:
+            lifecycle_hook_timeout = lifecycle_hook.get('timeout')
+            lifecycle_hook_type = lifecycle_hook.get('type', None)
+            lifecycle_hook_params = lifecycle_hook.get('params')
+            if lifecycle_hook_type == "zaqar":
+                lifecycle_hook_target = lifecycle_hook_params.get('queue')
+            else:
+                # lifecycle_hook_target = lifecycle_hook_params.get('url')
+                return self.RES_ERROR, _("Lifecycle hook type '%s' is not "
+                                         "implemented") % lifecycle_hook_type
+
         child = []
         for node_id in node_ids:
             kwargs = {
                 'name': 'node_delete_%s' % node_id[:8],
                 'cause': consts.CAUSE_DERIVED,
             }
+
+            if lifecycle_hook:
+                kwargs['cause'] = consts.CAUSE_DERIVED_LCH
+
             action_id = base.Action.create(self.context, node_id, action_name,
                                            **kwargs)
-            child.append(action_id)
+            child.append((action_id, node_id))
 
         if child:
-            dobj.Dependency.create(self.context, [c for c in child], self.id)
-            for cid in child:
-                # Build dependency and make the new action ready
-                ao.Action.update(self.context, cid,
-                                 {'status': base.Action.READY})
-            dispatcher.start_action()
+            dobj.Dependency.create(self.context, [aid for aid, nid in child],
+                                   self.id)
+            for action_id, node_id in child:
+                # Build dependency and make the new action ready or
+                # waiting for lifecycle completion if lifecycle hook properties
+                # are specified in deletion policy
+
+                if not lifecycle_hook:
+                    status = base.Action.READY
+                else:
+                    status = base.Action.WAITING_LIFECYCLE_COMPLETION
+
+                ao.Action.update(self.context, action_id,
+                                 {'status': status})
+                if lifecycle_hook:
+                    # lifecycle_hook_type has to be "zaqar"
+                    # post message to zaqar
+                    kwargs = {
+                        'user': self.context.user_id,
+                        'project': self.context.project_id,
+                        'domain': self.context.domain_id
+                    }
+
+                    notifier = msg.Message(lifecycle_hook_target, **kwargs)
+                    notifier.post_lifecycle_hook_message(
+                        action_id, node_id,
+                        consts.LIFECYCLE_NODE_TERMINATION)
+
+            res = None
+            if lifecycle_hook:
+                dispatcher.start_action()
+                res, reason = self._wait_for_dependents(lifecycle_hook_timeout)
+
+                if res == self.RES_LIFECYCLE_HOOK_TIMEOUT:
+                    self._handle_lifecycle_timeout(child)
+
+            if res is None or res == self.RES_LIFECYCLE_HOOK_TIMEOUT:
+                dispatcher.start_action()
+                res, reason = self._wait_for_dependents()
 
-            res, reason = self._wait_for_dependents()
             if res == self.RES_OK:
                 self.outputs['nodes_removed'] = node_ids
                 for node_id in node_ids:
                     self.entity.remove_node(node_id)
             else:
-                reason = _('Failed in deleting nodes.')
+                reason = 'Failed in deleting nodes.'
 
             return res, reason
 
@@ -333,7 +403,7 @@ class ClusterAction(base.Action):
 
         :returns: A tuple containing the result and the corresponding reason.
         """
-        reason = _('Deletion in progress.')
+        reason = 'Deletion in progress.'
         self.entity.set_status(self.context, consts.CS_DELETING, reason)
         node_ids = [node.id for node in self.entity.nodes]
 
@@ -353,7 +423,7 @@ class ClusterAction(base.Action):
         res = self.entity.do_delete(self.context)
         if not res:
             self.entity.eval_status(self.context, consts.CLUSTER_DELETE)
-            return self.RES_ERROR, _('Cannot delete cluster object.')
+            return self.RES_ERROR, 'Cannot delete cluster object.'
 
         # Remove all action records which target on deleted
         # cluster except the on-going CLUSTER_DELETE action from DB
@@ -382,16 +452,16 @@ class ClusterAction(base.Action):
         for nid in node_ids:
             node = no.Node.get(self.context, nid)
             if not node:
-                errors.append(_('Node %s is not found.') % nid)
+                errors.append('Node %s is not found.' % nid)
                 continue
 
             if node.cluster_id:
-                errors.append(_('Node %(n)s is already owned by cluster %(c)s.'
-                                ) % {'n': nid, 'c': node.cluster_id})
+                errors.append('Node %(n)s is already owned by cluster %(c)s.'
+                              '' % {'n': nid, 'c': node.cluster_id})
                 continue
 
             if node.status != "ACTIVE":
-                errors.append(_('Node %s is not in ACTIVE status.') % nid)
+                errors.append('Node %s is not in ACTIVE status.' % nid)
                 continue
 
             nodes.append(node)
@@ -399,7 +469,7 @@ class ClusterAction(base.Action):
         if len(errors) > 0:
             return self.RES_ERROR, '\n'.join(errors)
 
-        reason = _('Completed adding nodes.')
+        reason = 'Completed adding nodes.'
         # check the size constraint
         current = no.Node.count_by_cluster(self.context, self.target)
         desired = current + len(node_ids)
@@ -485,10 +555,10 @@ class ClusterAction(base.Action):
                 nodes.remove(node_id)
 
         if len(errors) > 0:
-            msg = _("Nodes not found: %s.") % errors
+            msg = "Nodes not found: %s." % errors
             return self.RES_ERROR, msg
 
-        reason = _('Completed deleting nodes.')
+        reason = 'Completed deleting nodes.'
         if len(nodes) == 0:
             return self.RES_OK, reason
 
@@ -532,26 +602,23 @@ class ClusterAction(base.Action):
 
             # The return value is None if node not found
             if not original_node:
-                errors.append(_('Original node %s not found.'
-                                ) % original)
+                errors.append('Original node %s not found.' % original)
                 continue
             if not replacement_node:
-                errors.append(_('Replacement node %s not found.'
-                                ) % replacement)
+                errors.append('Replacement node %s not found.' % replacement)
                 continue
             if original_node.cluster_id != self.target:
-                errors.append(_('Node %(o)s is not a member of the '
-                                'cluster %(c)s.') % {'o': original,
-                                                     'c': self.target})
+                errors.append('Node %(o)s is not a member of the '
+                              'cluster %(c)s.' % {'o': original,
+                                                  'c': self.target})
                 continue
             if replacement_node.cluster_id:
-                errors.append(_('Node %(r)s is already owned by cluster %(c)s.'
-                                ) % {'r': replacement,
-                                     'c': replacement_node.cluster_id})
+                errors.append(('Node %(r)s is already owned by cluster %(c)s.'
+                               ) % {'r': replacement,
+                                    'c': replacement_node.cluster_id})
                 continue
             if replacement_node.status != consts.NS_ACTIVE:
-                errors.append(_('Node %s is not in ACTIVE status.'
-                                ) % replacement)
+                errors.append('Node %s is not in ACTIVE status.' % replacement)
                 continue
             original_nodes.append(original_node)
             replacement_nodes.append(replacement_node)
@@ -560,7 +627,7 @@ class ClusterAction(base.Action):
             return self.RES_ERROR, '\n'.join(errors)
 
         result = self.RES_OK
-        reason = _('Completed replacing nodes.')
+        reason = 'Completed replacing nodes.'
 
         children = []
         for (original, replacement) in node_dict.items():
@@ -616,14 +683,23 @@ class ClusterAction(base.Action):
 
         child = []
         res = self.RES_OK
-        reason = _('Cluster checking completed.')
+        reason = 'Cluster checking completed.'
         for node in self.entity.nodes:
             node_id = node.id
+            need_delete = self.inputs.get('delete_check_action', False)
+            # delete some records of NODE_CHECK
+            if need_delete:
+                ao.Action.delete_by_target(
+                    self.context, node_id, action=[consts.NODE_CHECK],
+                    status=[consts.ACTION_SUCCEEDED, consts.ACTION_FAILED])
+
+            name = 'node_check_%s' % node_id[:8]
             action_id = base.Action.create(
-                self.context, node_id, consts.NODE_CHECK,
-                name='node_check_%s' % node_id[:8],
+                self.context, node_id, consts.NODE_CHECK, name=name,
                 cause=consts.CAUSE_DERIVED,
+                inputs=self.inputs
             )
+
             child.append(action_id)
 
         if child:
@@ -699,7 +775,7 @@ class ClusterAction(base.Action):
             children.append(action_id)
 
         res = self.RES_OK
-        reason = _('Cluster recovery succeeded.')
+        reason = 'Cluster recovery succeeded.'
         if children:
             dobj.Dependency.create(self.context, [c for c in children],
                                    self.id)
@@ -729,7 +805,7 @@ class ClusterAction(base.Action):
         if max_size is not None:
             kwargs['max_size'] = max_size
         self.entity.set_status(self.context, consts.CS_RESIZING,
-                               _('Cluster resize started.'), **kwargs)
+                               'Cluster resize started.', **kwargs)
 
     @profiler.trace('ClusterAction.do_resize', hide_args=False)
     def do_resize(self):
@@ -749,7 +825,7 @@ class ClusterAction(base.Action):
                 return result, reason
 
         # action input consolidated to action data now
-        reason = _('Cluster resize succeeded.')
+        reason = 'Cluster resize succeeded.'
         if 'deletion' in self.data:
             count = self.data['deletion']['count']
             candidates = self.data['deletion'].get('candidates', [])
@@ -792,7 +868,7 @@ class ClusterAction(base.Action):
             value = self.inputs.get('count', 1)
             success, count = utils.get_positive_int(value)
             if not success:
-                reason = _('Invalid count (%s) for scaling out.') % value
+                reason = 'Invalid count (%s) for scaling out.' % value
                 return self.RES_ERROR, reason
 
         # check provided params against current properties
@@ -805,12 +881,12 @@ class ClusterAction(base.Action):
             return self.RES_ERROR, result
 
         self.entity.set_status(self.context, consts.CS_RESIZING,
-                               _('Cluster scale out started.'),
+                               'Cluster scale out started.',
                                desired_capacity=new_size)
 
         result, reason = self._create_nodes(count)
         if result == self.RES_OK:
-            reason = _('Cluster scaling succeeded.')
+            reason = 'Cluster scaling succeeded.'
         self.entity.eval_status(self.context, consts.CLUSTER_SCALE_OUT)
 
         return result, reason
@@ -836,15 +912,15 @@ class ClusterAction(base.Action):
             value = self.inputs.get('count', 1)
             success, count = utils.get_positive_int(value)
             if not success:
-                reason = _('Invalid count (%s) for scaling in.') % value
+                reason = 'Invalid count (%s) for scaling in.' % value
                 return self.RES_ERROR, reason
 
         # check provided params against current properties
         # desired is checked when strict is True
         curr_size = no.Node.count_by_cluster(self.context, self.target)
         if count > curr_size:
-            msg = _("Triming count (%(count)s) to current "
-                    "cluster size (%(curr)s) for scaling in")
+            msg = ("Triming count (%(count)s) to current "
+                   "cluster size (%(curr)s) for scaling in")
             LOG.warning(msg, {'count': count, 'curr': curr_size})
             count = curr_size
         new_size = curr_size - count
@@ -855,7 +931,7 @@ class ClusterAction(base.Action):
             return self.RES_ERROR, result
 
         self.entity.set_status(self.context, consts.CS_RESIZING,
-                               _('Cluster scale in started.'),
+                               'Cluster scale in started.',
                                desired_capacity=new_size)
 
         # Choose victims randomly
@@ -868,7 +944,7 @@ class ClusterAction(base.Action):
         result, reason = self._delete_nodes(candidates)
 
         if result == self.RES_OK:
-            reason = _('Cluster scaling succeeded.')
+            reason = 'Cluster scaling succeeded.'
 
         self.entity.eval_status(self.context, consts.CLUSTER_SCALE_IN)
 
@@ -883,7 +959,7 @@ class ClusterAction(base.Action):
         inputs = dict(self.inputs)
         policy_id = inputs.pop('policy_id', None)
         if not policy_id:
-            return self.RES_ERROR, _('Policy not specified.')
+            return self.RES_ERROR, 'Policy not specified.'
 
         res, reason = self.entity.attach_policy(self.context, policy_id,
                                                 inputs)
@@ -903,7 +979,7 @@ class ClusterAction(base.Action):
         """
         policy_id = self.inputs.get('policy_id', None)
         if not policy_id:
-            return self.RES_ERROR, _('Policy not specified.')
+            return self.RES_ERROR, 'Policy not specified.'
 
         res, reason = self.entity.detach_policy(self.context, policy_id)
         result = self.RES_OK if res else self.RES_ERROR
@@ -922,7 +998,7 @@ class ClusterAction(base.Action):
         """
         policy_id = self.inputs.pop('policy_id', None)
         if not policy_id:
-            return self.RES_ERROR, _('Policy not specified.')
+            return self.RES_ERROR, 'Policy not specified.'
         res, reason = self.entity.update_policy(self.context, policy_id,
                                                 **self.inputs)
         result = self.RES_OK if res else self.RES_ERROR
@@ -946,7 +1022,7 @@ class ClusterAction(base.Action):
 
         child = []
         res = self.RES_OK
-        reason = _("Cluster operation '%s' completed.") % operation
+        reason = "Cluster operation '%s' completed." % operation
         nodes = inputs.pop('nodes')
         for node_id in nodes:
             action_id = base.Action.create(
@@ -983,7 +1059,7 @@ class ClusterAction(base.Action):
         # do pre-action policy checking
         self.policy_check(self.entity.id, 'BEFORE')
         if self.data['status'] != policy_mod.CHECK_OK:
-            reason = _('Policy check failure: %s') % self.data['reason']
+            reason = 'Policy check failure: %s' % self.data['reason']
             return self.RES_ERROR, reason
 
         result = self.RES_OK
@@ -991,7 +1067,7 @@ class ClusterAction(base.Action):
         method_name = action_name.replace('cluster', 'do')
         method = getattr(self, method_name, None)
         if method is None:
-            reason = _('Unsupported action: %s.') % self.action
+            reason = 'Unsupported action: %s.' % self.action
             return self.RES_ERROR, reason
 
         result, reason = method()
@@ -1000,7 +1076,7 @@ class ClusterAction(base.Action):
         if result == self.RES_OK:
             self.policy_check(self.entity.id, 'AFTER')
             if self.data['status'] != policy_mod.CHECK_OK:
-                reason = _('Policy check failure: %s') % self.data['reason']
+                reason = 'Policy check failure: %s' % self.data['reason']
                 return self.RES_ERROR, reason
 
         return result, reason
@@ -1022,7 +1098,7 @@ class ClusterAction(base.Action):
                                                forced)
         # Failed to acquire lock, return RES_RETRY
         if not res:
-            return self.RES_RETRY, _('Failed in locking cluster.')
+            return self.RES_RETRY, 'Failed in locking cluster.'
 
         try:
             res, reason = self._execute(**kwargs)
@@ -1035,3 +1111,22 @@ class ClusterAction(base.Action):
     def cancel(self):
         """Handler to cancel the execution of action."""
         return self.RES_OK
+
+
+def CompleteLifecycleProc(context, action_id):
+    """Complete lifecycle process."""
+
+    action = base.Action.load(context, action_id=action_id, project_safe=False)
+    if action is None:
+        LOG.error("Action %s could not be found.", action_id)
+        raise exception.ResourceNotFound(type='action', id=action_id)
+
+    if action.get_status() == consts.ACTION_WAITING_LIFECYCLE_COMPLETION:
+        action.set_status(base.Action.RES_LIFECYCLE_COMPLETE)
+        dispatcher.start_action()
+    else:
+        LOG.debug('Action %s status is not WAITING_LIFECYCLE.  '
+                  'Skip CompleteLifecycleProc', action_id)
+        return False
+
+    return True
