@@ -10,12 +10,16 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from oslo_config import cfg
-from oslo_log import log as logging
+import eventlet
+import random
 import time
 
-from senlin.common.i18n import _, _LE, _LI
+from oslo_config import cfg
+from oslo_log import log as logging
+
+from senlin.common.i18n import _
 from senlin.common import utils
+from senlin import objects
 from senlin.objects import action as ao
 from senlin.objects import cluster_lock as cl_obj
 from senlin.objects import node_lock as nl_obj
@@ -38,6 +42,7 @@ def cluster_lock_acquire(context, cluster_id, action_id, engine=None,
                          scope=CLUSTER_SCOPE, forced=False):
     """Try to lock the specified cluster.
 
+    :param context: the context used for DB operations.
     :param cluster_id: ID of the cluster to be locked.
     :param action_id: ID of the action which wants to lock the cluster.
     :param engine: ID of the engine which wants to lock the cluster.
@@ -50,9 +55,11 @@ def cluster_lock_acquire(context, cluster_id, action_id, engine=None,
 
     # Step 1: try lock the cluster - if the returned owner_id is the
     #         action id, it was a success
-    owners = cl_obj.ClusterLock.acquire(cluster_id, action_id, scope)
-    if action_id in owners:
-        return True
+    for retries in range(3):
+        owners = cl_obj.ClusterLock.acquire(cluster_id, action_id, scope)
+        if action_id in owners:
+            return True
+        eventlet.sleep(random.randrange(1, 3))
 
     # Step 2: Last resort is 'forced locking', only needed when retry failed
     if forced:
@@ -64,20 +71,21 @@ def cluster_lock_acquire(context, cluster_id, action_id, engine=None,
     action = ao.Action.get(context, owners[0])
     if (action and action.owner and action.owner != engine and
             utils.is_engine_dead(context, action.owner)):
-        LOG.info(_LI('The cluster %(c)s is locked by dead action %(a)s, '
-                     'try to steal the lock.'), {
-            'c': cluster_id,
-            'a': owners[0]
-        })
-        reason = _('Engine died when executing this action.')
+        LOG.info('The cluster %(c)s is locked by dead action %(a)s, '
+                 'try to steal the lock.',
+                 {'c': cluster_id, 'a': owners[0]})
+        dead_engine = action.owner
         owners = cl_obj.ClusterLock.steal(cluster_id, action_id)
-        # Mark the old action to failed.
-        ao.Action.mark_failed(context, action.id, time.time(), reason)
+        # Cleanse locks affected by the dead engine
+        objects.Service.gc_by_engine(dead_engine)
         return action_id in owners
 
-    LOG.error(_LE('Cluster is already locked by action %(old)s, '
-                  'action %(new)s failed grabbing the lock'),
-              {'old': str(owners), 'new': action_id})
+    lock_owners = []
+    for o in owners:
+        lock_owners.append(o[:8])
+    LOG.warning('Cluster is already locked by action %(old)s, '
+                'action %(new)s failed grabbing the lock',
+                {'old': str(lock_owners), 'new': action_id[:8]})
 
     return False
 
@@ -85,8 +93,8 @@ def cluster_lock_acquire(context, cluster_id, action_id, engine=None,
 def cluster_lock_release(cluster_id, action_id, scope):
     """Release the lock on the specified cluster.
 
-    :param cluster_id: ID of the node to be released.
-    :param action_id: ID of the action that attempts to release the node.
+    :param cluster_id: ID of the cluster to be released.
+    :param action_id: ID of the action that attempts to release the cluster.
     :param scope: The scope of the lock to be released.
     """
     return cl_obj.ClusterLock.release(cluster_id, action_id, scope)
@@ -96,7 +104,7 @@ def node_lock_acquire(context, node_id, action_id, engine=None,
                       forced=False):
     """Try to lock the specified node.
 
-    :param context: the context used for DB operations;
+    :param context: the context used for DB operations.
     :param node_id: ID of the node to be locked.
     :param action_id: ID of the action that attempts to lock the node.
     :param engine: ID of the engine that attempts to lock the node.
@@ -120,18 +128,16 @@ def node_lock_acquire(context, node_id, action_id, engine=None,
     action = ao.Action.get(context, owner)
     if (action and action.owner and action.owner != engine and
             utils.is_engine_dead(context, action.owner)):
-        LOG.info(_LI('The node %(n)s is locked by dead action %(a)s, '
-                     'try to steal the lock.'), {
-            'n': node_id,
-            'a': owner
-        })
+        LOG.info('The node %(n)s is locked by dead action %(a)s, '
+                 'try to steal the lock.',
+                 {'n': node_id, 'a': owner})
         reason = _('Engine died when executing this action.')
         nl_obj.NodeLock.steal(node_id, action_id)
         ao.Action.mark_failed(context, action.id, time.time(), reason)
         return True
 
-    LOG.error(_LE('Node is already locked by action %(old)s, '
-                  'action %(new)s failed grabbing the lock'),
+    LOG.error('Node is already locked by action %(old)s, '
+              'action %(new)s failed grabbing the lock',
               {'old': owner, 'new': action_id})
 
     return False
