@@ -113,6 +113,10 @@ class Action(base.SenlinObject, base.VersionedObjectDictCompat):
     def mark_succeeded(cls, context, action_id, timestamp):
         return db_api.action_mark_succeeded(context, action_id, timestamp)
 
+    @classmethod
+    def mark_ready(cls, context, action_id, timestamp):
+        return db_api.action_mark_ready(context, action_id, timestamp)
+
     @classmethod
     def mark_failed(cls, context, action_id, timestamp, reason=None):
         return db_api.action_mark_failed(context, action_id, timestamp, reason)
@@ -130,8 +134,12 @@ class Action(base.SenlinObject, base.VersionedObjectDictCompat):
         return db_api.action_acquire_random_ready(context, owner, timestamp)
 
     @classmethod
-    def abandon(cls, context, action_id):
-        return db_api.action_abandon(context, action_id)
+    def acquire_first_ready(cls, context, owner, timestamp):
+        return db_api.action_acquire_first_ready(context, owner, timestamp)
+
+    @classmethod
+    def abandon(cls, context, action_id, values=None):
+        return db_api.action_abandon(context, action_id, values)
 
     @classmethod
     def signal(cls, context, action_id, value):
@@ -153,6 +161,21 @@ class Action(base.SenlinObject, base.VersionedObjectDictCompat):
     def delete(cls, context, action_id):
         db_api.action_delete(context, action_id)
 
+    @classmethod
+    def delete_by_target(cls, context, target, action=None,
+                         action_excluded=None, status=None):
+        """Delete an action with the target and other given params.
+
+        :param target: The ID of the target cluster/node
+        :param action: A list of actions to be included.
+        :param action_excluded: A list of actions to be excluded.
+        :param status: A list of statuses to be delete filtered.
+        :return: None.
+        """
+        return db_api.action_delete_by_target(context, target, action=action,
+                                              action_excluded=action_excluded,
+                                              status=status)
+
     def to_dict(self):
         if self.id:
             dep_on = dobj.Dependency.get_depended(self.context, self.id)
