@@ -46,6 +46,7 @@ class Node(base.SenlinObject, base.VersionedObjectDictCompat):
         'domain': fields.StringField(nullable=True),
         'dependents': fields.JsonField(nullable=True),
         'profile_name': fields.StringField(nullable=True),
+        'profile_created_at': fields.StringField(nullable=True),
     }
 
     @staticmethod
@@ -58,6 +59,9 @@ class Node(base.SenlinObject, base.VersionedObjectDictCompat):
             elif field == 'profile_name':
                 p = db_obj['profile']
                 obj['profile_name'] = p.name if p else 'Unknown'
+            elif field == 'profile_created_at':
+                p = db_obj['profile']
+                obj['profile_created_at'] = p.created_at if p else None
             else:
                 obj[field] = db_obj[field]
 
@@ -124,20 +128,21 @@ class Node(base.SenlinObject, base.VersionedObjectDictCompat):
 
     @classmethod
     def get_all(cls, context, **kwargs):
-        # TODO(anyone): This has to be improved. 1. The DB API called always
-        # does pagination which is an overkill. 2. we may need a new API for
-        # getting node IDs only since that is a common case for internal use.
         objs = db_api.node_get_all(context, **kwargs)
         return [cls._from_db_object(context, cls(), obj) for obj in objs]
 
     @classmethod
-    def get_all_by_cluster(cls, context, cluster_id, **kwargs):
-        # TODO(anyone): This has to be improved. 1. The DB API called doesn't
-        # support filters. 2. we may need a new API for getting node IDs only
-        # since that is a common case for internal use.
-        objs = db_api.node_get_all_by_cluster(context, cluster_id, **kwargs)
+    def get_all_by_cluster(cls, context, cluster_id, filters=None,
+                           project_safe=True):
+        objs = db_api.node_get_all_by_cluster(
+            context, cluster_id, filters=filters, project_safe=project_safe)
         return [cls._from_db_object(context, cls(), obj) for obj in objs]
 
+    @classmethod
+    def ids_by_cluster(cls, context, cluster_id, filters=None):
+        """An internal API for retrieving node ids only."""
+        return db_api.node_ids_by_cluster(context, cluster_id, filters=filters)
+
     @classmethod
     def count_by_cluster(cls, context, cluster_id, **kwargs):
         return db_api.node_count_by_cluster(context, cluster_id, **kwargs)
