@@ -14,7 +14,7 @@
 Policy for placing nodes based on Nova server groups.
 
 NOTE:  For full documentation about how the affinity policy works, check:
-http://docs.openstack.org/developer/senlin/developer/policies/affinity_v1.html
+https://docs.openstack.org/senlin/latest/developer/policies/affinity_v1.html
 """
 
 import re
@@ -25,13 +25,11 @@ from senlin.common import constraints
 from senlin.common import consts
 from senlin.common import context
 from senlin.common import exception as exc
-from senlin.common.i18n import _, _LE
+from senlin.common.i18n import _
 from senlin.common import scaleutils as su
 from senlin.common import schema
 from senlin.common import utils
-from senlin.objects import cluster as co
 from senlin.objects import cluster_policy as cpo
-from senlin.objects import node as no
 from senlin.policies import base
 
 
@@ -75,9 +73,10 @@ class AffinityPolicy(base.Policy):
     )
 
     _POLICIES_VALUES = (
-        AFFINITY, ANTI_AFFINITY,
+        # NOTE: soft policies are supported from compute micro version 2.15
+        AFFINITY, SOFT_AFFINITY, ANTI_AFFINITY, SOFT_ANTI_AFFINITY,
     ) = (
-        'affinity', 'anti-affinity',
+        'affinity', 'soft-affinity', 'anti-affinity', 'soft-anti-affinity',
     )
 
     properties_schema = {
@@ -118,7 +117,7 @@ class AffinityPolicy(base.Policy):
 
         az_name = self.properties.get(self.AVAILABILITY_ZONE)
         if az_name:
-            nc = self.nova(context.user, context.project)
+            nc = self.nova(context.user_id, context.project_id)
             valid_azs = nc.validate_azs([az_name])
             if not valid_azs:
                 msg = _("The specified %(key)s '%(value)s' could not be "
@@ -159,7 +158,7 @@ class AffinityPolicy(base.Policy):
             except exc.InternalError as ex:
                 msg = _("Failed in retrieving servergroup '%s'."
                         ) % group_name
-                LOG.exception(_LE('%(msg)s: %(ex)s') % {
+                LOG.exception('%(msg)s: %(ex)s' % {
                               'msg': msg, 'ex': six.text_type(ex)})
                 return False, msg
 
@@ -186,7 +185,7 @@ class AffinityPolicy(base.Policy):
                     policies=[group.get(self.GROUP_POLICIES)])
             except Exception as ex:
                 msg = _('Failed in creating servergroup.')
-                LOG.exception(_LE('%(msg)s: %(ex)s') % {
+                LOG.exception('%(msg)s: %(ex)s' % {
                     'msg': msg, 'ex': six.text_type(ex)})
                 return False, msg
 
@@ -226,7 +225,7 @@ class AffinityPolicy(base.Policy):
                 nc.server_group_delete(group_id)
             except Exception as ex:
                 msg = _('Failed in deleting servergroup.')
-                LOG.exception(_LE('%(msg)s: %(ex)s') % {
+                LOG.exception('%(msg)s: %(ex)s' % {
                     'msg': msg, 'ex': six.text_type(ex)})
                 return False, msg
 
@@ -260,9 +259,9 @@ class AffinityPolicy(base.Policy):
         elif action.action == consts.NODE_CREATE:
             count = 1
         else:  # CLUSTER_RESIZE
-            db_cluster = co.Cluster.get(action.context, cluster_id)
-            current = no.Node.count_by_cluster(action.context, cluster_id)
-            su.parse_resize_params(action, db_cluster, current)
+            cluster = action.entity
+            current = len(cluster.nodes)
+            su.parse_resize_params(action, cluster, current)
             if 'creation' not in action.data:
                 return
             count = action.data['creation']['count']
@@ -274,8 +273,8 @@ class AffinityPolicy(base.Policy):
         # special handling for vSphere DRS case where we need to find out
         # the name of the vSphere host which has DRS enabled.
         if self.enable_drs:
-            cluster_obj = co.Cluster.get(action.context, cluster_id)
-            nc = self.nova(cluster_obj.user, cluster_obj.project)
+            obj = action.entity
+            nc = self.nova(obj.user, obj.project)
 
             hypervisors = nc.hypervisor_list()
             hv_id = ''
