@@ -13,8 +13,8 @@
 """
 Policy for load-balancing among nodes in a cluster.
 
-NOTE: For full documentation about how the deletion policy works, check:
-https://docs.openstack.org/senlin/latest/developer/policies/
+NOTE: For full documentation about how the load-balancing policy works,
+check: https://docs.openstack.org/senlin/latest/developer/policies/
 load_balance_v1.html
 """
 
@@ -46,7 +46,10 @@ class LoadBalancingPolicy(base.Policy):
     VERSIONS = {
         '1.0': [
             {'status': consts.SUPPORTED, 'since': '2016.04'}
-        ]
+        ],
+        '1.1': [
+            {'status': consts.SUPPORTED, 'since': '2018.01'}
+        ],
     }
 
     PRIORITY = 500
@@ -68,17 +71,17 @@ class LoadBalancingPolicy(base.Policy):
     ]
 
     KEYS = (
-        POOL, VIP, HEALTH_MONITOR, LB_STATUS_TIMEOUT
+        POOL, VIP, HEALTH_MONITOR, LB_STATUS_TIMEOUT, LOADBALANCER
     ) = (
-        'pool', 'vip', 'health_monitor', 'lb_status_timeout'
+        'pool', 'vip', 'health_monitor', 'lb_status_timeout', 'loadbalancer'
     )
 
     _POOL_KEYS = (
         POOL_PROTOCOL, POOL_PROTOCOL_PORT, POOL_SUBNET,
-        POOL_LB_METHOD, POOL_ADMIN_STATE_UP, POOL_SESSION_PERSISTENCE,
+        POOL_LB_METHOD, POOL_ADMIN_STATE_UP, POOL_SESSION_PERSISTENCE, POOL_ID,
     ) = (
         'protocol', 'protocol_port', 'subnet',
-        'lb_method', 'admin_state_up', 'session_persistence',
+        'lb_method', 'admin_state_up', 'session_persistence', 'id',
     )
 
     PROTOCOLS = (
@@ -115,10 +118,10 @@ class LoadBalancingPolicy(base.Policy):
 
     HEALTH_MONITOR_KEYS = (
         HM_TYPE, HM_DELAY, HM_TIMEOUT, HM_MAX_RETRIES, HM_ADMIN_STATE_UP,
-        HM_HTTP_METHOD, HM_URL_PATH, HM_EXPECTED_CODES,
+        HM_HTTP_METHOD, HM_URL_PATH, HM_EXPECTED_CODES, HM_ID,
     ) = (
         'type', 'delay', 'timeout', 'max_retries', 'admin_state_up',
-        'http_method', 'url_path', 'expected_codes',
+        'http_method', 'url_path', 'expected_codes', 'id',
     )
 
     _SESSION_PERSISTENCE_KEYS = (
@@ -179,6 +182,11 @@ class LoadBalancingPolicy(base.Policy):
                     },
                     default={},
                 ),
+                POOL_ID: schema.String(
+                    _('ID of pool for the cluster on which nodes can '
+                      'be connected.'),
+                    default=None,
+                ),
             },
         ),
         VIP: schema.Map(
@@ -258,12 +266,21 @@ class LoadBalancingPolicy(base.Policy):
                 HM_EXPECTED_CODES: schema.String(
                     _('Expected HTTP codes for a passing HTTP(S) monitor.'),
                 ),
+                HM_ID: schema.String(
+                    _('ID of the health manager for the loadbalancer.'),
+                    default=None,
+                ),
             },
         ),
         LB_STATUS_TIMEOUT: schema.Integer(
             _('Time in second to wait for loadbalancer to become ready '
               'after senlin requests LBaaS V2 service for operations.'),
             default=300,
+        ),
+        LOADBALANCER: schema.String(
+            _('Name or ID of loadbalancer for the cluster on which nodes can '
+              'be connected.'),
+            default=None,
         )
     }
 
@@ -274,7 +291,7 @@ class LoadBalancingPolicy(base.Policy):
         self.vip_spec = self.properties.get(self.VIP, {})
         self.hm_spec = self.properties.get(self.HEALTH_MONITOR, None)
         self.lb_status_timeout = self.properties.get(self.LB_STATUS_TIMEOUT)
-        self.lb = None
+        self.lb = self.properties.get(self.LOADBALANCER, None)
 
     def validate(self, context, validate_props=False):
         super(LoadBalancingPolicy, self).validate(context, validate_props)
@@ -282,7 +299,8 @@ class LoadBalancingPolicy(base.Policy):
         if not validate_props:
             return True
 
-        nc = self.network(context.user, context.project)
+        nc = self.network(context.user_id, context.project_id)
+        oc = self.octavia(context.user_id, context.project_id)
 
         # validate pool subnet
         name_or_id = self.pool_spec.get(self.POOL_SUBNET)
@@ -302,6 +320,15 @@ class LoadBalancingPolicy(base.Policy):
                     ) % {'key': self.VIP_SUBNET, 'value': name_or_id}
             raise exc.InvalidSpec(message=msg)
 
+        # validate loadbalancer
+        if self.lb:
+            try:
+                oc.loadbalancer_get(self.lb)
+            except exc.InternalError:
+                msg = _("The specified %(key)s '%(value)s' could not be found."
+                        ) % {'key': self.LOADBALANCER, 'value': self.lb}
+                raise exc.InvalidSpec(message=msg)
+
     def attach(self, cluster, enabled=True):
         """Routine to be invoked when policy is to be attached to a cluster.
 
@@ -320,10 +347,19 @@ class LoadBalancingPolicy(base.Policy):
         # TODO(Anyone): Check if existing nodes has conflicts regarding the
         # subnets. Each VM addresses detail has a key named to the network
         # which can be used for validation.
-        res, data = lb_driver.lb_create(self.vip_spec, self.pool_spec,
-                                        self.hm_spec)
-        if res is False:
-            return False, data
+        if self.lb:
+            data = {}
+            data['preexisting'] = True
+            data['loadbalancer'] = self.lb
+            data['pool'] = self.pool_spec.get(self.POOL_ID, None)
+            data['vip_address'] = self.vip_spec.get(self.VIP_ADDRESS, None)
+            if self.hm_spec and self.hm_spec.get(self.HM_ID, None):
+                data['healthmonitor'] = self.hm_spec.get(self.HM_ID)
+        else:
+            res, data = lb_driver.lb_create(self.vip_spec, self.pool_spec,
+                                            self.hm_spec)
+            if res is False:
+                return False, data
 
         port = self.pool_spec.get(self.POOL_PROTOCOL_PORT)
         subnet = self.pool_spec.get(self.POOL_SUBNET)
@@ -336,7 +372,8 @@ class LoadBalancingPolicy(base.Policy):
                 # were created and return the failure reason.
                 # TODO(anyone): May need to "roll-back" changes caused by any
                 # successful member_add() calls.
-                lb_driver.lb_delete(**data)
+                if not self.lb:
+                    lb_driver.lb_delete(**data)
                 return False, 'Failed in adding node into lb pool'
 
             node.data.update({'lb_member': member_id})
@@ -371,15 +408,26 @@ class LoadBalancingPolicy(base.Policy):
         if policy_data is None:
             return True, reason
 
-        res, reason = lb_driver.lb_delete(**policy_data)
-        if res is False:
-            return False, reason
+        is_existed = policy_data.get('preexisting', False)
+        if not is_existed:
+            res, reason = lb_driver.lb_delete(**policy_data)
+            if res is False:
+                return False, reason
 
-        for node in cluster.nodes:
-            if 'lb_member' in node.data:
-                node.data.pop('lb_member')
-                values = {'data': node.data}
-                no.Node.update(oslo_context.get_current(), node.id, values)
+            for node in cluster.nodes:
+                if 'lb_member' in node.data:
+                    node.data.pop('lb_member')
+                    values = {'data': node.data}
+                    no.Node.update(oslo_context.get_current(),
+                                   node.id, values)
+        else:
+            # the lb pool is existed, we need to remove servers from it
+            nodes = cluster.nodes
+            failed = self._remove_member(oslo_context.get_current(),
+                                         [node.id for node in nodes],
+                                         cp, lb_driver)
+            if failed:
+                return False, _('Failed to remove servers from existed LB.')
 
         lb_data = cluster.data.get('loadbalancers', {})
         if lb_data and isinstance(lb_data, dict):
@@ -437,7 +485,7 @@ class LoadBalancingPolicy(base.Policy):
 
         return candidates
 
-    def _remove_member(self, candidates, policy, action, driver,
+    def _remove_member(self, context, candidates, policy, driver,
                        handle_err=True):
         # Load policy data
         policy_data = self._extract_policy_data(policy.data)
@@ -446,7 +494,7 @@ class LoadBalancingPolicy(base.Policy):
 
         failed_nodes = []
         for node_id in candidates:
-            node = no.Node.get(action.context, node_id=node_id)
+            node = no.Node.get(context, node_id=node_id)
             node_data = node.data or {}
             member_id = node_data.get('lb_member', None)
             if member_id is None:
@@ -464,11 +512,11 @@ class LoadBalancingPolicy(base.Policy):
             else:
                 node.data.pop('lb_member', None)
                 values['data'] = node.data
-            no.Node.update(action.context, node_id, values)
+            no.Node.update(context, node_id, values)
 
         return failed_nodes
 
-    def _add_member(self, candidates, policy, action, driver):
+    def _add_member(self, context, candidates, policy, driver):
         # Load policy data
         policy_data = self._extract_policy_data(policy.data)
         lb_id = policy_data['loadbalancer']
@@ -478,7 +526,7 @@ class LoadBalancingPolicy(base.Policy):
 
         failed_nodes = []
         for node_id in candidates:
-            node = no.Node.get(action.context, node_id=node_id)
+            node = no.Node.get(context, node_id=node_id)
             node_data = node.data or {}
             member_id = node_data.get('lb_member', None)
             if member_id:
@@ -496,7 +544,7 @@ class LoadBalancingPolicy(base.Policy):
             else:
                 node.data.update({'lb_member': member_id})
                 values['data'] = node.data
-            no.Node.update(action.context, node_id, values)
+            no.Node.update(context, node_id, values)
 
         return failed_nodes
 
@@ -528,7 +576,7 @@ class LoadBalancingPolicy(base.Policy):
 
         # was a member of lb pool, check whether has been recreated
         if recovery is not None and recovery == consts.RECOVER_RECREATE:
-            self._remove_member(candidates, policy, action, driver,
+            self._remove_member(action.context, candidates, policy, driver,
                                 handle_err=False)
             data.pop('lb_member', None)
             values['data'] = data
@@ -553,6 +601,12 @@ class LoadBalancingPolicy(base.Policy):
         if len(candidates) == 0:
             return
 
+        hooks = action.data.get('hooks', {})
+        # if hooks properties are defined, defer removal of nodes from LB
+        # pool to the pre_op call during DEL_NODE action execution
+        if hooks:
+            return
+
         obj = action.entity
         lb_driver = self.lbaas(obj.user, obj.project)
         lb_driver.lb_status_timeout = self.lb_status_timeout
@@ -560,7 +614,8 @@ class LoadBalancingPolicy(base.Policy):
                                                self.id)
 
         # Remove nodes that will be deleted from lb pool
-        failed_nodes = self._remove_member(candidates, cp, action, lb_driver)
+        failed_nodes = self._remove_member(action.context, candidates,
+                                           cp, lb_driver)
 
         if failed_nodes:
             error = _('Failed in removing deleted node(s) from lb pool: %s'
@@ -581,6 +636,7 @@ class LoadBalancingPolicy(base.Policy):
         :param action: The action object that triggered this operation.
         :returns: Nothing.
         """
+
         # TODO(Yanyanhu): Need special handling for cross-az scenario
         # which is supported by Neutron lbaas.
         candidates = self._get_post_candidates(action)
@@ -599,7 +655,8 @@ class LoadBalancingPolicy(base.Policy):
                 return
 
         # Add new nodes to lb pool
-        failed_nodes = self._add_member(candidates, cp, action, lb_driver)
+        failed_nodes = self._add_member(action.context, candidates,
+                                        cp, lb_driver)
         if failed_nodes:
             error = _('Failed in adding nodes into lb pool: %s') % failed_nodes
             action.data['status'] = base.CHECK_ERROR
