@@ -14,12 +14,11 @@ from oslo_config import cfg
 
 from senlin.common import constraints
 from senlin.common import consts
+from senlin.common import exception as exc
 from senlin.common.i18n import _
 from senlin.common import scaleutils as su
 from senlin.common import schema
 from senlin.common import utils
-from senlin.objects import cluster as co
-from senlin.objects import node as no
 from senlin.policies import base
 
 
@@ -116,8 +115,8 @@ class ScalingPolicy(base.Policy):
         :param name: Name for the policy object.
         :param spec: A dictionary containing the detailed specification for
                      the policy.
-        :param \*\*kwargs: Other optional parameters for policy object
-                           creation.
+        :param dict kwargs: Other optional parameters for policy object
+                            creation.
         :return: An object of `ScalingPolicy`.
         """
         super(ScalingPolicy, self).__init__(name, spec, **kwargs)
@@ -134,6 +133,21 @@ class ScalingPolicy(base.Policy):
         self.best_effort = adjustment[self.BEST_EFFORT]
         self.cooldown = adjustment[self.COOLDOWN]
 
+    def validate(self, context, validate_props=False):
+        super(ScalingPolicy, self).validate(context, validate_props)
+
+        if self.adjustment_number <= 0:
+            msg = _("the 'number' for 'adjustment' must be > 0")
+            raise exc.InvalidSpec(message=msg)
+
+        if self.adjustment_min_step < 0:
+            msg = _("the 'min_step' for 'adjustment' must be >= 0")
+            raise exc.InvalidSpec(message=msg)
+
+        if self.cooldown < 0:
+            msg = _("the 'cooldown' for 'adjustment' must be >= 0")
+            raise exc.InvalidSpec(message=msg)
+
     def _calculate_adjustment_count(self, current_size):
         """Calculate adjustment count based on current_size.
 
@@ -169,7 +183,9 @@ class ScalingPolicy(base.Policy):
 
         # Use action input if count is provided
         count_value = action.inputs.get('count', None)
-        current = no.Node.count_by_cluster(action.context, cluster_id)
+        cluster = action.entity
+        current = len(cluster.nodes)
+
         if count_value is None:
             # count not specified, calculate it
             count_value = self._calculate_adjustment_count(current)
@@ -186,7 +202,6 @@ class ScalingPolicy(base.Policy):
             return
 
         # Check size constraints
-        cluster = co.Cluster.get(action.context, cluster_id)
         max_size = cluster.max_size
         if max_size == -1:
             max_size = cfg.CONF.max_nodes_per_cluster
