@@ -174,8 +174,8 @@ class Profile(object):
             metadata = {}
 
         try:
-            profile = cls(name, spec, metadata=metadata, user=ctx.user,
-                          project=ctx.project)
+            profile = cls(name, spec, metadata=metadata, user=ctx.user_id,
+                          project=ctx.project_id)
             profile.validate(True)
         except (exc.ResourceNotFound, exc.ESchema) as ex:
             error = _("Failed in creating profile %(name)s: %(error)s"
@@ -223,12 +223,32 @@ class Profile(object):
         profile = cls.load(ctx, profile_id=obj.profile_id)
         return profile.do_create(obj)
 
+    @classmethod
+    @profiler.trace('Profile.create_cluster_object', hide_args=False)
+    def create_cluster_object(cls, ctx, obj):
+        profile = cls.load(ctx, profile_id=obj.profile_id)
+        try:
+            ret = profile.do_cluster_create(obj)
+        except NotImplementedError:
+            return None
+        return ret
+
     @classmethod
     @profiler.trace('Profile.delete_object', hide_args=False)
     def delete_object(cls, ctx, obj, **params):
         profile = cls.load(ctx, profile_id=obj.profile_id)
         return profile.do_delete(obj, **params)
 
+    @classmethod
+    @profiler.trace('Profile.delete_cluster_object', hide_args=False)
+    def delete_cluster_object(cls, ctx, obj, **params):
+        profile = cls.load(ctx, profile_id=obj.profile_id)
+        try:
+            ret = profile.do_cluster_delete(obj, **params)
+        except NotImplementedError:
+            return None
+        return ret
+
     @classmethod
     @profiler.trace('Profile.update_object', hide_args=False)
     def update_object(cls, ctx, obj, new_profile_id=None, **params):
@@ -417,10 +437,18 @@ class Profile(object):
         """For subclass to override."""
         raise NotImplementedError
 
+    def do_cluster_create(self, obj):
+        """For subclass to override."""
+        raise NotImplementedError
+
     def do_delete(self, obj, **params):
         """For subclass to override."""
         raise NotImplementedError
 
+    def do_cluster_delete(self, obj):
+        """For subclass to override."""
+        raise NotImplementedError
+
     def do_update(self, obj, new_profile, **params):
         """For subclass to override."""
         LOG.warning("Update operation not supported.")
