@@ -849,7 +849,7 @@ class ServerProfile(base.Profile):
                 kwargs['scheduler_hints'] = hints
 
         server = None
-        resource_id = 'UNKNOWN'
+        resource_id = None
         try:
             server = self.compute(obj).server_create(**kwargs)
             self.compute(obj).wait_for_server(server.id)
@@ -873,10 +873,10 @@ class ServerProfile(base.Profile):
                   caught.
         :raises: `EResourceDeletion` if interaction with compute service fails.
         """
-        if not obj.physical_id:
+        server_id = obj.physical_id
+        if not server_id:
             return True
 
-        server_id = obj.physical_id
         ignore_missing = params.get('ignore_missing', True)
         internal_ports = obj.data.get('internal_ports', [])
         force = params.get('force', False)
@@ -1278,8 +1278,14 @@ class ServerProfile(base.Profile):
             image_id = server_data['image']['id']
         else:
             image_id = server_data['image']
+        attached_volumes = []
+        if ('attached_volumes' in server_data and
+                len(server_data['attached_volumes']) > 0):
+            for volume in server_data['attached_volumes']:
+                attached_volumes.append(volume['id'])
         details = {
             'image': image_id,
+            'attached_volumes': attached_volumes,
             'flavor': server_data['flavor']['id'],
         }
         for key in known_keys:
@@ -1425,9 +1431,14 @@ class ServerProfile(base.Profile):
         try:
             server = self.compute(obj).server_get(obj.physical_id)
         except exc.InternalError as ex:
-            raise exc.EResourceOperation(op='checking', type='server',
-                                         id=obj.physical_id,
-                                         message=six.text_type(ex))
+            if "No Server found" in six.text_type(ex):
+                raise exc.EServerNotFound(type='server',
+                                          id=obj.physical_id,
+                                          message=six.text_type(ex))
+            else:
+                raise exc.EResourceOperation(op='checking', type='server',
+                                             id=obj.physical_id,
+                                             message=six.text_type(ex))
 
         if (server is None or server.status != 'ACTIVE'):
             return False
@@ -1475,33 +1486,62 @@ class ServerProfile(base.Profile):
         return True
 
     def handle_rebuild(self, obj, **options):
+        """Handler for the rebuild operation.
+
+        :param obj: The node object.
+        :param dict options: A list for operations each of which has a name
+            and optionally a map from parameter to values.
+        :returns: The server ID if successful or None if failed.
+        """
         if not obj.physical_id:
-            return False
+            return None
 
         server_id = obj.physical_id
-        driver = self.compute(obj)
+        nova_driver = self.compute(obj)
         try:
-            server = driver.server_get(server_id)
+            server = nova_driver.server_get(server_id)
         except exc.InternalError as ex:
             raise exc.EResourceOperation(op='rebuilding', type='server',
                                          id=server_id,
                                          message=six.text_type(ex))
 
-        if server is None or server.image is None:
-            return False
+        if server is None:
+            return None
+        # when booting a nova server from volume, the image property
+        # can be ignored.
+        # we try to find a volume which is bootable and use its image_id
+        # for the server.
+        if server.image:
+            image_id = server.image
+        elif server.attached_volumes:
+            cinder_driver = self.block_storage(obj)
+            for volume_ids in server.attached_volumes:
+                try:
+                    vs = cinder_driver.volume_get(volume_ids['id'])
+                    if vs.is_bootable:
+                        image_id = vs.volume_image_metadata['image_id']
+                except exc.InternalError as ex:
+                    raise exc.EResourceOperation(op='rebuild', type='server',
+                                                 id=obj.physical_id,
+                                                 message=six.text_type(ex))
+        else:
+            msg = _("server doesn't have an image and it has no "
+                    "bootable volume")
+            raise exc.EResourceOperation(op="rebuild", type="server",
+                                         id=obj.physical_id,
+                                         message=msg)
 
-        image_id = server.image['id']
         admin_pass = self.properties.get(self.ADMIN_PASS)
+        name = self.properties[self.NAME] or obj.name
         try:
-            driver.server_rebuild(server_id, image_id,
-                                  self.properties.get(self.NAME),
-                                  admin_pass)
-            driver.wait_for_server(server_id, 'ACTIVE')
+            nova_driver.server_rebuild(server_id, image_id,
+                                       name, admin_pass)
+            nova_driver.wait_for_server(server_id, 'ACTIVE')
+            return server_id
         except exc.InternalError as ex:
             raise exc.EResourceOperation(op='rebuilding', type='server',
                                          id=server_id,
                                          message=six.text_type(ex))
-        return True
 
     def handle_change_password(self, obj, **options):
         """Handler for the change_password operation."""
