@@ -740,7 +740,8 @@ class NodeControllerTest(shared.ControllerTest, base.SenlinTestCase):
     def test_node_delete_success(self, mock_call, mock_parse, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'delete', True)
         nid = 'aaaa-bbbb-cccc'
-        req = self._delete('/nodes/%(node_id)s' % {'node_id': nid})
+        req = self._delete('/nodes/%(node_id)s' % {'node_id': nid},
+                           params={'force': False})
 
         obj = mock.Mock()
         mock_parse.return_value = obj
@@ -751,7 +752,8 @@ class NodeControllerTest(shared.ControllerTest, base.SenlinTestCase):
         result = {'location': '/actions/FAKE_ID'}
         self.assertEqual(res, result)
         mock_parse.assert_called_once_with(
-            'NodeDeleteRequest', req, {'identity': 'aaaa-bbbb-cccc'})
+            'NodeDeleteRequest', req,
+            {'identity': 'aaaa-bbbb-cccc', 'force': False})
         mock_call.assert_called_once_with(req.context, 'node_delete', obj)
 
     @mock.patch.object(util, 'parse_request')
@@ -760,7 +762,8 @@ class NodeControllerTest(shared.ControllerTest, base.SenlinTestCase):
                                                mock_parse, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'delete', True)
         nid = {'k1': 'v1'}
-        req = self._delete('/nodes/%(node_id)s' % {'node_id': nid})
+        req = self._delete('/nodes/%(node_id)s' % {'node_id': nid},
+                           params={'force': False})
 
         mock_parse.side_effect = exc.HTTPBadRequest("bad node")
         ex = self.assertRaises(exc.HTTPBadRequest,
@@ -769,12 +772,13 @@ class NodeControllerTest(shared.ControllerTest, base.SenlinTestCase):
         self.assertEqual("bad node", six.text_type(ex))
         self.assertFalse(mock_call.called)
         mock_parse.assert_called_once_with(
-            'NodeDeleteRequest', req, {'identity': nid})
+            'NodeDeleteRequest', req, {'identity': nid, 'force': False})
 
     def test_node_delete_err_denied_policy(self, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'delete', False)
         nid = 'aaaa-bbbb-cccc'
-        req = self._delete('/nodes/%(node_id)s' % {'node_id': nid})
+        req = self._delete('/nodes/%(node_id)s' % {'node_id': nid},
+                           params={'force': False})
 
         resp = shared.request_with_middleware(fault.FaultWrapper,
                                               self.controller.delete,
@@ -789,7 +793,8 @@ class NodeControllerTest(shared.ControllerTest, base.SenlinTestCase):
                                    mock_parse, mock_enforce):
         self._mock_enforce_setup(mock_enforce, 'delete', True)
         nid = 'aaaa-bbbb-cccc'
-        req = self._delete('/nodes/%(node_id)s' % {'node_id': nid})
+        req = self._delete('/nodes/%(node_id)s' % {'node_id': nid},
+                           params={'force': False})
 
         error = senlin_exc.ResourceNotFound(type='node', id=nid)
         mock_call.side_effect = shared.to_remote_error(error)
@@ -801,6 +806,27 @@ class NodeControllerTest(shared.ControllerTest, base.SenlinTestCase):
         self.assertEqual(404, resp.json['code'])
         self.assertEqual('ResourceNotFound', resp.json['error']['type'])
 
+    @mock.patch.object(util, 'parse_request')
+    @mock.patch.object(rpc_client.EngineClient, 'call')
+    def test_node_delete_force(self, mock_call, mock_parse, mock_enforce):
+        self._mock_enforce_setup(mock_enforce, 'delete', True)
+        nid = 'aaaa-bbbb-cccc'
+        req = self._delete('/nodes/%(node_id)s' % {'node_id': nid},
+                           params={'force': 'true'})
+
+        obj = mock.Mock()
+        mock_parse.return_value = obj
+        mock_call.return_value = {'action': 'FAKE_ID'}
+        params = {'node_id': nid, 'body': {'force': True}}
+        res = self.controller.delete(req, **params)
+
+        result = {'location': '/actions/FAKE_ID'}
+        self.assertEqual(res, result)
+        mock_parse.assert_called_once_with(
+            'NodeDeleteRequest', req,
+            {'identity': 'aaaa-bbbb-cccc', 'force': True})
+        mock_call.assert_called_once_with(req.context, 'node_delete', obj)
+
     @mock.patch.object(util, 'parse_request')
     @mock.patch.object(rpc_client.EngineClient, 'call')
     def test_node_action_check_success(self, mock_call,
