@@ -10,10 +10,10 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import six
 import time
 
-import six
-
+from oslo_utils import timeutils as tu
 from senlin.common import consts
 from senlin.common import exception
 from senlin.db.sqlalchemy import api as db_api
@@ -25,9 +25,9 @@ from senlin.tests.unit.db import shared
 
 def _create_action(context, action_json=shared.sample_action, **kwargs):
     data = parser.simple_parse(action_json)
-    data['user'] = context.user
-    data['project'] = context.project
-    data['domain'] = context.domain
+    data['user'] = context.user_id
+    data['project'] = context.project_id
+    data['domain'] = context.domain_id
     data.update(kwargs)
     return db_api.action_create(context, data)
 
@@ -50,9 +50,9 @@ class DBAPIActionTest(base.SenlinTestCase):
         self.assertEqual(data['status'], action.status)
         self.assertEqual(data['status_reason'], action.status_reason)
         self.assertEqual(10, action.inputs['max_size'])
-        self.assertEqual(self.ctx.user, action.user)
-        self.assertEqual(self.ctx.project, action.project)
-        self.assertEqual(self.ctx.domain, action.domain)
+        self.assertEqual(self.ctx.user_id, action.user)
+        self.assertEqual(self.ctx.project_id, action.project)
+        self.assertEqual(self.ctx.domain_id, action.domain)
         self.assertIsNone(action.outputs)
 
     def test_action_update(self):
@@ -156,6 +156,37 @@ class DBAPIActionTest(base.SenlinTestCase):
         retobj = db_api.action_get(new_ctx, action.id, project_safe=False)
         self.assertIsNotNone(retobj)
 
+    def test_acquire_first_ready_none(self):
+        data = {'created_at': tu.utcnow(True)}
+
+        _create_action(self.ctx, **data)
+        result = db_api.action_acquire_first_ready(self.ctx, 'fake_o',
+                                                   tu.utcnow(True))
+        self.assertIsNone(result)
+
+    def test_acquire_first_ready_one(self):
+        data = {'created_at': tu.utcnow(True)}
+        _create_action(self.ctx, **data)
+
+        result = db_api.action_acquire_first_ready(self.ctx, 'fake_o',
+                                                   tu.utcnow(True))
+        self.assertIsNone(result)
+
+    def test_acquire_first_ready_mult(self):
+        data = {
+            'created_at': tu.utcnow(True),
+            'status': 'READY',
+        }
+        action1 = _create_action(self.ctx, **data)
+        time.sleep(1)
+
+        data['created_at'] = tu.utcnow(True)
+        _create_action(self.ctx, **data)
+
+        result = db_api.action_acquire_first_ready(self.ctx, 'fake_o',
+                                                   time.time())
+        self.assertEqual(action1.id, result.id)
+
     def test_action_acquire_random_ready(self):
         specs = [
             {'name': 'A01', 'status': 'INIT'},
@@ -171,6 +202,7 @@ class DBAPIActionTest(base.SenlinTestCase):
         timestamp = time.time()
         action = db_api.action_acquire_random_ready(self.ctx, worker,
                                                     timestamp)
+
         self.assertIn(action.name, ('A02', 'A04'))
         self.assertEqual('worker2', action.owner)
         self.assertEqual(consts.ACTION_RUNNING, action.status)
@@ -409,6 +441,56 @@ class DBAPIActionTest(base.SenlinTestCase):
 
         return id_of
 
+    def test_engine_mark_failed_with_depended(self):
+        timestamp = time.time()
+        id_of = self._prepare_action_mark_failed_cancel()
+        with db_api.session_for_write() as session:
+            db_api._mark_engine_failed(session, id_of['A01'],
+                                       timestamp, 'BOOM')
+        for aid in [id_of['A02'], id_of['A03'], id_of['A04']]:
+            action = db_api.action_get(self.ctx, aid)
+            self.assertEqual(consts.ACTION_FAILED, action.status)
+            self.assertEqual('BOOM', action.status_reason)
+            self.assertEqual(timestamp, action.end_time)
+
+        action = db_api.action_get(self.ctx, id_of['A01'])
+        self.assertEqual(consts.ACTION_FAILED, action.status)
+        self.assertEqual('BOOM', action.status_reason)
+        self.assertEqual(timestamp, action.end_time)
+
+        for aid in [id_of['A02'], id_of['A03'], id_of['A04']]:
+            result = db_api.dependency_get_dependents(self.ctx, aid)
+            self.assertEqual(0, len(result))
+
+    def test_engine_mark_failed_without_depended(self):
+        timestamp = time.time()
+        id_of = self._prepare_action_mark_failed_cancel()
+        with db_api.session_for_write() as session:
+            db_api._mark_engine_failed(session, id_of['A02'],
+                                       timestamp, 'BOOM')
+
+        for aid in [id_of['A03'], id_of['A04']]:
+            action = db_api.action_get(self.ctx, aid)
+            self.assertEqual(consts.ACTION_INIT, action.status)
+            self.assertNotEqual('BOOM', action.status_reason)
+            self.assertNotEqual(timestamp, action.end_time)
+
+        action = db_api.action_get(self.ctx, id_of['A01'])
+        self.assertEqual(consts.ACTION_WAITING, action.status)
+        self.assertNotEqual('BOOM', action.status_reason)
+        self.assertNotEqual(timestamp, action.end_time)
+
+        action_d = db_api.action_get(self.ctx, id_of['A02'])
+        self.assertEqual(consts.ACTION_FAILED, action_d.status)
+        self.assertEqual('BOOM', action_d.status_reason)
+        self.assertEqual(timestamp, action_d.end_time)
+
+        for aid in [id_of['A03'], id_of['A04']]:
+            result = db_api.dependency_get_dependents(self.ctx, aid)
+            self.assertEqual(1, len(result))
+        result = db_api.dependency_get_dependents(self.ctx, id_of['A02'])
+        self.assertEqual(0, len(result))
+
     def test_action_mark_failed(self):
         timestamp = time.time()
         id_of = self._prepare_action_mark_failed_cancel()
@@ -435,6 +517,30 @@ class DBAPIActionTest(base.SenlinTestCase):
         result = db_api.dependency_get_dependents(self.ctx, id_of['A01'])
         self.assertEqual(0, len(result))
 
+    def test_action_mark_ready(self):
+        timestamp = time.time()
+
+        specs = [
+            {'name': 'A01', 'status': 'INIT', 'target': 'cluster_001'},
+            {'name': 'A02', 'status': 'INIT', 'target': 'node_001'},
+            {'name': 'A03', 'status': 'INIT', 'target': 'node_002'},
+            {'name': 'A04', 'status': 'INIT', 'target': 'node_003'},
+            {'name': 'A05', 'status': 'INIT', 'target': 'cluster_002'},
+            {'name': 'A06', 'status': 'INIT', 'target': 'cluster_003'},
+            {'name': 'A07', 'status': 'INIT', 'target': 'cluster_004'},
+        ]
+
+        id_of = {}
+        for spec in specs:
+            action = _create_action(self.ctx, **spec)
+            id_of[spec['name']] = action.id
+
+        db_api.action_mark_ready(self.ctx, id_of['A01'], timestamp)
+
+        action = db_api.action_get(self.ctx, id_of['A01'])
+        self.assertEqual(consts.ACTION_READY, action.status)
+        self.assertEqual(timestamp, action.end_time)
+
     def test_action_acquire(self):
         action = _create_action(self.ctx)
         db_api.action_update(self.ctx, action.id, {'status': 'READY'})
@@ -558,8 +664,9 @@ class DBAPIActionTest(base.SenlinTestCase):
         before_abandon = db_api.action_get(self.ctx, action.id)
         self.assertEqual(spec['owner'], before_abandon.owner)
         self.assertEqual(spec['start_time'], before_abandon.start_time)
+        self.assertIsNone(before_abandon.data)
 
-        db_api.action_abandon(self.ctx, action.id)
+        db_api.action_abandon(self.ctx, action.id, {})
         after_abandon = db_api.action_get(self.ctx, action.id)
 
         self.assertIsNone(after_abandon.owner)
@@ -567,3 +674,27 @@ class DBAPIActionTest(base.SenlinTestCase):
         self.assertEqual('The action was abandoned.',
                          after_abandon.status_reason)
         self.assertEqual(consts.ACTION_READY, after_abandon.status)
+        self.assertIsNone(after_abandon.data)
+
+    def test_action_abandon_with_params(self):
+        spec = {
+            "owner": "test_owner",
+            "start_time": 14506893904.0
+        }
+        action = _create_action(self.ctx, **spec)
+
+        before_abandon = db_api.action_get(self.ctx, action.id)
+        self.assertEqual(spec['owner'], before_abandon.owner)
+        self.assertEqual(spec['start_time'], before_abandon.start_time)
+        self.assertIsNone(before_abandon.data)
+
+        db_api.action_abandon(self.ctx, action.id,
+                              {'data': {'retries': 1}})
+        after_abandon = db_api.action_get(self.ctx, action.id)
+
+        self.assertIsNone(after_abandon.owner)
+        self.assertIsNone(after_abandon.start_time)
+        self.assertEqual('The action was abandoned.',
+                         after_abandon.status_reason)
+        self.assertEqual(consts.ACTION_READY, after_abandon.status)
+        self.assertEqual({'retries': 1}, after_abandon.data)
