@@ -13,8 +13,8 @@
 import mock
 from oslo_config import cfg
 
-from senlin.drivers.openstack import heat_v1
-from senlin.drivers.openstack import sdk
+from senlin.drivers.os import heat_v1
+from senlin.drivers import sdk
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
