@@ -15,8 +15,8 @@ import copy
 import mock
 from oslo_config import cfg
 
-from senlin.drivers.openstack import keystone_v3 as kv3
-from senlin.drivers.openstack import sdk
+from senlin.drivers.os import keystone_v3 as kv3
+from senlin.drivers import sdk
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
@@ -38,101 +38,6 @@ class TestKeystoneV3(base.SenlinTestCase):
         self.assertEqual(self.conn, kc.conn)
         self.assertEqual(self.conn.session, kc.session)
 
-    def test_trust_list(self, mock_create):
-        self.conn.identity.trusts.return_value = ('foo', 'bar')
-        mock_create.return_value = self.conn
-        kc = kv3.KeystoneClient({'k': 'v'})
-
-        res = kc.trust_list(p1='v1', p2='v2')
-
-        self.assertEqual(['foo', 'bar'], res)
-        self.conn.identity.trusts.assert_called_once_with(p1='v1', p2='v2')
-
-    def test_trust_delete(self, mock_create):
-        mock_create.return_value = self.conn
-        kc = kv3.KeystoneClient({'k': 'v'})
-
-        res = kc.trust_delete('value')
-
-        self.assertEqual(None, res)
-        self.conn.identity.delete_trust.assert_called_once_with(
-            'value', ignore_missing=True)
-
-    def test_user_get(self, mock_create):
-        self.conn.identity.find_user.return_value = 'user1'
-        mock_create.return_value = self.conn
-        kc = kv3.KeystoneClient({'k': 'v'})
-
-        res = kc.user_get('user_id_or_name')
-
-        self.assertEqual('user1', res)
-        self.conn.identity.find_user.assert_called_once_with(
-            'user_id_or_name', ignore_missing=False)
-
-    def test_endpoint_get(self, mock_create):
-        self.conn.identity.endpoints.return_value = ['fake_endpoint']
-        mock_create.return_value = self.conn
-        kc = kv3.KeystoneClient({'k': 'v'})
-
-        res = kc.endpoint_get('FAKE_ID')
-
-        self.assertEqual('fake_endpoint', res)
-        self.conn.identity.endpoints.assert_called_once_with(
-            service_id='FAKE_ID')
-        self.conn.reset_mock()
-
-        # with region
-        res = kc.endpoint_get('FAKE_ID', 'FAKE_REGION')
-
-        self.assertEqual('fake_endpoint', res)
-        self.conn.identity.endpoints.assert_called_once_with(
-            service_id='FAKE_ID', region='FAKE_REGION')
-        self.conn.reset_mock()
-
-        # with interface
-        res = kc.endpoint_get('FAKE_ID', None, 'public')
-
-        self.assertEqual('fake_endpoint', res)
-        self.conn.identity.endpoints.assert_called_once_with(
-            service_id='FAKE_ID', interface='public')
-        self.conn.reset_mock()
-
-        # returning None
-        self.conn.identity.endpoints.return_value = []
-
-        res = kc.endpoint_get('FAKE_ID')
-
-        self.assertEqual(None, res)
-        self.conn.identity.endpoints.assert_called_once_with(
-            service_id='FAKE_ID')
-
-    def test_service_get(self, mock_create):
-        self.conn.identity.services.return_value = ['fake_service']
-        mock_create.return_value = self.conn
-        kc = kv3.KeystoneClient({'k': 'v'})
-
-        res = kc.service_get('FAKE_TYPE')
-
-        self.assertEqual('fake_service', res)
-        self.conn.identity.services.assert_called_once_with(type='FAKE_TYPE')
-        self.conn.reset_mock()
-
-        # with name
-        res = kc.service_get('FAKE_TYPE', 'FAKE_NAME')
-
-        self.assertEqual('fake_service', res)
-        self.conn.identity.services.assert_called_once_with(type='FAKE_TYPE',
-                                                            name='FAKE_NAME')
-        self.conn.reset_mock()
-
-        # returning none
-        self.conn.identity.services.return_value = []
-
-        res = kc.service_get('FAKE_TYPE')
-
-        self.assertEqual(None, res)
-        self.conn.identity.services.assert_called_once_with(type='FAKE_TYPE')
-
     def test_trust_get_by_trustor(self, mock_create):
         trust1 = mock.Mock()
         trust1.trustee_user_id = 'USER_A_ID'
@@ -150,7 +55,7 @@ class TestKeystoneV3(base.SenlinTestCase):
         mock_create.return_value = self.conn
         kc = kv3.KeystoneClient({'k': 'v'})
 
-        # no trustee/projec filter, matching 1st
+        # no trustee/project filter, matching 1st
         res = kc.trust_get_by_trustor('USER_A')
         self.assertEqual(trust1, res)
 
@@ -168,7 +73,7 @@ class TestKeystoneV3(base.SenlinTestCase):
 
         # No matching record found
         res = kc.trust_get_by_trustor('USER_A', 'USER_C_ID')
-        self.assertEqual(None, res)
+        self.assertIsNone(res)
 
         get_calls = [mock.call(trustor_user_id='USER_A')]
         self.conn.identity.trusts.assert_has_calls(get_calls * 5)
@@ -213,27 +118,26 @@ class TestKeystoneV3(base.SenlinTestCase):
 
     @mock.patch.object(sdk, 'authenticate')
     def test_get_token(self, mock_auth, mock_create):
-        access_info = mock.Mock()
+        access_info = {'token': '123', 'user_id': 'abc', 'project_id': 'xyz'}
         mock_auth.return_value = access_info
 
         token = kv3.KeystoneClient.get_token(key='value')
 
         mock_auth.assert_called_once_with(key='value')
-        self.assertEqual(access_info.auth_token, token)
+        self.assertEqual('123', token)
 
     @mock.patch.object(sdk, 'authenticate')
     def test_get_user_id(self, mock_auth, mock_create):
-        access_info = mock.Mock()
+        access_info = {'token': '123', 'user_id': 'abc', 'project_id': 'xyz'}
         mock_auth.return_value = access_info
 
         user_id = kv3.KeystoneClient.get_user_id(key='value')
 
         mock_auth.assert_called_once_with(key='value')
-        self.assertEqual(access_info.user_id, user_id)
+        self.assertEqual('abc', user_id)
 
     def test_get_service_credentials(self, mock_create):
-        cfg.CONF.set_override('auth_url', 'FAKE_URL',
-                              group='authentication')
+        cfg.CONF.set_override('auth_url', 'FAKE_URL', group='authentication')
         cfg.CONF.set_override('service_username', 'FAKE_USERNAME',
                               group='authentication')
         cfg.CONF.set_override('service_password', 'FAKE_PASSWORD',
@@ -265,3 +169,33 @@ class TestKeystoneV3(base.SenlinTestCase):
             key1='value1', password='NEW_PASSWORD')
 
         self.assertEqual(new_expected, actual)
+
+    def test_validate_regions(self, mock_create):
+        self.conn.identity.regions.return_value = [
+            {'id': 'R1', 'parent_region_id': None},
+            {'id': 'R2', 'parent_region_id': None},
+            {'id': 'R3', 'parent_region_id': 'R1'},
+        ]
+        mock_create.return_value = self.conn
+
+        kc = kv3.KeystoneClient({'k': 'v'})
+
+        res = kc.validate_regions(['R1', 'R4'])
+
+        self.assertIn('R1', res)
+        self.assertNotIn('R4', res)
+
+        res = kc.validate_regions([])
+        self.assertEqual([], res)
+
+    def test_get_senlin_endpoint(self, mock_create):
+        cfg.CONF.set_override('default_region_name', 'RegionN')
+        self.conn.session.get_endpoint.return_value = 'http://web.com:1234/v1'
+        mock_create.return_value = self.conn
+        kc = kv3.KeystoneClient({'k': 'v'})
+
+        res = kc.get_senlin_endpoint()
+
+        self.assertEqual('http://web.com:1234/v1', res)
+        self.conn.session.get_endpoint.assert_called_once_with(
+            service_type='clustering', interface='public', region='RegionN')
