@@ -10,49 +10,81 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import types
+
 import mock
 from openstack import connection
-from openstack import profile
-from openstack import transport
 from oslo_serialization import jsonutils
 from requests import exceptions as req_exc
 import six
 
 from senlin.common import exception as senlin_exc
-from senlin.drivers.openstack import sdk
+from senlin.drivers import sdk
 from senlin.tests.unit.common import base
+from senlin import version
 
 
 class OpenStackSDKTest(base.SenlinTestCase):
 
     def setUp(self):
         super(OpenStackSDKTest, self).setUp()
+        self.app_version = version.version_info.version_string()
 
-    def test_parser_exception_http_exception_with_details(self):
+    def test_parse_exception_http_exception_with_details(self):
         details = jsonutils.dumps({
             'error': {
                 'code': 404,
                 'message': 'Resource BAR is not found.'
             }
         })
-        raw = sdk.exc.ResourceNotFound('A message', details, 404)
+        raw = sdk.exc.ResourceNotFound(message='A message', details=details,
+                                       response=None, http_status=404)
         ex = self.assertRaises(senlin_exc.InternalError,
                                sdk.parse_exception, raw)
 
         self.assertEqual(404, ex.code)
         self.assertEqual('Resource BAR is not found.', six.text_type(ex))
+        # key name is not 'error' case
+        details = jsonutils.dumps({
+            'forbidden': {
+                'code': 403,
+                'message': 'Quota exceeded for instances.'
+            }
+        })
+        raw = sdk.exc.ResourceNotFound(message='A message', details=details,
+                                       http_status=403)
+        ex = self.assertRaises(senlin_exc.InternalError,
+                               sdk.parse_exception, raw)
 
-    def test_parser_exception_http_exception_no_details(self):
-        details = "An error message"
+        self.assertEqual(403, ex.code)
+        self.assertEqual('Quota exceeded for instances.', six.text_type(ex))
 
-        raw = sdk.exc.ResourceNotFound('A message.', details, 404)
+    def test_parse_exception_http_exception_no_details(self):
+        resp = mock.Mock(headers={'x-openstack-request-id': 'FAKE_ID'})
+        resp.json.return_value = {}
+        resp.status_code = 404
+
+        raw = sdk.exc.ResourceNotFound(message='A message.', details=None,
+                                       response=resp, http_status=404)
         ex = self.assertRaises(senlin_exc.InternalError,
                                sdk.parse_exception, raw)
 
         self.assertEqual(404, ex.code)
         self.assertEqual('A message.', six.text_type(ex))
 
-    def test_parser_exception_http_exception_code_displaced(self):
+    def test_parse_exception_http_exception_no_details_no_response(self):
+        details = "An error message"
+
+        raw = sdk.exc.ResourceNotFound(message='A message.', details=details,
+                                       http_status=404)
+        raw.details = None
+        raw.response = None
+        ex = self.assertRaises(senlin_exc.InternalError,
+                               sdk.parse_exception, raw)
+        self.assertEqual(404, ex.code)
+        self.assertEqual('A message.', six.text_type(ex))
+
+    def test_parse_exception_http_exception_code_displaced(self):
         details = jsonutils.dumps({
             'code': 400,
             'error': {
@@ -60,23 +92,24 @@ class OpenStackSDKTest(base.SenlinTestCase):
             }
         })
 
-        raw = sdk.exc.HttpException('A message.', details)
+        raw = sdk.exc.HttpException(
+            message='A message.', details=details, http_status=400)
         ex = self.assertRaises(senlin_exc.InternalError,
                                sdk.parse_exception, raw)
 
         self.assertEqual(400, ex.code)
         self.assertEqual('Resource BAR is in error state.', six.text_type(ex))
 
-    def test_parser_exception_sdk_exception(self):
-        raw = sdk.exc.AuthorizationFailure()
+    def test_parse_exception_sdk_exception(self):
+        raw = sdk.exc.InvalidResponse('INVALID')
 
         ex = self.assertRaises(senlin_exc.InternalError,
                                sdk.parse_exception, raw)
 
         self.assertEqual(500, ex.code)
-        self.assertEqual('AuthorizationFailure', six.text_type(ex))
+        self.assertEqual('InvalidResponse', six.text_type(ex))
 
-    def test_parser_exception_request_exception(self):
+    def test_parse_exception_request_exception(self):
         raw = req_exc.HTTPError(401, 'ERROR')
 
         ex = self.assertRaises(senlin_exc.InternalError,
@@ -85,7 +118,7 @@ class OpenStackSDKTest(base.SenlinTestCase):
         self.assertEqual(401, ex.code)
         self.assertEqual('[Errno 401] ERROR', ex.message)
 
-    def test_parser_exception_other_exceptions(self):
+    def test_parse_exception_other_exceptions(self):
         raw = Exception('Unknown Error')
 
         ex = self.assertRaises(senlin_exc.InternalError,
@@ -96,11 +129,12 @@ class OpenStackSDKTest(base.SenlinTestCase):
 
     def test_translate_exception_wrapper(self):
 
-        test_func = mock.Mock()
-        test_func.__name__ = 'test_func'
+        @sdk.translate_exception
+        def test_func(driver):
+            return driver.__name__
 
         res = sdk.translate_exception(test_func)
-        self.assertEqual('function', res.__class__.__name__)
+        self.assertEqual(types.FunctionType, type(res))
 
     def test_translate_exception_with_exception(self):
 
@@ -116,46 +150,58 @@ class OpenStackSDKTest(base.SenlinTestCase):
         self.assertEqual(500, ex.code)
         self.assertEqual('BOOM', ex.message)
 
-    @mock.patch.object(profile, 'Profile')
     @mock.patch.object(connection, 'Connection')
-    def test_create_connection(self, mock_conn, mock_profile):
-        x_profile = mock.Mock()
-        mock_profile.return_value = x_profile
+    def test_create_connection_token(self, mock_conn):
         x_conn = mock.Mock()
         mock_conn.return_value = x_conn
 
-        res = sdk.create_connection({'foo': 'bar'})
+        res = sdk.create_connection({'token': 'TOKEN', 'foo': 'bar'})
 
         self.assertEqual(x_conn, res)
-        mock_profile.assert_called_once_with()
-        mock_conn.assert_called_once_with(profile=x_profile,
-                                          user_agent=sdk.USER_AGENT,
-                                          foo='bar')
+        mock_conn.assert_called_once_with(
+            app_name=sdk.USER_AGENT, app_version=self.app_version,
+            identity_api_version='3',
+            messaging_api_version='2',
+            region_name=None,
+            auth_type='token',
+            token='TOKEN',
+            foo='bar')
 
-    @mock.patch.object(profile, 'Profile')
     @mock.patch.object(connection, 'Connection')
-    def test_create_connection_with_region(self, mock_conn, mock_profile):
-        x_profile = mock.Mock()
-        mock_profile.return_value = x_profile
+    def test_create_connection_password(self, mock_conn):
+        x_conn = mock.Mock()
+        mock_conn.return_value = x_conn
+
+        res = sdk.create_connection({'user_id': '123', 'password': 'abc',
+                                     'foo': 'bar'})
+
+        self.assertEqual(x_conn, res)
+        mock_conn.assert_called_once_with(
+            app_name=sdk.USER_AGENT, app_version=self.app_version,
+            identity_api_version='3',
+            messaging_api_version='2',
+            region_name=None,
+            user_id='123',
+            password='abc',
+            foo='bar')
+
+    @mock.patch.object(connection, 'Connection')
+    def test_create_connection_with_region(self, mock_conn):
         x_conn = mock.Mock()
         mock_conn.return_value = x_conn
 
         res = sdk.create_connection({'region_name': 'REGION_ONE'})
 
         self.assertEqual(x_conn, res)
-        mock_profile.assert_called_once_with()
-        x_profile.set_region.assert_called_once_with(x_profile.ALL,
-                                                     'REGION_ONE')
-        mock_conn.assert_called_once_with(profile=x_profile,
-                                          user_agent=sdk.USER_AGENT)
+        mock_conn.assert_called_once_with(
+            app_name=sdk.USER_AGENT, app_version=self.app_version,
+            identity_api_version='3',
+            messaging_api_version='2',
+            region_name='REGION_ONE')
 
-    @mock.patch.object(profile, 'Profile')
     @mock.patch.object(connection, 'Connection')
     @mock.patch.object(sdk, 'parse_exception')
-    def test_create_connection_with_exception(self, mock_parse, mock_conn,
-                                              mock_profile):
-        x_profile = mock.Mock()
-        mock_profile.return_value = x_profile
+    def test_create_connection_with_exception(self, mock_parse, mock_conn):
         ex_raw = Exception('Whatever')
         mock_conn.side_effect = ex_raw
         mock_parse.side_effect = senlin_exc.InternalError(code=123,
@@ -164,49 +210,28 @@ class OpenStackSDKTest(base.SenlinTestCase):
         ex = self.assertRaises(senlin_exc.InternalError,
                                sdk.create_connection)
 
-        mock_profile.assert_called_once_with()
-        mock_conn.assert_called_once_with(profile=x_profile,
-                                          user_agent=sdk.USER_AGENT)
+        mock_conn.assert_called_once_with(
+            app_name=sdk.USER_AGENT, app_version=self.app_version,
+            identity_api_version='3',
+            messaging_api_version='2',
+            region_name=None)
         mock_parse.assert_called_once_with(ex_raw)
         self.assertEqual(123, ex.code)
         self.assertEqual('BOOM', ex.message)
 
     @mock.patch.object(sdk, 'create_connection')
-    @mock.patch.object(transport, 'Transport')
-    def test_authenticate(self, mock_transport, mock_conn):
+    def test_authenticate(self, mock_conn):
         x_conn = mock_conn.return_value
-        x_auth = x_conn.session.authenticator
-        x_transport = mock_transport.return_value
-        x_access_info = mock.Mock()
-        mock_authorize = mock.Mock(return_value=x_access_info)
-        x_auth.authorize = mock_authorize
+        x_conn.session.get_token.return_value = 'TOKEN'
+        x_conn.session.get_user_id.return_value = 'test-user-id'
+        x_conn.session.get_project_id.return_value = 'test-project-id'
+        access_info = {
+            'token': 'TOKEN',
+            'user_id': 'test-user-id',
+            'project_id': 'test-project-id'
+        }
 
         res = sdk.authenticate(foo='bar')
 
-        self.assertEqual(x_access_info, res)
+        self.assertEqual(access_info, res)
         mock_conn.assert_called_once_with({'foo': 'bar'})
-        mock_transport.assert_called_once_with()
-        mock_authorize.assert_called_once_with(x_transport)
-
-    @mock.patch.object(sdk, 'create_connection')
-    @mock.patch.object(transport, 'Transport')
-    @mock.patch.object(sdk, 'parse_exception')
-    def test_authenticate_with_exception(self, mock_parse, mock_transport,
-                                         mock_conn):
-        x_conn = mock_conn.return_value
-        x_auth = x_conn.session.authenticator
-        x_transport = mock_transport.return_value
-        error = Exception('test exception')
-        x_auth.authorize.side_effect = error
-        mock_parse.side_effect = senlin_exc.InternalError(code=123,
-                                                          message='BOOM')
-
-        ex = self.assertRaises(senlin_exc.InternalError,
-                               sdk.authenticate, foo='bar')
-
-        mock_conn.assert_called_once_with({'foo': 'bar'})
-        mock_transport.assert_called_once_with()
-        x_auth.authorize.assert_called_once_with(x_transport)
-        mock_parse.assert_called_once_with(error)
-        self.assertEqual(123, ex.code)
-        self.assertEqual('BOOM', ex.message)
