@@ -11,6 +11,7 @@
 # under the License.
 
 import copy
+import eventlet
 
 import mock
 from oslo_config import cfg
@@ -23,6 +24,7 @@ from senlin.common import exception
 from senlin.common import utils as common_utils
 from senlin.engine.actions import base as ab
 from senlin.engine import cluster as cluster_mod
+from senlin.engine import dispatcher
 from senlin.engine import environment
 from senlin.engine import event as EVENT
 from senlin.engine import node as node_mod
@@ -69,8 +71,6 @@ class ActionBaseTest(base.SenlinTestCase):
             'created_at': timeutils.utcnow(True),
             'updated_at': None,
             'data': {'data_key': 'data_value'},
-            'user': USER_ID,
-            'project': PROJECT_ID,
         }
 
     def _verify_new_action(self, obj, target, action):
@@ -348,9 +348,14 @@ class ActionBaseTest(base.SenlinTestCase):
     @mock.patch.object(ao.Action, 'mark_succeeded')
     @mock.patch.object(ao.Action, 'mark_failed')
     @mock.patch.object(ao.Action, 'mark_cancelled')
+    @mock.patch.object(ao.Action, 'mark_ready')
     @mock.patch.object(ao.Action, 'abandon')
-    def test_set_status(self, mock_abandon, mark_cancel, mark_fail,
-                        mark_succeed, mock_event, mock_error, mock_info):
+    @mock.patch.object(dispatcher, 'start_action')
+    @mock.patch.object(eventlet, 'sleep')
+    def test_set_status(self, mock_sleep, mock_start, mock_abandon,
+                        mark_ready, mark_cancel, mark_fail,
+                        mark_succeed, mock_event, mock_error,
+                        mock_info):
         action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx, id='FAKE_ID')
         action.entity = mock.Mock()
 
@@ -381,11 +386,27 @@ class ActionBaseTest(base.SenlinTestCase):
         mark_cancel.assert_called_once_with(action.context, 'FAKE_ID',
                                             mock.ANY)
 
+        mark_fail.reset_mock()
+        action.set_status(action.RES_LIFECYCLE_COMPLETE, 'LIFECYCLE COMPLETE')
+        self.assertEqual(action.SUCCEEDED, action.status)
+        self.assertEqual('LIFECYCLE COMPLETE', action.status_reason)
+        mark_ready.assert_called_once_with(action.context, 'FAKE_ID', mock.ANY)
+
         mark_fail.reset_mock()
         action.set_status(action.RES_RETRY, 'BUSY')
         self.assertEqual(action.READY, action.status)
         self.assertEqual('BUSY', action.status_reason)
-        mock_abandon.assert_called_once_with(action.context, 'FAKE_ID')
+        mock_start.assert_called_once_with(action.id)
+        mock_sleep.assert_called_once_with(10)
+        mock_abandon.assert_called_once_with(
+            action.context, 'FAKE_ID', {'data': {'retries': 1}})
+
+        mark_fail.reset_mock()
+        action.data = {'retries': 3}
+        action.set_status(action.RES_RETRY, 'BUSY')
+        self.assertEqual(action.RES_ERROR, action.status)
+        mark_fail.assert_called_once_with(action.context, 'FAKE_ID', mock.ANY,
+                                          'BUSY')
 
     @mock.patch.object(EVENT, 'info')
     @mock.patch.object(EVENT, 'error')
@@ -615,8 +636,8 @@ class ActionPolicyCheckTest(base.SenlinTestCase):
 
     def _create_policy(self):
         values = {
-            'user': self.ctx.user,
-            'project': self.ctx.project,
+            'user': self.ctx.user_id,
+            'project': self.ctx.project_id,
         }
         policy = fakes.TestPolicy('DummyPolicy', 'test-policy', **values)
         policy.store(self.ctx)
@@ -849,7 +870,7 @@ class ActionProcTest(base.SenlinTestCase):
         self.assertTrue(res)
         mock_load.assert_called_once_with(self.ctx, action_id='ACTION_ID',
                                           project_safe=False)
-        mock_event_info.assert_called_once_with(action, 'start')
+        mock_event_info.assert_called_once_with(action, 'start', 'ACTION_I')
         mock_status.assert_called_once_with(action.RES_OK, 'BIG SUCCESS')
 
     @mock.patch.object(EVENT, 'info')
@@ -868,5 +889,5 @@ class ActionProcTest(base.SenlinTestCase):
         self.assertFalse(res)
         mock_load.assert_called_once_with(self.ctx, action_id='ACTION',
                                           project_safe=False)
-        mock_info.assert_called_once_with(action, 'start')
+        mock_info.assert_called_once_with(action, 'start', 'ACTION')
         mock_status.assert_called_once_with(action.RES_ERROR, 'Boom!')
