@@ -12,14 +12,21 @@
 
 import mock
 
+from senlin.common import consts
+from senlin.common import exception as exc
 from senlin.engine.actions import base as ab
 from senlin.engine.actions import cluster_action as ca
 from senlin.engine import cluster as cm
+from senlin.engine import dispatcher
 from senlin.engine import senlin_lock
 from senlin.policies import base as pb
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
+CLUSTER_ID = 'e1cfd82b-dc95-46ad-86e8-37864d7be1cd'
+OBJID = '571fffb8-f41c-4cbc-945c-cb2937d76f19'
+ACTION_ID = '4c2cead2-fd74-418a-9d12-bd2d9bd7a812'
+
 
 @mock.patch.object(cm.Cluster, 'load')
 class ClusterActionTest(base.SenlinTestCase):
@@ -178,3 +185,62 @@ class ClusterActionTest(base.SenlinTestCase):
         action = ca.ClusterAction('ID', 'CLUSTER_DELETE', self.ctx)
         res = action.cancel()
         self.assertEqual(action.RES_OK, res)
+
+
+class CompleteLifecycleProcTest(base.SenlinTestCase):
+
+    def setUp(self):
+        super(CompleteLifecycleProcTest, self).setUp()
+
+        self.ctx = utils.dummy_context()
+
+    @mock.patch.object(dispatcher, 'start_action')
+    @mock.patch.object(ab.Action, 'load')
+    def test_complete_lifecycle_proc_successful(self, mock_load,
+                                                mock_dispatcher_start):
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
+        mock_obj = mock.Mock()
+        action.entity = mock_obj
+        mock_get_status = self.patchobject(action, 'get_status')
+        mock_get_status.return_value = \
+            consts.ACTION_WAITING_LIFECYCLE_COMPLETION
+        mock_set_status = self.patchobject(action, 'set_status')
+        mock_load.return_value = action
+
+        res = ca.CompleteLifecycleProc(self.ctx, 'ACTION_ID')
+
+        self.assertTrue(res)
+        mock_load.assert_called_once_with(self.ctx, action_id='ACTION_ID',
+                                          project_safe=False)
+        mock_get_status.assert_called_once_with()
+        mock_set_status.assert_called_once_with(action.RES_LIFECYCLE_COMPLETE)
+        mock_dispatcher_start.assert_called_once_with()
+
+    @mock.patch.object(ab.Action, 'load')
+    def test_complete_lifecycle_proc_failed_action_not_found(self, mock_load):
+        mock_load.return_value = None
+
+        self.assertRaises(exc.ResourceNotFound,
+                          ca.CompleteLifecycleProc,
+                          self.ctx, 'ACTION')
+
+    @mock.patch.object(dispatcher, 'start_action')
+    @mock.patch.object(ab.Action, 'load')
+    def test_complete_lifecycle_proc_warning(self, mock_load,
+                                             mock_dispatcher_start):
+        action = ab.Action(OBJID, 'OBJECT_ACTION', self.ctx)
+        mock_obj = mock.Mock()
+        action.entity = mock_obj
+        mock_get_status = self.patchobject(action, 'get_status')
+        mock_get_status.return_value = consts.ACTION_SUCCEEDED
+        mock_set_status = self.patchobject(action, 'set_status')
+        mock_load.return_value = action
+
+        res = ca.CompleteLifecycleProc(self.ctx, 'ACTION_ID')
+
+        self.assertFalse(res)
+        mock_load.assert_called_once_with(self.ctx, action_id='ACTION_ID',
+                                          project_safe=False)
+        mock_get_status.assert_called_once_with()
+        mock_set_status.assert_not_called()
+        mock_dispatcher_start.assert_not_called()
