@@ -22,9 +22,11 @@ from senlin.common.i18n import _
 from senlin.common import scaleutils as su
 from senlin.common import utils as common_utils
 from senlin.engine.actions import base as am
+from senlin.engine.actions import cluster_action as ca
 from senlin.engine import dispatcher
 from senlin.engine import node as nm
 from senlin.engine import service
+from senlin.objects import action as ao
 from senlin.objects import base as obj_base
 from senlin.objects import cluster as co
 from senlin.objects import cluster_policy as cpo
@@ -144,8 +146,8 @@ class ClusterTest(base.SenlinTestCase):
                  min_size=0, max_size=-1, timeout=3600, metadata={},
                  dependents={}, data={}, next_index=1, status='INIT',
                  config={},
-                 status_reason='Initializing', user=self.ctx.user,
-                 project=self.ctx.project, domain=self.ctx.domain))
+                 status_reason='Initializing', user=self.ctx.user_id,
+                 project=self.ctx.project_id, domain=self.ctx.domain_id))
         mock_action.assert_called_once_with(
             self.ctx,
             '12345678ABC', 'CLUSTER_CREATE',
@@ -188,8 +190,8 @@ class ClusterTest(base.SenlinTestCase):
                  min_size=1, max_size=5, timeout=3600, metadata={},
                  dependents={}, data={}, next_index=1, status='INIT',
                  config={'k1': 'v1'},
-                 status_reason='Initializing', user=self.ctx.user,
-                 project=self.ctx.project, domain=self.ctx.domain))
+                 status_reason='Initializing', user=self.ctx.user_id,
+                 project=self.ctx.project_id, domain=self.ctx.domain_id))
         mock_action.assert_called_once_with(
             self.ctx,
             '12345678ABC', 'CLUSTER_CREATE',
@@ -1396,6 +1398,35 @@ class ClusterTest(base.SenlinTestCase):
         )
         notify.assert_called_once_with()
 
+    @mock.patch.object(ao.Action, 'delete_by_target')
+    @mock.patch.object(am.Action, 'create')
+    @mock.patch.object(co.Cluster, 'find')
+    @mock.patch.object(dispatcher, 'start_action')
+    def test_cluster_check_with_delete(self, notify, mock_find, mock_action,
+                                       mock_delete):
+        x_cluster = mock.Mock(id='CID', user='USER', project='PROJECT')
+        mock_find.return_value = x_cluster
+        mock_action.return_value = 'ACTION_ID'
+        req = orco.ClusterCheckRequest(identity='C1',
+                                       params={'delete_check_action': True})
+
+        res = self.eng.cluster_check(self.ctx, req.obj_to_primitive())
+
+        self.assertEqual({'action': 'ACTION_ID'}, res)
+        mock_find.assert_called_once_with(self.ctx, 'C1')
+        mock_delete.assert_called_once_with(
+            self.ctx, 'CID', action=['CLUSTER_CHECK'],
+            status=['SUCCEEDED', 'FAILED']
+        )
+        mock_action.assert_called_once_with(
+            self.ctx, 'CID', consts.CLUSTER_CHECK,
+            name='cluster_check_CID',
+            cause=consts.CAUSE_RPC,
+            status=am.Action.READY,
+            inputs={'delete_check_action': True},
+        )
+        notify.assert_called_once_with()
+
     @mock.patch.object(am.Action, 'create')
     @mock.patch.object(co.Cluster, 'find')
     @mock.patch.object(dispatcher, 'start_action')
@@ -1419,6 +1450,30 @@ class ClusterTest(base.SenlinTestCase):
         )
         notify.assert_called_once_with()
 
+    @mock.patch.object(am.Action, 'create')
+    @mock.patch.object(co.Cluster, 'find')
+    @mock.patch.object(dispatcher, 'start_action')
+    def test_cluster_check_project_is_none(self, notify, mock_find,
+                                           mock_action):
+        x_cluster = mock.Mock(id='CID', user='USER')
+        mock_find.return_value = x_cluster
+        mock_action.return_value = 'ACTION_ID'
+        req = orco.ClusterCheckRequest(identity='C1')
+
+        result = self.eng.cluster_check(self.ctx, req.obj_to_primitive())
+
+        self.assertIsNotNone(x_cluster.user)
+        self.assertEqual({'action': 'ACTION_ID'}, result)
+        mock_find.assert_called_once_with(self.ctx, 'C1')
+        mock_action.assert_called_once_with(
+            self.ctx, 'CID', consts.CLUSTER_CHECK,
+            name='cluster_check_CID',
+            cause=consts.CAUSE_RPC,
+            status=am.Action.READY,
+            inputs={},
+        )
+        notify.assert_called_once_with()
+
     @mock.patch.object(co.Cluster, 'find')
     def test_cluster_check_cluster_not_found(self, mock_find):
         mock_find.side_effect = exc.ResourceNotFound(type='cluster',
@@ -1890,7 +1945,7 @@ class ClusterTest(base.SenlinTestCase):
         mock_policies.return_value = []
         mock_receivers.return_value = []
         mock_action.return_value = 'ACTION_ID'
-        req = orco.ClusterDeleteRequest(identity='IDENTITY')
+        req = orco.ClusterDeleteRequest(identity='IDENTITY', force=False)
 
         result = self.eng.cluster_delete(self.ctx, req.obj_to_primitive())
 
@@ -1913,7 +1968,8 @@ class ClusterTest(base.SenlinTestCase):
         cluster = mock.Mock(id='cluster1', status='ACTIVE',
                             dependents=dependents)
         mock_find.return_value = cluster
-        req = orco.ClusterDeleteRequest(identity='FAKE_CLUSTER')
+        req = orco.ClusterDeleteRequest(identity='FAKE_CLUSTER',
+                                        force=False)
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.cluster_delete,
@@ -1928,7 +1984,8 @@ class ClusterTest(base.SenlinTestCase):
     def test_cluster_delete_not_found(self, mock_find):
         mock_find.side_effect = exc.ResourceNotFound(type='cluster',
                                                      id='Bogus')
-        req = orco.ClusterDeleteRequest(identity='Bogus')
+        req = orco.ClusterDeleteRequest(identity='Bogus',
+                                        force=False)
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.cluster_delete,
@@ -1944,7 +2001,8 @@ class ClusterTest(base.SenlinTestCase):
                            consts.CS_DELETING, consts.CS_RECOVERING]:
             fake_cluster = mock.Mock(id='12345678AB', status=bad_status)
             mock_find.return_value = fake_cluster
-            req = orco.ClusterDeleteRequest(identity='BUSY')
+            req = orco.ClusterDeleteRequest(identity='BUSY',
+                                            force=False)
 
             ex = self.assertRaises(rpc.ExpectedException,
                                    self.eng.cluster_delete,
@@ -1961,7 +2019,8 @@ class ClusterTest(base.SenlinTestCase):
         x_obj = mock.Mock(id='12345678AB', dependents={})
         mock_find.return_value = x_obj
         mock_policies.return_value = [mock.Mock()]
-        req = orco.ClusterDeleteRequest(identity='IDENTITY')
+        req = orco.ClusterDeleteRequest(identity='IDENTITY',
+                                        force=False)
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.cluster_delete,
@@ -1983,7 +2042,8 @@ class ClusterTest(base.SenlinTestCase):
         mock_find.return_value = x_obj
         mock_policies.return_value = []
         mock_receivers.return_value = [mock.Mock()]
-        req = orco.ClusterDeleteRequest(identity='IDENTITY')
+        req = orco.ClusterDeleteRequest(identity='IDENTITY',
+                                        force=False)
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.cluster_delete,
@@ -2007,7 +2067,8 @@ class ClusterTest(base.SenlinTestCase):
         mock_find.return_value = x_obj
         mock_policies.return_value = [mock.Mock()]
         mock_receivers.return_value = [mock.Mock()]
-        req = orco.ClusterDeleteRequest(identity='IDENTITY')
+        req = orco.ClusterDeleteRequest(identity='IDENTITY',
+                                        force=False)
 
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.cluster_delete,
@@ -2022,3 +2083,48 @@ class ClusterTest(base.SenlinTestCase):
         mock_policies.assert_called_once_with(self.ctx, '12345678AB')
         mock_receivers.assert_called_once_with(
             self.ctx, filters={'cluster_id': '12345678AB'})
+
+    @mock.patch.object(am.Action, 'create')
+    @mock.patch.object(ro.Receiver, 'get_all')
+    @mock.patch.object(cpo.ClusterPolicy, 'get_all')
+    @mock.patch.object(co.Cluster, 'find')
+    @mock.patch.object(dispatcher, 'start_action')
+    def test_cluster_delete_force(self, notify, mock_find, mock_policies,
+                                  mock_receivers, mock_action):
+        for bad_status in [consts.CS_CREATING, consts.CS_UPDATING,
+                           consts.CS_DELETING, consts.CS_RECOVERING]:
+            x_obj = mock.Mock(id='12345678AB', status=bad_status,
+                              dependents={})
+            mock_find.return_value = x_obj
+            mock_policies.return_value = []
+            mock_receivers.return_value = []
+            mock_action.return_value = 'ACTION_ID'
+            req = orco.ClusterDeleteRequest(identity='IDENTITY', force=True)
+
+            result = self.eng.cluster_delete(self.ctx, req.obj_to_primitive())
+
+            self.assertEqual({'action': 'ACTION_ID'}, result)
+            mock_find.assert_called_with(self.ctx, 'IDENTITY')
+            mock_policies.assert_called_with(self.ctx, '12345678AB')
+            mock_receivers.assert_called_with(
+                self.ctx, filters={'cluster_id': '12345678AB'})
+            mock_action.assert_called_with(
+                self.ctx, '12345678AB', 'CLUSTER_DELETE',
+                name='cluster_delete_12345678',
+                cause=consts.CAUSE_RPC,
+                status=am.Action.READY)
+
+            notify.assert_called_with()
+
+    @mock.patch.object(ca, 'CompleteLifecycleProc')
+    def test_cluster_complete_lifecycle(self, mock_lifecycle):
+        req = orco.ClusterCompleteLifecycleRequest(
+            identity='CLUSTER', lifecycle_action_token='NODE_ACTION_ID')
+
+        # do it
+        res = self.eng.cluster_complete_lifecycle(self.ctx,
+                                                  req.obj_to_primitive())
+
+        # verify
+        self.assertEqual({'action': 'NODE_ACTION_ID'}, res)
+        mock_lifecycle.assert_called_once_with(self.ctx, 'NODE_ACTION_ID')
