@@ -18,6 +18,7 @@ import six
 from senlin.common import consts
 from senlin.common import exception as exc
 from senlin.common.i18n import _
+from senlin.common import utils as common_utils
 from senlin.engine.actions import base as action_mod
 from senlin.engine import dispatcher
 from senlin.engine import environment
@@ -166,9 +167,9 @@ class NodeTest(base.SenlinTestCase):
                 'index': -1,
                 'role': '',
                 'metadata': {},
-                'user': self.ctx.user,
-                'project': self.ctx.project,
-                'domain': self.ctx.domain,
+                'user': self.ctx.user_id,
+                'project': self.ctx.project_id,
+                'domain': self.ctx.domain_id,
                 'data': {},
                 'init_at': mock.ANY,
                 'dependents': {},
@@ -183,6 +184,7 @@ class NodeTest(base.SenlinTestCase):
             status=action_mod.Action.READY)
         notify.assert_called_once_with()
 
+    @mock.patch.object(common_utils, 'format_node_name')
     @mock.patch.object(action_mod.Action, 'create')
     @mock.patch.object(no.Node, 'create')
     @mock.patch.object(co.Cluster, 'get_next_index')
@@ -191,16 +193,19 @@ class NodeTest(base.SenlinTestCase):
     @mock.patch.object(dispatcher, 'start_action')
     def test_node_create_same_profile(self, notify, mock_profile,
                                       mock_cluster, mock_index,
-                                      mock_node, mock_action):
+                                      mock_node, mock_action,
+                                      mock_node_name):
         mock_profile.return_value = mock.Mock(id='PROFILE_ID',
                                               type='PROFILE_TYPE')
-        x_cluster = mock.Mock(id='CLUSTER_ID', profile_id='PROFILE_ID')
+        x_cluster = mock.Mock(id='CLUSTER_ID', profile_id='PROFILE_ID',
+                              config={})
         mock_cluster.return_value = x_cluster
         mock_index.return_value = 12345
         x_node = mock.Mock(id='NODE_ID')
         x_node.to_dict.return_value = {'foo': 'bar'}
         mock_node.return_value = x_node
         mock_action.return_value = 'ACTION_ID'
+        mock_node_name.return_value = "GENERATED_NAME"
         req = orno.NodeCreateRequestBody(name='NODE1',
                                          profile_id='PROFILE_NAME',
                                          cluster_id='FAKE_CLUSTER')
@@ -214,15 +219,15 @@ class NodeTest(base.SenlinTestCase):
         mock_node.assert_called_once_with(
             self.ctx,
             {
-                'name': 'NODE1',
+                'name': 'GENERATED_NAME',
                 'profile_id': 'PROFILE_ID',
                 'cluster_id': 'CLUSTER_ID',
                 'index': 12345,
                 'role': '',
                 'metadata': {},
-                'user': self.ctx.user,
-                'project': self.ctx.project,
-                'domain': self.ctx.domain,
+                'user': self.ctx.user_id,
+                'project': self.ctx.project_id,
+                'domain': self.ctx.domain_id,
                 'data': {},
                 'init_at': mock.ANY,
                 'dependents': {},
@@ -237,6 +242,7 @@ class NodeTest(base.SenlinTestCase):
             status=action_mod.Action.READY)
         notify.assert_called_once_with()
 
+    @mock.patch.object(common_utils, "format_node_name")
     @mock.patch.object(action_mod.Action, 'create')
     @mock.patch.object(no.Node, 'create')
     @mock.patch.object(co.Cluster, 'get_next_index')
@@ -245,18 +251,21 @@ class NodeTest(base.SenlinTestCase):
     @mock.patch.object(dispatcher, 'start_action')
     def test_node_create_same_profile_type(self, notify, mock_profile,
                                            mock_cluster, mock_index,
-                                           mock_node, mock_action):
+                                           mock_node, mock_action,
+                                           mock_node_name):
         mock_profile.side_effect = [
             mock.Mock(id='NODE_PROFILE_ID', type='PROFILE_TYPE'),
             mock.Mock(id='CLUSTER_PROFILE_ID', type='PROFILE_TYPE'),
         ]
-        x_cluster = mock.Mock(id='CLUSTER_ID', profile_id='CLUSTER_PROFILE_ID')
+        x_cluster = mock.Mock(id='CLUSTER_ID', profile_id='CLUSTER_PROFILE_ID',
+                              config={})
         mock_cluster.return_value = x_cluster
         mock_index.return_value = 12345
         x_node = mock.Mock(id='NODE_ID')
         x_node.to_dict.return_value = {'foo': 'bar'}
         mock_node.return_value = x_node
         mock_action.return_value = 'ACTION_ID'
+        mock_node_name.return_value = 'GENERATED_NAME'
         req = orno.NodeCreateRequestBody(name='NODE1',
                                          profile_id='PROFILE_NAME',
                                          cluster_id='FAKE_CLUSTER')
@@ -273,7 +282,7 @@ class NodeTest(base.SenlinTestCase):
         mock_node.assert_called_once_with(
             self.ctx,
             {
-                'name': 'NODE1',
+                'name': 'GENERATED_NAME',
                 'profile_id': 'NODE_PROFILE_ID',
                 'cluster_id': 'CLUSTER_ID',
                 'physical_id': None,
@@ -282,9 +291,9 @@ class NodeTest(base.SenlinTestCase):
                 'metadata': {},
                 'status': 'INIT',
                 'status_reason': 'Initializing',
-                'user': self.ctx.user,
-                'project': self.ctx.project,
-                'domain': self.ctx.domain,
+                'user': self.ctx.user_id,
+                'project': self.ctx.project_id,
+                'domain': self.ctx.domain_id,
                 'data': {},
                 'dependents': {},
                 'init_at': mock.ANY,
@@ -296,8 +305,9 @@ class NodeTest(base.SenlinTestCase):
             status=action_mod.Action.READY)
         notify.assert_called_once_with()
 
+    @mock.patch.object(po.Profile, 'find')
     @mock.patch.object(no.Node, 'get_by_name')
-    def test_node_create_name_conflict(self, mock_get):
+    def test_node_create_name_conflict(self, mock_find, mock_get):
         cfg.CONF.set_override('name_unique', True)
         mock_get.return_value = mock.Mock()
         req = orno.NodeCreateRequestBody(name='NODE1',
@@ -329,7 +339,6 @@ class NodeTest(base.SenlinTestCase):
     @mock.patch.object(co.Cluster, 'find')
     @mock.patch.object(po.Profile, 'find')
     def test_node_create_cluster_not_found(self, mock_profile, mock_cluster):
-        mock_profile.return_value = mock.Mock()
         mock_cluster.side_effect = exc.ResourceNotFound(type='cluster',
                                                         id='Bogus')
         req = orno.NodeCreateRequestBody(name='NODE1',
@@ -343,7 +352,6 @@ class NodeTest(base.SenlinTestCase):
         self.assertEqual(exc.BadRequest, ex.exc_info[0])
         self.assertEqual("The specified cluster 'Bogus' could not be found.",
                          six.text_type(ex.exc_info[1]))
-        mock_profile.assert_called_once_with(self.ctx, 'PROFILE_NAME')
         mock_cluster.assert_called_once_with(self.ctx, 'Bogus')
 
     @mock.patch.object(co.Cluster, 'find')
@@ -569,6 +577,30 @@ class NodeTest(base.SenlinTestCase):
             mock.call(self.ctx, 'OLD_PROFILE_ID'),
         ])
 
+    @mock.patch.object(po.Profile, 'find')
+    @mock.patch.object(no.Node, 'find')
+    def test_node_update_dumplicated_profile(self, mock_find, mock_profile):
+        mock_find.return_value = mock.Mock(profile_id='OLD_PROFILE_ID')
+        mock_profile.side_effect = [
+            mock.Mock(id='OLD_PROFILE_ID', type='PROFILE_TYPE'),
+            mock.Mock(id='OLD_PROFILE_ID', type='PROFILE_TYPE'),
+        ]
+
+        req = orno.NodeUpdateRequest(identity='FAKE_NODE',
+                                     profile_id='OLD_PROFILE_ID')
+        ex = self.assertRaises(rpc.ExpectedException,
+                               self.eng.node_update,
+                               self.ctx, req.obj_to_primitive())
+
+        self.assertEqual(exc.BadRequest, ex.exc_info[0])
+        self.assertEqual("No property needs an update.",
+                         six.text_type(ex.exc_info[1]))
+        mock_find.assert_called_once_with(self.ctx, 'FAKE_NODE')
+        mock_profile.assert_has_calls([
+            mock.call(self.ctx, 'OLD_PROFILE_ID'),
+            mock.call(self.ctx, 'OLD_PROFILE_ID'),
+        ])
+
     @mock.patch.object(no.Node, 'find')
     def test_node_update_no_property_for_update(self, mock_find):
         x_obj = mock.Mock(id='FAKE_NODE_ID', name='NODE1', role='ROLE1',
@@ -594,7 +626,7 @@ class NodeTest(base.SenlinTestCase):
                                            dependents={})
         mock_action.return_value = 'ACTION_ID'
 
-        req = orno.NodeDeleteRequest(identity='FAKE_NODE')
+        req = orno.NodeDeleteRequest(identity='FAKE_NODE', force=False)
         result = self.eng.node_delete(self.ctx, req.obj_to_primitive())
 
         self.assertEqual({'action': 'ACTION_ID'}, result)
@@ -610,7 +642,7 @@ class NodeTest(base.SenlinTestCase):
     def test_node_delete_node_not_found(self, mock_find):
         mock_find.side_effect = exc.ResourceNotFound(type='node', id='Bogus')
 
-        req = orno.NodeDeleteRequest(identity='Bogus')
+        req = orno.NodeDeleteRequest(identity='Bogus', force=False)
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.node_delete, self.ctx,
                                req.obj_to_primitive())
@@ -626,7 +658,7 @@ class NodeTest(base.SenlinTestCase):
                            consts.NS_DELETING, consts.NS_RECOVERING]:
             fake_node = mock.Mock(id='12345678AB', status=bad_status)
             mock_find.return_value = fake_node
-            req = orno.NodeDeleteRequest(identity='BUSY')
+            req = orno.NodeDeleteRequest(identity='BUSY', force=False)
             ex = self.assertRaises(rpc.ExpectedException,
                                    self.eng.node_delete,
                                    self.ctx, req.obj_to_primitive())
@@ -641,7 +673,7 @@ class NodeTest(base.SenlinTestCase):
         dependents = {'nodes': ['NODE1']}
         node = mock.Mock(id='NODE_ID', status='ACTIVE', dependents=dependents)
         mock_find.return_value = node
-        req = orno.NodeDeleteRequest(identity='node1')
+        req = orno.NodeDeleteRequest(identity='node1', force=False)
         ex = self.assertRaises(rpc.ExpectedException,
                                self.eng.node_delete,
                                self.ctx, req.obj_to_primitive())
@@ -650,6 +682,30 @@ class NodeTest(base.SenlinTestCase):
                          "by other clusters and/or nodes.",
                          six.text_type(ex.exc_info[1]))
 
+    @mock.patch.object(dispatcher, 'start_action')
+    @mock.patch.object(action_mod.Action, 'create')
+    @mock.patch.object(no.Node, 'find')
+    def test_node_delete_force(self, mock_find, mock_action, mock_start):
+        for bad_status in [consts.NS_CREATING, consts.NS_UPDATING,
+                           consts.NS_DELETING, consts.NS_RECOVERING]:
+
+            mock_find.return_value = mock.Mock(id='12345678AB',
+                                               status=bad_status,
+                                               dependents={})
+            mock_action.return_value = 'ACTION_ID'
+
+            req = orno.NodeDeleteRequest(identity='FAKE_NODE', force=True)
+            result = self.eng.node_delete(self.ctx, req.obj_to_primitive())
+
+            self.assertEqual({'action': 'ACTION_ID'}, result)
+            mock_find.assert_called_with(self.ctx, 'FAKE_NODE')
+            mock_action.assert_called_with(
+                self.ctx, '12345678AB', consts.NODE_DELETE,
+                name='node_delete_12345678',
+                cause=consts.CAUSE_RPC,
+                status=action_mod.Action.READY)
+            mock_start.assert_called_with()
+
     @mock.patch.object(environment.Environment, 'get_profile')
     @mock.patch.object(pb.Profile, 'adopt_node')
     def test__node_adopt_preview(self, mock_adopt, mock_profile):
@@ -797,9 +853,9 @@ class NodeTest(base.SenlinTestCase):
             'status_reason': 'Node adopted successfully',
             'init_at': mock.ANY,
             'created_at': mock.ANY,
-            'user': self.ctx.user,
-            'project': self.ctx.project,
-            'domain': self.ctx.domain
+            'user': self.ctx.user_id,
+            'project': self.ctx.project_id,
+            'domain': self.ctx.domain_id
         }
         mock_create.assert_called_once_with(self.ctx, attrs)
 
