@@ -26,7 +26,6 @@ from senlin.objects import cluster as obj_cluster
 from senlin.objects import health_registry as hr
 from senlin.objects import node as obj_node
 from senlin.objects import profile as obj_profile
-from senlin.objects.requests import nodes as vorn
 from senlin.rpc import client as rpc_client
 from senlin.tests.unit.common import base
 
@@ -57,15 +56,14 @@ class TestNovaNotificationEndpoint(base.SenlinTestCase):
     def test_init(self, mock_rpc, mock_filter):
         x_filter = mock_filter.return_value
         event_map = {
-            'compute.instance.delete.end': 'DELETE',
             'compute.instance.pause.end': 'PAUSE',
             'compute.instance.power_off.end': 'POWER_OFF',
             'compute.instance.rebuild.error': 'REBUILD',
             'compute.instance.shutdown.end': 'SHUTDOWN',
             'compute.instance.soft_delete.end': 'SOFT_DELETE',
         }
-
-        obj = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER')
+        recover_action = {'operation': 'REBUILD'}
+        obj = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER', recover_action)
 
         mock_filter.assert_called_once_with(
             publisher_id='^compute.*',
@@ -84,7 +82,9 @@ class TestNovaNotificationEndpoint(base.SenlinTestCase):
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info(self, mock_rpc, mock_context, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {
             'metadata': {
@@ -100,27 +100,30 @@ class TestNovaNotificationEndpoint(base.SenlinTestCase):
         call_ctx = mock.Mock()
         mock_context.return_value = call_ctx
 
-        res = endpoint.info(ctx, 'PUBLISHER', 'compute.instance.delete.end',
+        res = endpoint.info(ctx, 'PUBLISHER', 'compute.instance.shutdown.end',
                             payload, metadata)
 
         self.assertIsNone(res)
         x_rpc.call.assert_called_once_with(call_ctx, 'node_recover', mock.ANY)
         req = x_rpc.call.call_args[0][2]
-        self.assertIsInstance(req, vorn.NodeRecoverRequest)
+        self.assertIsInstance(req, objects.NodeRecoverRequest)
         self.assertEqual('FAKE_NODE', req.identity)
         expected_params = {
-            'event': 'DELETE',
+            'event': 'SHUTDOWN',
             'state': 'shutoff',
             'instance_id': 'PHYSICAL_ID',
             'timestamp': 'TIMESTAMP',
             'publisher': 'PUBLISHER',
+            'operation': 'REBUILD'
         }
         self.assertEqual(expected_params, req.params)
 
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info_no_metadata(self, mock_rpc, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {'metadata': {}}
         metadata = {'timestamp': 'TIMESTAMP'}
@@ -134,7 +137,9 @@ class TestNovaNotificationEndpoint(base.SenlinTestCase):
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info_no_cluster_in_metadata(self, mock_rpc, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {'metadata': {'foo': 'bar'}}
         metadata = {'timestamp': 'TIMESTAMP'}
@@ -148,7 +153,9 @@ class TestNovaNotificationEndpoint(base.SenlinTestCase):
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info_cluster_id_not_match(self, mock_rpc, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {'metadata': {'cluster_id': 'FOOBAR'}}
         metadata = {'timestamp': 'TIMESTAMP'}
@@ -162,7 +169,9 @@ class TestNovaNotificationEndpoint(base.SenlinTestCase):
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info_event_type_not_interested(self, mock_rpc, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {'metadata': {'cluster_id': 'CLUSTER_ID'}}
         metadata = {'timestamp': 'TIMESTAMP'}
@@ -176,7 +185,9 @@ class TestNovaNotificationEndpoint(base.SenlinTestCase):
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info_no_node_id(self, mock_rpc, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {'metadata': {'cluster_id': 'CLUSTER_ID'}}
         metadata = {'timestamp': 'TIMESTAMP'}
@@ -191,7 +202,9 @@ class TestNovaNotificationEndpoint(base.SenlinTestCase):
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info_default_values(self, mock_rpc, mock_context, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.NovaNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {
             'metadata': {
@@ -204,20 +217,21 @@ class TestNovaNotificationEndpoint(base.SenlinTestCase):
         call_ctx = mock.Mock()
         mock_context.return_value = call_ctx
 
-        res = endpoint.info(ctx, 'PUBLISHER', 'compute.instance.delete.end',
+        res = endpoint.info(ctx, 'PUBLISHER', 'compute.instance.shutdown.end',
                             payload, metadata)
 
         self.assertIsNone(res)
         x_rpc.call.assert_called_once_with(call_ctx, 'node_recover', mock.ANY)
         req = x_rpc.call.call_args[0][2]
-        self.assertIsInstance(req, vorn.NodeRecoverRequest)
+        self.assertIsInstance(req, objects.NodeRecoverRequest)
         self.assertEqual('NODE_ID', req.identity)
         expected_params = {
-            'event': 'DELETE',
+            'event': 'SHUTDOWN',
             'state': 'Unknown',
             'instance_id': 'Unknown',
             'timestamp': 'TIMESTAMP',
             'publisher': 'PUBLISHER',
+            'operation': 'REBUILD',
         }
         self.assertEqual(expected_params, req.params)
 
@@ -231,8 +245,8 @@ class TestHeatNotificationEndpoint(base.SenlinTestCase):
         event_map = {
             'orchestration.stack.delete.end': 'DELETE',
         }
-
-        obj = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER')
+        recover_action = {'operation': 'REBUILD'}
+        obj = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER', recover_action)
 
         mock_filter.assert_called_once_with(
             publisher_id='^orchestration.*',
@@ -247,11 +261,14 @@ class TestHeatNotificationEndpoint(base.SenlinTestCase):
         self.assertEqual('PROJECT', obj.project_id)
         self.assertEqual('CLUSTER', obj.cluster_id)
 
-    @mock.patch.object(context, 'get_service_context')
+    @mock.patch.object(context.RequestContext, 'from_dict')
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info(self, mock_rpc, mock_context, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
+        ctx = mock.Mock()
         payload = {
             'tags': {
                 'cluster_id=CLUSTER_ID',
@@ -265,16 +282,14 @@ class TestHeatNotificationEndpoint(base.SenlinTestCase):
         metadata = {'timestamp': 'TIMESTAMP'}
         call_ctx = mock.Mock()
         mock_context.return_value = call_ctx
-        ctx = mock.Mock()
 
         res = endpoint.info(ctx, 'PUBLISHER', 'orchestration.stack.delete.end',
                             payload, metadata)
 
         self.assertIsNone(res)
-        mock_context.assert_called_once_with(project='PROJECT', user='USER')
         x_rpc.call.assert_called_once_with(call_ctx, 'node_recover', mock.ANY)
         req = x_rpc.call.call_args[0][2]
-        self.assertIsInstance(req, vorn.NodeRecoverRequest)
+        self.assertIsInstance(req, objects.NodeRecoverRequest)
         self.assertEqual('FAKE_NODE', req.identity)
         expected_params = {
             'event': 'DELETE',
@@ -282,13 +297,16 @@ class TestHeatNotificationEndpoint(base.SenlinTestCase):
             'stack_id': 'PHYSICAL_ID',
             'timestamp': 'TIMESTAMP',
             'publisher': 'PUBLISHER',
+            'operation': 'REBUILD',
         }
         self.assertEqual(expected_params, req.params)
 
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info_event_type_not_interested(self, mock_rpc, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {'tags': {'cluster_id': 'CLUSTER_ID'}}
         metadata = {'timestamp': 'TIMESTAMP'}
@@ -303,7 +321,9 @@ class TestHeatNotificationEndpoint(base.SenlinTestCase):
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info_no_tag(self, mock_rpc, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {'tags': None}
         metadata = {'timestamp': 'TIMESTAMP'}
@@ -317,7 +337,9 @@ class TestHeatNotificationEndpoint(base.SenlinTestCase):
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info_empty_tag(self, mock_rpc, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {'tags': []}
         metadata = {'timestamp': 'TIMESTAMP'}
@@ -331,7 +353,9 @@ class TestHeatNotificationEndpoint(base.SenlinTestCase):
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info_no_cluster_in_tag(self, mock_rpc, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {'tags': ['foo', 'bar']}
         metadata = {'timestamp': 'TIMESTAMP'}
@@ -345,7 +369,9 @@ class TestHeatNotificationEndpoint(base.SenlinTestCase):
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info_no_node_in_tag(self, mock_rpc, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {'tags': ['cluster_id=C1ID']}
         metadata = {'timestamp': 'TIMESTAMP'}
@@ -359,7 +385,9 @@ class TestHeatNotificationEndpoint(base.SenlinTestCase):
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info_cluster_id_not_match(self, mock_rpc, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {'tags': ['cluster_id=FOOBAR', 'cluster_node_id=N2']}
         metadata = {'timestamp': 'TIMESTAMP'}
@@ -370,11 +398,13 @@ class TestHeatNotificationEndpoint(base.SenlinTestCase):
         self.assertIsNone(res)
         self.assertEqual(0, x_rpc.node_recover.call_count)
 
-    @mock.patch.object(context, 'get_service_context')
+    @mock.patch.object(context.RequestContext, 'from_dict')
     @mock.patch('senlin.rpc.client.EngineClient')
     def test_info_default_values(self, mock_rpc, mock_context, mock_filter):
         x_rpc = mock_rpc.return_value
-        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        endpoint = hm.HeatNotificationEndpoint('PROJECT', 'CLUSTER_ID',
+                                               recover_action)
         ctx = mock.Mock()
         payload = {
             'tags': [
@@ -393,7 +423,7 @@ class TestHeatNotificationEndpoint(base.SenlinTestCase):
         self.assertIsNone(res)
         x_rpc.call.assert_called_once_with(call_ctx, 'node_recover', mock.ANY)
         req = x_rpc.call.call_args[0][2]
-        self.assertIsInstance(req, vorn.NodeRecoverRequest)
+        self.assertIsInstance(req, objects.NodeRecoverRequest)
         self.assertEqual('NODE_ID', req.identity)
         expected_params = {
             'event': 'DELETE',
@@ -401,6 +431,7 @@ class TestHeatNotificationEndpoint(base.SenlinTestCase):
             'stack_id': 'Unknown',
             'timestamp': 'TIMESTAMP',
             'publisher': 'PUBLISHER',
+            'operation': 'REBUILD',
         }
         self.assertEqual(expected_params, req.params)
 
@@ -416,7 +447,7 @@ class TestListenerProc(base.SenlinTestCase):
                                 mock_target, mock_novaendpoint,
                                 mock_heatendpoint):
         cfg.CONF.set_override('nova_control_exchange', 'FAKE_EXCHANGE',
-                              group='health_manager', enforce_type=True)
+                              group='health_manager')
 
         x_listener = mock.Mock()
         mock_listener.return_value = x_listener
@@ -427,13 +458,16 @@ class TestListenerProc(base.SenlinTestCase):
         x_endpoint = mock.Mock()
         mock_novaendpoint.return_value = x_endpoint
 
-        res = hm.ListenerProc('FAKE_EXCHANGE', 'PROJECT_ID', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        res = hm.ListenerProc('FAKE_EXCHANGE', 'PROJECT_ID', 'CLUSTER_ID',
+                              recover_action)
 
         self.assertIsNone(res)
         mock_transport.assert_called_once_with(cfg.CONF)
         mock_target.assert_called_once_with(topic="versioned_notifications",
                                             exchange='FAKE_EXCHANGE')
-        mock_novaendpoint.assert_called_once_with('PROJECT_ID', 'CLUSTER_ID')
+        mock_novaendpoint.assert_called_once_with('PROJECT_ID', 'CLUSTER_ID',
+                                                  recover_action)
         mock_listener.assert_called_once_with(
             x_transport, [x_target], [x_endpoint],
             executor='threading', pool="senlin-listeners")
@@ -451,13 +485,16 @@ class TestListenerProc(base.SenlinTestCase):
         x_endpoint = mock.Mock()
         mock_heatendpoint.return_value = x_endpoint
 
-        res = hm.ListenerProc('heat', 'PROJECT_ID', 'CLUSTER_ID')
+        recover_action = {'operation': 'REBUILD'}
+        res = hm.ListenerProc('heat', 'PROJECT_ID', 'CLUSTER_ID',
+                              recover_action)
 
         self.assertIsNone(res)
         mock_transport.assert_called_once_with(cfg.CONF)
         mock_target.assert_called_once_with(topic="notifications",
                                             exchange='heat')
-        mock_heatendpoint.assert_called_once_with('PROJECT_ID', 'CLUSTER_ID')
+        mock_heatendpoint.assert_called_once_with('PROJECT_ID', 'CLUSTER_ID',
+                                                  recover_action)
         mock_listener.assert_called_once_with(
             x_transport, [x_target], [x_endpoint],
             executor='threading', pool="senlin-listeners")
@@ -518,7 +555,7 @@ class TestHealthManager(base.SenlinTestCase):
         # assertions
         mock_claim.assert_called_once_with(self.hm.ctx, self.hm.engine_id)
         mock_calls = [
-            mock.call(self.hm._poll_cluster, None, None, 'CID1', 12),
+            mock.call(self.hm._poll_cluster, None, None, 'CID1', 12, {})
         ]
         mock_add_timer.assert_has_calls(mock_calls)
         self.assertEqual(2, len(self.hm.registries))
@@ -543,7 +580,7 @@ class TestHealthManager(base.SenlinTestCase):
             self.hm.registries[1])
 
     @mock.patch.object(hm, "_chase_up")
-    @mock.patch.object(obj_node.Node, 'get_all')
+    @mock.patch.object(obj_node.Node, 'get_all_by_cluster')
     @mock.patch.object(hm.HealthManager, "_wait_for_action")
     @mock.patch.object(obj_cluster.Cluster, 'get')
     @mock.patch.object(context, 'get_service_context')
@@ -561,14 +598,15 @@ class TestHealthManager(base.SenlinTestCase):
         x_action_recover = {'action': 'RECOVER_ID'}
         mock_rpc.side_effect = [x_action_check, x_action_recover]
 
+        recover_action = {'operation': 'REBUILD'}
         # do it
-        res = self.hm._poll_cluster('CLUSTER_ID', 456)
+        res = self.hm._poll_cluster('CLUSTER_ID', 456, recover_action)
 
         self.assertEqual(mock_chase.return_value, res)
         mock_get.assert_called_once_with(self.hm.ctx, 'CLUSTER_ID',
                                          project_safe=False)
-        mock_ctx.assert_called_once_with(user=x_cluster.user,
-                                         project=x_cluster.project)
+        mock_ctx.assert_called_once_with(user_id=x_cluster.user,
+                                         project_id=x_cluster.project)
         mock_rpc.assert_has_calls([
             mock.call(ctx, 'cluster_check', mock.ANY),
             mock.call(ctx, 'node_recover', mock.ANY)
@@ -582,8 +620,9 @@ class TestHealthManager(base.SenlinTestCase):
     def test__poll_cluster_not_found(self, mock_check, mock_get, mock_chase):
         mock_get.return_value = None
 
+        recover_action = {'operation': 'REBUILD'}
         # do it
-        res = self.hm._poll_cluster('CLUSTER_ID', 123)
+        res = self.hm._poll_cluster('CLUSTER_ID', 123, recover_action)
 
         self.assertEqual(mock_chase.return_value, res)
         self.assertEqual(0, mock_check.call_count)
@@ -601,13 +640,15 @@ class TestHealthManager(base.SenlinTestCase):
         mock_ctx.return_value = ctx
         mock_check.side_effect = Exception("boom")
 
+        recover_action = {'operation': 'REBUILD'}
         # do it
-        res = self.hm._poll_cluster('CLUSTER_ID', 123)
+        res = self.hm._poll_cluster('CLUSTER_ID', 123, recover_action)
 
         self.assertEqual(mock_chase.return_value, res)
         mock_get.assert_called_once_with(self.hm.ctx, 'CLUSTER_ID',
                                          project_safe=False)
-        mock_ctx.assert_called_once_with(user='USER_ID', project='PROJECT_ID')
+        mock_ctx.assert_called_once_with(user_id='USER_ID',
+                                         project_id='PROJECT_ID')
         mock_check.assert_called_once_with(ctx, 'cluster_check', mock.ANY)
         mock_chase.assert_called_once_with(mock.ANY, 123)
 
@@ -626,13 +667,15 @@ class TestHealthManager(base.SenlinTestCase):
         x_action_check = {'action': 'CHECK_ID'}
         mock_rpc.return_value = x_action_check
 
+        recover_action = {'operation': 'REBUILD'}
         # do it
-        res = self.hm._poll_cluster('CLUSTER_ID', 456)
+        res = self.hm._poll_cluster('CLUSTER_ID', 456, recover_action)
 
         self.assertEqual(mock_chase.return_value, res)
         mock_get.assert_called_once_with(self.hm.ctx, 'CLUSTER_ID',
                                          project_safe=False)
-        mock_ctx.assert_called_once_with(user='USER_ID', project='PROJECT_ID')
+        mock_ctx.assert_called_once_with(user_id='USER_ID',
+                                         project_id='PROJECT_ID')
         mock_rpc.assert_called_once_with(ctx, 'cluster_check', mock.ANY)
         mock_wait.assert_called_once_with(ctx, "CHECK_ID", 456)
         mock_chase.assert_called_once_with(mock.ANY, 456)
@@ -641,17 +684,18 @@ class TestHealthManager(base.SenlinTestCase):
     @mock.patch.object(obj_cluster.Cluster, 'get')
     def test__add_listener_nova(self, mock_cluster, mock_profile):
         cfg.CONF.set_override('nova_control_exchange', 'FAKE_NOVA_EXCHANGE',
-                              group='health_manager', enforce_type=True)
+                              group='health_manager')
         x_listener = mock.Mock()
         mock_add_thread = self.patchobject(self.hm.TG, 'add_thread',
                                            return_value=x_listener)
         x_cluster = mock.Mock(project='PROJECT_ID', profile_id='PROFILE_ID')
         mock_cluster.return_value = x_cluster
-        x_profile = mock.Mock(type_name='os.nova.server-1.0')
+        x_profile = mock.Mock(type='os.nova.server-1.0')
         mock_profile.return_value = x_profile
 
+        recover_action = {'operation': 'REBUILD'}
         # do it
-        res = self.hm._add_listener('CLUSTER_ID')
+        res = self.hm._add_listener('CLUSTER_ID', recover_action)
 
         # assertions
         self.assertEqual(x_listener, res)
@@ -660,23 +704,25 @@ class TestHealthManager(base.SenlinTestCase):
         mock_profile.assert_called_once_with(self.hm.ctx, 'PROFILE_ID',
                                              project_safe=False)
         mock_add_thread.assert_called_once_with(
-            hm.ListenerProc, 'FAKE_NOVA_EXCHANGE', 'PROJECT_ID', 'CLUSTER_ID')
+            hm.ListenerProc, 'FAKE_NOVA_EXCHANGE', 'PROJECT_ID', 'CLUSTER_ID',
+            recover_action)
 
     @mock.patch.object(obj_profile.Profile, 'get')
     @mock.patch.object(obj_cluster.Cluster, 'get')
     def test__add_listener_heat(self, mock_cluster, mock_profile):
         cfg.CONF.set_override('heat_control_exchange', 'FAKE_HEAT_EXCHANGE',
-                              group='health_manager', enforce_type=True)
+                              group='health_manager')
         x_listener = mock.Mock()
         mock_add_thread = self.patchobject(self.hm.TG, 'add_thread',
                                            return_value=x_listener)
         x_cluster = mock.Mock(project='PROJECT_ID', profile_id='PROFILE_ID')
         mock_cluster.return_value = x_cluster
-        x_profile = mock.Mock(type_name='os.heat.stack-1.0')
+        x_profile = mock.Mock(type='os.heat.stack-1.0')
         mock_profile.return_value = x_profile
 
+        recover_action = {'operation': 'REBUILD'}
         # do it
-        res = self.hm._add_listener('CLUSTER_ID')
+        res = self.hm._add_listener('CLUSTER_ID', recover_action)
 
         # assertions
         self.assertEqual(x_listener, res)
@@ -685,7 +731,8 @@ class TestHealthManager(base.SenlinTestCase):
         mock_profile.assert_called_once_with(self.hm.ctx, 'PROFILE_ID',
                                              project_safe=False)
         mock_add_thread.assert_called_once_with(
-            hm.ListenerProc, 'FAKE_HEAT_EXCHANGE', 'PROJECT_ID', 'CLUSTER_ID')
+            hm.ListenerProc, 'FAKE_HEAT_EXCHANGE', 'PROJECT_ID', 'CLUSTER_ID',
+            recover_action)
 
     @mock.patch.object(obj_profile.Profile, 'get')
     @mock.patch.object(obj_cluster.Cluster, 'get')
@@ -693,11 +740,12 @@ class TestHealthManager(base.SenlinTestCase):
         mock_add_thread = self.patchobject(self.hm.TG, 'add_thread')
         x_cluster = mock.Mock(project='PROJECT_ID', profile_id='PROFILE_ID')
         mock_cluster.return_value = x_cluster
-        x_profile = mock.Mock(type_name='other.types-1.0')
+        x_profile = mock.Mock(type='other.types-1.0')
         mock_profile.return_value = x_profile
 
+        recover_action = {'operation': 'REBUILD'}
         # do it
-        res = self.hm._add_listener('CLUSTER_ID')
+        res = self.hm._add_listener('CLUSTER_ID', recover_action)
 
         # assertions
         self.assertIsNone(res)
@@ -712,8 +760,9 @@ class TestHealthManager(base.SenlinTestCase):
         mock_get.return_value = None
         mock_add_thread = self.patchobject(self.hm.TG, 'add_thread')
 
+        recover_action = {'operation': 'REBUILD'}
         # do it
-        res = self.hm._add_listener('CLUSTER_ID')
+        res = self.hm._add_listener('CLUSTER_ID', recover_action)
 
         # assertions
         self.assertIsNone(res)
@@ -730,14 +779,16 @@ class TestHealthManager(base.SenlinTestCase):
             'cluster_id': 'CCID',
             'interval': 12,
             'check_type': consts.NODE_STATUS_POLLING,
+            'params': {'recover_action': [{'name': 'REBUILD'}]},
         }
+        recover_action = {'operation': 'REBUILD'}
         res = self.hm._start_check(entry)
 
         expected = copy.deepcopy(entry)
         expected['timer'] = x_timer
         self.assertEqual(expected, res)
         mock_add_timer.assert_called_once_with(
-            self.hm._poll_cluster, None, None, 'CCID', 12)
+            self.hm._poll_cluster, None, None, 'CCID', 12, recover_action)
 
     def test__start_check_for_listening(self):
         x_listener = mock.Mock()
@@ -747,13 +798,15 @@ class TestHealthManager(base.SenlinTestCase):
         entry = {
             'cluster_id': 'CCID',
             'check_type': consts.LIFECYCLE_EVENTS,
+            'params': {'recover_action': [{'name': 'REBUILD'}]},
         }
+        recover_action = {'operation': 'REBUILD'}
         res = self.hm._start_check(entry)
 
         expected = copy.deepcopy(entry)
         expected['listener'] = x_listener
         self.assertEqual(expected, res)
-        mock_add_listener.assert_called_once_with('CCID')
+        mock_add_listener.assert_called_once_with('CCID', recover_action)
 
     def test__start_check_for_listening_failed(self):
         mock_add_listener = self.patchobject(self.hm, '_add_listener',
@@ -762,16 +815,19 @@ class TestHealthManager(base.SenlinTestCase):
         entry = {
             'cluster_id': 'CCID',
             'check_type': consts.LIFECYCLE_EVENTS,
+            'params': {'recover_action': [{'name': 'REBUILD'}]},
         }
+        recover_action = {'operation': 'REBUILD'}
         res = self.hm._start_check(entry)
 
         self.assertIsNone(res)
-        mock_add_listener.assert_called_once_with('CCID')
+        mock_add_listener.assert_called_once_with('CCID', recover_action)
 
     def test__start_check_other_types(self):
         entry = {
             'cluster_id': 'CCID',
             'check_type': 'BOGUS TYPE',
+            'params': {'recover_action': [{'name': 'REBUILD'}]},
         }
         res = self.hm._start_check(entry)
 
@@ -846,7 +902,8 @@ class TestHealthManager(base.SenlinTestCase):
         mock_reg_create.assert_called_once_with(
             ctx, 'CLUSTER_ID', consts.NODE_STATUS_POLLING, 50, {}, 'ENGINE_ID',
             enabled=True)
-        mock_add_tm.assert_called_with(mock_poll, None, None, 'CLUSTER_ID', 50)
+        mock_add_tm.assert_called_with(mock_poll, None, None, 'CLUSTER_ID', 50,
+                                       {})
         self.assertEqual(1, len(self.hm.registries))
 
     @mock.patch.object(hr.HealthRegistry, 'create')
@@ -924,3 +981,34 @@ class TestHealthManager(base.SenlinTestCase):
         self.assertIn({'cluster_id': 'FAKE_ID', 'enabled': False},
                       self.hm.rt['registries'])
         mock_update.assert_called_once_with(ctx, 'FAKE_ID', {'enabled': False})
+
+    @mock.patch.object(context, 'get_admin_context')
+    @mock.patch.object(hr.HealthRegistry, 'get')
+    def test_get_manager_engine(self, mock_get, mock_ctx):
+        ctx = mock.Mock()
+        mock_ctx.return_value = ctx
+
+        registry = mock.Mock(engine_id='fake')
+        mock_get.return_value = registry
+
+        result = hm.get_manager_engine('CID')
+
+        self.assertEqual(result, 'fake')
+
+        mock_get.assert_called_once_with(ctx, 'CID')
+        self.assertTrue(mock_ctx.called)
+
+    @mock.patch.object(context, 'get_admin_context')
+    @mock.patch.object(hr.HealthRegistry, 'get')
+    def test_get_manager_engine_none(self, mock_get, mock_ctx):
+        ctx = mock.Mock()
+        mock_ctx.return_value = ctx
+
+        mock_get.return_value = None
+
+        result = hm.get_manager_engine('CID')
+
+        self.assertIsNone(result)
+
+        mock_get.assert_called_once_with(ctx, 'CID')
+        self.assertTrue(mock_ctx.called)
