@@ -78,9 +78,9 @@ class TestNode(base.SenlinTestCase):
         self.assertIsNone(node_info.physical_id)
         self.assertEqual(CLUSTER_ID, node_info.cluster_id)
         self.assertEqual(PROFILE_ID, node_info.profile_id)
-        self.assertEqual(self.context.user, node_info.user)
-        self.assertEqual(self.context.project, node_info.project)
-        self.assertEqual(self.context.domain, node_info.domain)
+        self.assertEqual(self.context.user_id, node_info.user)
+        self.assertEqual(self.context.project_id, node_info.project)
+        self.assertEqual(self.context.domain_id, node_info.domain)
         self.assertEqual(1, node_info.index)
         self.assertEqual('first_node', node.role)
 
@@ -94,8 +94,8 @@ class TestNode(base.SenlinTestCase):
         self.assertEqual({}, node_info.data)
 
     def test_node_store_update(self):
-        node = nodem.Node('node1', PROFILE_ID, "", user=self.context.user,
-                          project=self.context.project)
+        node = nodem.Node('node1', PROFILE_ID, "", user=self.context.user_id,
+                          project=self.context.project_id)
         node_id = node.store(self.context)
 
         node.name = 'new_name'
@@ -306,7 +306,7 @@ class TestNode(base.SenlinTestCase):
         self.assertTrue(res)
         mock_status.assert_called_once_with(self.context, consts.NS_DELETING,
                                             "Deletion in progress")
-        self.assertFalse(mock_delete.called)
+        self.assertTrue(mock_delete.called)
         mock_db_delete.assert_called_once_with(self.context, node.id)
 
     @mock.patch.object(nodem.Node, 'set_status')
@@ -575,6 +575,26 @@ class TestNode(base.SenlinTestCase):
 
         self.assertFalse(res)
 
+    @mock.patch.object(nodem.Node, 'set_status')
+    @mock.patch.object(pb.Profile, 'check_object')
+    def test_node_check_no_server(self, mock_check, mock_status):
+        node = nodem.Node('node1', PROFILE_ID, '')
+        node.physical_id = 'd94d6333-82e6-4f87-b7ab-b786776df9d1'
+
+        err = exception.EServerNotFound(type='server',
+                                        id=node.physical_id,
+                                        message='No Server found')
+        mock_check.side_effect = err
+
+        res = node.do_check(self.context)
+
+        self.assertTrue(res)
+        mock_status.assert_called_once_with(
+            self.context, consts.NS_ERROR,
+            "Failed in found server '%s': No Server found."
+            % node.physical_id,
+            physical_id=None)
+
     @mock.patch.object(nodem.Node, 'set_status')
     @mock.patch.object(pb.Profile, 'recover_object')
     def test_node_recover_new_object(self, mock_recover, mock_status):
@@ -605,9 +625,9 @@ class TestNode(base.SenlinTestCase):
         self.assertEqual({'recovery': 'RECREATE'}, node.data)
         mock_status.assert_has_calls([
             mock.call(self.context, 'RECOVERING',
-                      reason='Recover in progress'),
+                      reason='Recovery in progress'),
             mock.call(self.context, consts.NS_ACTIVE,
-                      reason='Recover succeeded',
+                      reason='Recovery succeeded',
                       physical_id=new_id,
                       data={'recovery': 'RECREATE'})])
 
@@ -627,9 +647,9 @@ class TestNode(base.SenlinTestCase):
         self.assertEqual(PROFILE_ID, node.profile_id)
         mock_status.assert_has_calls([
             mock.call(self.context, 'RECOVERING',
-                      reason='Recover in progress'),
+                      reason='Recovery in progress'),
             mock.call(self.context, consts.NS_ACTIVE,
-                      reason='Recover succeeded')])
+                      reason='Recovery succeeded')])
 
     @mock.patch.object(nodem.Node, 'set_status')
     def test_node_recover_check_active(self, mock_status):
@@ -675,9 +695,9 @@ class TestNode(base.SenlinTestCase):
         self.assertEqual(PROFILE_ID, node.profile_id)
         mock_status.assert_has_calls([
             mock.call(self.context, 'RECOVERING',
-                      reason='Recover in progress'),
+                      reason='Recovery in progress'),
             mock.call(self.context, consts.NS_ACTIVE,
-                      reason='Recover succeeded',
+                      reason='Recovery succeeded',
                       physical_id=new_id,
                       data={'recovery': 'RECREATE'})])
 
@@ -711,9 +731,9 @@ class TestNode(base.SenlinTestCase):
         self.assertEqual(PROFILE_ID, node.profile_id)
         mock_status.assert_has_calls([
             mock.call(self.context, 'RECOVERING',
-                      reason='Recover in progress'),
+                      reason='Recovery in progress'),
             mock.call(self.context, consts.NS_ACTIVE,
-                      reason='Recover succeeded',
+                      reason='Recovery succeeded',
                       physical_id=new_id,
                       data={'recovery': 'RECREATE'})])
 
@@ -748,9 +768,9 @@ class TestNode(base.SenlinTestCase):
         self.assertEqual(PROFILE_ID, node.profile_id)
         mock_status.assert_has_calls([
             mock.call(self.context, 'RECOVERING',
-                      reason='Recover in progress'),
+                      reason='Recovery in progress'),
             mock.call(self.context, consts.NS_ACTIVE,
-                      reason='Recover succeeded',
+                      reason='Recovery succeeded',
                       physical_id=new_id,
                       data={'recovery': 'RECREATE'})])
 
@@ -788,9 +808,9 @@ class TestNode(base.SenlinTestCase):
         self.assertEqual(PROFILE_ID, node.profile_id)
         mock_status.assert_has_calls([
             mock.call(self.context, 'RECOVERING',
-                      reason='Recover in progress'),
+                      reason='Recovery in progress'),
             mock.call(self.context, consts.NS_ACTIVE,
-                      reason='Recover succeeded',
+                      reason='Recovery succeeded',
                       physical_id=new_id,
                       data={'recovery': 'RECREATE'})])
 
@@ -807,9 +827,9 @@ class TestNode(base.SenlinTestCase):
         self.assertFalse(res)
         mock_status.assert_has_calls([
             mock.call(self.context, 'RECOVERING',
-                      reason='Recover in progress'),
+                      reason='Recovery in progress'),
             mock.call(self.context, consts.NS_ERROR,
-                      reason='Recover failed')])
+                      reason='Recovery failed')])
 
     def test_node_recover_no_physical_id(self):
         node = nodem.Node('node1', PROFILE_ID, None)
