@@ -76,13 +76,13 @@ class TestMessageEvent(testtools.TestCase):
                         'inputs': '{}',
                         'name': 'fake_name',
                         'outputs': '{}',
-                        'project': self.ctx.project,
+                        'project': self.ctx.project_id,
                         'start_time': 1.23,
                         'status': 'RUNNING',
                         'status_reason': 'Good',
                         'target': cluster_id,
                         'timeout': 3600,
-                        'user': self.ctx.user,
+                        'user': self.ctx.user_id,
                     },
                     'senlin_object.name': 'ActionPayload',
                     'senlin_object.namespace': 'senlin',
@@ -169,13 +169,13 @@ class TestMessageEvent(testtools.TestCase):
                         'inputs': '{}',
                         'name': 'fake_name',
                         'outputs': '{}',
-                        'project': self.ctx.project,
+                        'project': self.ctx.project_id,
                         'start_time': 1.23,
                         'status': 'RUNNING',
                         'status_reason': 'Good',
                         'target': node_id,
                         'timeout': 3600,
-                        'user': self.ctx.user,
+                        'user': self.ctx.user_id,
                     },
                     'senlin_object.name': 'ActionPayload',
                     'senlin_object.namespace': 'senlin',
