@@ -17,7 +17,6 @@ import six
 import testtools
 
 from senlin.common import consts
-from senlin.objects import base
 from senlin.objects import fields as senlin_fields
 
 CONF = cfg.CONF
@@ -108,119 +107,6 @@ class TestBoolean(TestField):
         self.assertEqual('False', self.field.stringify(False))
 
 
-class TestObject(TestField):
-
-    def setUp(self):
-        super(TestObject, self).setUp()
-
-        @base.base.VersionedObjectRegistry.register
-        class TestableObject(base.base.VersionedObject):
-            fields = {
-                'uuid': fields.StringField(),
-            }
-
-        test_inst = TestableObject()
-        self._test_cls = TestableObject
-        self.field = fields.Field(senlin_fields.Object('TestableObject'))
-        self.coerce_good_values = [(test_inst, test_inst)]
-        self.coerce_bad_values = [1, 'foo']
-        self.to_primitive_values = [(test_inst, test_inst.obj_to_primitive())]
-        self.from_primitive_values = [(test_inst.obj_to_primitive(),
-                                       test_inst),
-                                      (test_inst, test_inst)]
-
-    def test_from_primitive(self):
-        # we ignore this test case
-        pass
-
-    def test_stringify(self):
-        # and this one is ignored as well
-        pass
-
-    def test_get_schema(self):
-        self.assertEqual(
-            {
-                'properties': {
-                    'versioned_object.changes': {
-                        'items': {'type': 'string'}, 'type': 'array'
-                    },
-                    'versioned_object.data': {
-                        'additionalProperties': False,
-                        'description': 'fields of TestableObject',
-                        'properties': {
-                            'uuid': {'readonly': False, 'type': ['string']}
-                        },
-                        'required': ['uuid'],
-                        'type': 'object',
-                    },
-                    'versioned_object.name': {
-                        'type': 'string',
-                    },
-                    'versioned_object.namespace': {
-                        'type': 'string',
-                    },
-                    'versioned_object.version': {
-                        'type': 'string',
-                    },
-                },
-                'readonly': False,
-                'required': [
-                    'versioned_object.namespace',
-                    'versioned_object.name',
-                    'versioned_object.version',
-                    'versioned_object.data',
-                ],
-                'type': 'object',
-            },
-            self.field.get_schema())
-
-
-class TestDummyObject(testtools.TestCase):
-
-    def setUp(self):
-        super(TestDummyObject, self).setUp()
-
-        @base.base.VersionedObjectRegistry.register
-        class TestableObject(base.base.VersionedObject):
-            fields = {}
-
-        self.field = fields.Field(senlin_fields.Object('TestableObject'))
-
-    def test_get_schema(self):
-        self.assertEqual(
-            {
-                'properties': {
-                    'versioned_object.changes': {
-                        'items': {'type': 'string'}, 'type': 'array'
-                    },
-                    'versioned_object.data': {
-                        'additionalProperties': False,
-                        'description': 'fields of TestableObject',
-                        'properties': {},
-                        'type': 'object',
-                    },
-                    'versioned_object.name': {
-                        'type': 'string',
-                    },
-                    'versioned_object.namespace': {
-                        'type': 'string',
-                    },
-                    'versioned_object.version': {
-                        'type': 'string',
-                    },
-                },
-                'readonly': False,
-                'required': [
-                    'versioned_object.namespace',
-                    'versioned_object.name',
-                    'versioned_object.version',
-                    'versioned_object.data',
-                ],
-                'type': 'object',
-            },
-            self.field.get_schema())
-
-
 class TestJson(TestField):
     def setUp(self):
         super(TestJson, self).setUp()
@@ -377,7 +263,10 @@ class TestName(TestField):
                                sot.coerce,
                                obj, 'attr', 'value/bad')
         self.assertEqual("The value for the 'attr' (value/bad) contains "
-                         "illegal characters.", six.text_type(ex))
+                         "illegal characters. It must contain only "
+                         "alphanumeric or \"_-.~\" characters and must start "
+                         "with letter.",
+                         six.text_type(ex))
 
     def test_get_schema(self):
         sot = senlin_fields.Name(2, 200)
@@ -425,21 +314,21 @@ class TestCapacity(TestField):
         self.assertEqual('100', self.field.stringify('100'))
 
     def test_init(self):
-        CONF.set_override('max_nodes_per_cluster', 300, enforce_type=True)
+        CONF.set_override('max_nodes_per_cluster', 300)
         sot = senlin_fields.Capacity()
 
         self.assertEqual(0, sot.minimum)
         self.assertEqual(300, sot.maximum)
 
     def test_init_with_values(self):
-        CONF.set_override('max_nodes_per_cluster', 300, enforce_type=True)
+        CONF.set_override('max_nodes_per_cluster', 300)
         sot = senlin_fields.Capacity(2, 200)
 
         self.assertEqual(2, sot.minimum)
         self.assertEqual(200, sot.maximum)
 
     def test_init_invalid(self):
-        CONF.set_override('max_nodes_per_cluster', 100, enforce_type=True)
+        CONF.set_override('max_nodes_per_cluster', 100)
 
         ex = self.assertRaises(ValueError,
                                senlin_fields.Capacity,
@@ -514,7 +403,7 @@ class TestCapacity(TestField):
         )
 
     def test_get_schema_default(self):
-        cfg.CONF.set_override('max_nodes_per_cluster', 100, enforce_type=True)
+        cfg.CONF.set_override('max_nodes_per_cluster', 100)
         sot = senlin_fields.Capacity()
         self.assertEqual(
             {
@@ -680,10 +569,10 @@ class TestIdentityList(TestField):
         )
 
 
-class TestAdjustmentType(TestField):
+class TestAdjustmentTypeField(TestField):
 
     def setUp(self):
-        super(TestAdjustmentType, self).setUp()
+        super(TestAdjustmentTypeField, self).setUp()
 
         self.field = senlin_fields.AdjustmentTypeField()
         self.coerce_good_values = [
@@ -700,11 +589,52 @@ class TestAdjustmentType(TestField):
         self.assertEqual("'EXACT_CAPACITY'",
                          self.field.stringify('EXACT_CAPACITY'))
 
+    def test_get_schema(self):
+        self.assertEqual(
+            {
+                'type': ['string'],
+                'readonly': False,
+                'enum': ['EXACT_CAPACITY', 'CHANGE_IN_CAPACITY',
+                         'CHANGE_IN_PERCENTAGE']
+            },
+            self.field.get_schema()
+        )
 
-class TestClusterActionName(TestField):
+
+class TestAdjustmentType(TestField):
+    def setUp(self):
+        super(TestAdjustmentType, self).setUp()
+
+        self.field = senlin_fields.AdjustmentType()
+        self.coerce_good_values = [
+            ('EXACT_CAPACITY', 'EXACT_CAPACITY'),
+            ('CHANGE_IN_CAPACITY', 'CHANGE_IN_CAPACITY'),
+            ('CHANGE_IN_PERCENTAGE', 'CHANGE_IN_PERCENTAGE')
+        ]
+        self.coerce_bad_values = ['BOGUS']
+
+        self.to_primitive_values = self.coerce_good_values[0:1]
+        self.from_primitive_values = self.coerce_good_values[0:1]
+
+    def test_stringify(self):
+        self.assertEqual("'EXACT_CAPACITY'",
+                         self.field.stringify('EXACT_CAPACITY'))
+
+    def test_get_schema(self):
+        self.assertEqual(
+            {
+                'type': ['string'],
+                'enum': ['EXACT_CAPACITY', 'CHANGE_IN_CAPACITY',
+                         'CHANGE_IN_PERCENTAGE']
+            },
+            self.field.get_schema()
+        )
+
+
+class TestClusterActionNameField(TestField):
 
     def setUp(self):
-        super(TestClusterActionName, self).setUp()
+        super(TestClusterActionNameField, self).setUp()
         self.field = senlin_fields.ClusterActionNameField()
         self.coerce_good_values = [
             (action, action) for action in consts.CLUSTER_ACTION_NAMES]
@@ -717,11 +647,61 @@ class TestClusterActionName(TestField):
         self.assertEqual("'CLUSTER_RESIZE'",
                          self.field.stringify('CLUSTER_RESIZE'))
 
+    def test_get_schema(self):
+        self.assertEqual(
+            {
+                'type': ['string'],
+                'readonly': False,
+                'enum': ['CLUSTER_CREATE', 'CLUSTER_DELETE',
+                         'CLUSTER_UPDATE', 'CLUSTER_ADD_NODES',
+                         'CLUSTER_DEL_NODES', 'CLUSTER_RESIZE',
+                         'CLUSTER_CHECK', 'CLUSTER_RECOVER',
+                         'CLUSTER_REPLACE_NODES', 'CLUSTER_SCALE_OUT',
+                         'CLUSTER_SCALE_IN', 'CLUSTER_ATTACH_POLICY',
+                         'CLUSTER_DETACH_POLICY', 'CLUSTER_UPDATE_POLICY',
+                         'CLUSTER_OPERATION']
+            },
+            self.field.get_schema()
+        )
 
-class TestReceiverType(TestField):
+
+class TestClusterActionName(TestField):
 
     def setUp(self):
-        super(TestReceiverType, self).setUp()
+        super(TestClusterActionName, self).setUp()
+        self.field = senlin_fields.ClusterActionName()
+        self.coerce_good_values = [
+            (action, action) for action in consts.CLUSTER_ACTION_NAMES]
+        self.coerce_bad_values = ['BOGUS']
+
+        self.to_primitive_values = self.coerce_good_values[0:1]
+        self.from_primitive_values = self.coerce_good_values[0:1]
+
+    def test_stringify(self):
+        self.assertEqual("'CLUSTER_RESIZE'",
+                         self.field.stringify('CLUSTER_RESIZE'))
+
+    def test_get_schema(self):
+        self.assertEqual(
+            {
+                'type': ['string'],
+                'enum': ['CLUSTER_CREATE', 'CLUSTER_DELETE',
+                         'CLUSTER_UPDATE', 'CLUSTER_ADD_NODES',
+                         'CLUSTER_DEL_NODES', 'CLUSTER_RESIZE',
+                         'CLUSTER_CHECK', 'CLUSTER_RECOVER',
+                         'CLUSTER_REPLACE_NODES', 'CLUSTER_SCALE_OUT',
+                         'CLUSTER_SCALE_IN', 'CLUSTER_ATTACH_POLICY',
+                         'CLUSTER_DETACH_POLICY', 'CLUSTER_UPDATE_POLICY',
+                         'CLUSTER_OPERATION']
+            },
+            self.field.get_schema()
+        )
+
+
+class TestReceiverTypeField(TestField):
+
+    def setUp(self):
+        super(TestReceiverTypeField, self).setUp()
         self.field = senlin_fields.ReceiverTypeField()
         self.coerce_good_values = [
             (action, action) for action in consts.RECEIVER_TYPES]
@@ -733,3 +713,39 @@ class TestReceiverType(TestField):
     def test_stringify(self):
         self.assertEqual("'message'",
                          self.field.stringify('message'))
+
+    def test_get_schema(self):
+        self.assertEqual(
+            {
+                'type': ['string'],
+                'readonly': False,
+                'enum': ['webhook', 'message']
+            },
+            self.field.get_schema()
+        )
+
+
+class TestReceiverType(TestField):
+
+    def setUp(self):
+        super(TestReceiverType, self).setUp()
+        self.field = senlin_fields.ReceiverType()
+        self.coerce_good_values = [
+            (action, action) for action in consts.RECEIVER_TYPES]
+        self.coerce_bad_values = ['BOGUS']
+
+        self.to_primitive_values = self.coerce_good_values[0:1]
+        self.from_primitive_values = self.coerce_good_values[0:1]
+
+    def test_stringify(self):
+        self.assertEqual("'message'",
+                         self.field.stringify('message'))
+
+    def test_get_schema(self):
+        self.assertEqual(
+            {
+                'type': ['string'],
+                'enum': ['webhook', 'message']
+            },
+            self.field.get_schema()
+        )
