@@ -84,3 +84,38 @@ class TestHealthRegistry(testtools.TestCase):
         self.assertEqual(x_obj, result)
         mock_get.assert_called_once_with(self.ctx, 'FAKE')
         mock_from.assert_called_once_with(self.ctx, mock.ANY, x_registry)
+
+    @mock.patch.object(base.SenlinObject, '_from_db_object')
+    @mock.patch.object(db_api, 'registry_get_by_param')
+    def test_get_by_engine(self, mock_get, mock_from):
+        x_registry = mock.Mock()
+        x_registry.cluster_id = 'FAKE'
+        x_registry.engine_id = 'FAKE_ENGINE'
+        mock_get.return_value = x_registry
+
+        x_obj = mock.Mock()
+        mock_from.return_value = x_obj
+
+        result = hro.HealthRegistry.get_by_engine(
+            self.ctx, 'FAKE_ENGINE', 'FAKE')
+
+        self.assertEqual(x_obj, result)
+        mock_get.assert_called_once_with(
+            self.ctx, {"cluster_id": "FAKE", "engine_id": "FAKE_ENGINE"})
+        mock_from.assert_called_once_with(self.ctx, mock.ANY, x_registry)
+
+    @mock.patch.object(hro.HealthRegistry, 'update')
+    def test_disable(self, mock_update):
+        hro.HealthRegistry.disable_registry(
+            self.ctx, "FAKE_ID")
+
+        mock_update.assert_called_once_with(
+            self.ctx, "FAKE_ID", {"enabled": False})
+
+    @mock.patch.object(hro.HealthRegistry, 'update')
+    def test_enable(self, mock_update):
+        hro.HealthRegistry.enable_registry(
+            self.ctx, "FAKE_ID")
+
+        mock_update.assert_called_once_with(
+            self.ctx, "FAKE_ID", {"enabled": True})
