@@ -100,8 +100,8 @@ class TestNode(base.SenlinTestCase):
             'name': 'test_node',
             'profile_id': PROFILE_ID,
             'cluster_id': CLUSTER_ID,
-            'user': self.ctx.user,
-            'project': self.ctx.project,
+            'user': self.ctx.user_id,
+            'project': self.ctx.project_id,
             'index': -1,
             'init_at': timeutils.utcnow(True),
             'status': 'Initializing',
