@@ -36,6 +36,7 @@ class TestLoadBalancingPolicy(base.SenlinTestCase):
             'version': '1.0',
             'properties': {
                 'pool': {
+                    'id': '',
                     'protocol': 'HTTP',
                     'protocol_port': 80,
                     'subnet': 'internal-subnet',
@@ -71,6 +72,7 @@ class TestLoadBalancingPolicy(base.SenlinTestCase):
         self.patchobject(driver_base, 'SenlinDriver', return_value=self.sd)
         self.lb_driver = mock.Mock()
         self.net_driver = mock.Mock()
+        self.octavia_driver = mock.Mock()
 
     @mock.patch.object(lb_policy.LoadBalancingPolicy, 'validate')
     def test_init(self, mock_validate):
@@ -97,6 +99,7 @@ class TestLoadBalancingPolicy(base.SenlinTestCase):
             'version': '1.0',
             'properties': {
                 'pool': {
+                    'id': None,
                     'protocol': 'HTTP',
                     'protocol_port': 80,
                     'subnet': 'internal-subnet',
@@ -127,6 +130,38 @@ class TestLoadBalancingPolicy(base.SenlinTestCase):
                          policy.lb_status_timeout)
         self.assertIsNone(policy.lb)
 
+    def test_loadbalancer_value(self):
+        spec = {
+            'type': 'senlin.policy.loadbalance',
+            'version': '1.0',
+            'properties': {
+                'loadbalancer': 'LB_ID',
+                'pool': {
+                    'id': 'POOL_ID',
+                    'subnet': 'internal-subnet'
+                },
+                'vip': {
+                    'address': '192.168.1.100',
+                    'subnet': 'external-subnet'
+                },
+                'health_monitor': {
+                    'id': 'HM_ID'
+                }
+            }
+        }
+        self.spec['properties']['pool']['id'] = 'POOL_ID'
+        self.spec['properties']['health_monitor']['id'] = 'HM_ID'
+        self.spec['properties']['loadbalancer'] = 'LB_ID'
+        self.spec['properties']['pool']['session_persistence'] = {}
+        self.spec['properties']['vip']['connection_limit'] = -1
+        policy = lb_policy.LoadBalancingPolicy('test-policy', spec)
+        self.assertIsNone(policy.id)
+        self.assertEqual('test-policy', policy.name)
+        self.assertEqual('senlin.policy.loadbalance-1.0', policy.type)
+        self.assertEqual(self.spec['properties']['pool'], policy.pool_spec)
+        self.assertEqual(self.spec['properties']['vip'], policy.vip_spec)
+        self.assertEqual(self.spec['properties']['loadbalancer'], policy.lb)
+
     @mock.patch.object(policy_base.Policy, 'validate')
     def test_validate_shallow(self, mock_validate):
         policy = lb_policy.LoadBalancingPolicy('test-policy', self.spec)
@@ -141,6 +176,7 @@ class TestLoadBalancingPolicy(base.SenlinTestCase):
     def test_validate_pool_subnet_notfound(self, mock_validate):
         policy = lb_policy.LoadBalancingPolicy('test-policy', self.spec)
         policy._networkclient = self.net_driver
+        policy._octaviaclient = self.octavia_driver
         ctx = mock.Mock(user='user1', project='project1')
         self.net_driver.subnet_get = mock.Mock(
             side_effect=exc.InternalError(code='404', message='not found'))
@@ -156,6 +192,7 @@ class TestLoadBalancingPolicy(base.SenlinTestCase):
     def test_validate_vip_subnet_notfound(self, mock_validate):
         policy = lb_policy.LoadBalancingPolicy('test-policy', self.spec)
         policy._networkclient = self.net_driver
+        policy._octaviaclient = self.octavia_driver
         ctx = mock.Mock(user='user1', project='project1')
         self.net_driver.subnet_get = mock.Mock(
             side_effect=[
@@ -173,6 +210,23 @@ class TestLoadBalancingPolicy(base.SenlinTestCase):
         self.assertEqual("The specified subnet 'external-subnet' could not "
                          "be found.", six.text_type(ex))
 
+    @mock.patch.object(policy_base.Policy, 'validate')
+    def test_validate_loadbalancer_notfund(self, mock_validate):
+        self.spec['properties']['loadbalancer'] = "LB_ID"
+        policy = lb_policy.LoadBalancingPolicy('test-policy', self.spec)
+        policy._networkclient = self.net_driver
+        policy._octaviaclient = self.octavia_driver
+        ctx = mock.Mock(user='user1', project='project1')
+        self.octavia_driver.loadbalancer_get = mock.Mock(
+            side_effect=exc.InternalError(code='404', message='not found'))
+
+        ex = self.assertRaises(exc.InvalidSpec, policy.validate, ctx, True)
+
+        mock_validate.assert_called_with(ctx, True)
+        self.octavia_driver.loadbalancer_get.assert_called_once_with('LB_ID')
+        self.assertEqual("The specified loadbalancer 'LB_ID' could not "
+                         "be found.", six.text_type(ex))
+
     @mock.patch.object(lb_policy.LoadBalancingPolicy, '_build_policy_data')
     @mock.patch.object(policy_base.Policy, 'attach')
     @mock.patch.object(no.Node, 'update')
@@ -523,8 +577,105 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         self.lb_driver.lb_delete.assert_called_once_with(**policy_data)
         self.assertEqual({}, cluster.data)
 
+    @mock.patch.object(lb_policy.LoadBalancingPolicy, '_remove_member')
+    def test_detach_existed_lbass_succeeded(self, m_remove, m_extract,
+                                            m_load):
+        cp = mock.Mock()
+        policy_data = {
+            'loadbalancer': 'LB_ID',
+            'listener': 'LISTENER_ID',
+            'pool': 'POOL_ID',
+            'healthmonitor': 'HM_ID',
+            'preexisting': True,
+        }
+        cp_data = {
+            'LoadBalancingPolicy': {
+                'version': '1.0',
+                'data': policy_data
+            }
+        }
+        cp.data = cp_data
+        m_load.return_value = cp
+        m_extract.return_value = policy_data
+
+        policy = lb_policy.LoadBalancingPolicy('test-policy', self.spec)
+        policy._lbaasclient = self.lb_driver
+        cluster = mock.Mock(
+            id='CLUSTER_ID',
+            data={
+                'loadbalancers': {
+                    policy.id: {'vip_address': '192.168.1.100'}
+
+                }
+            })
+        node = mock.Mock(id='fake', data={})
+        cluster.nodes = [node]
+        m_remove.return_value = []
+
+        res, data = policy.detach(cluster)
+
+        self.assertTrue(res)
+        self.assertEqual('LB resources deletion succeeded.', data)
+        m_load.assert_called_once_with(mock.ANY, cluster.id, policy.id)
+        m_extract.assert_called_once_with(cp_data)
+        m_remove.assert_called_with(mock.ANY, ['fake'], cp, self.lb_driver)
+        self.assertEqual({}, cluster.data)
+
+    @mock.patch.object(lb_policy.LoadBalancingPolicy, '_remove_member')
+    def test_detach_existed_lbass_failed(self, m_remove, m_extract, m_load):
+        cp = mock.Mock()
+        policy_data = {
+            'loadbalancer': 'LB_ID',
+            'listener': 'LISTENER_ID',
+            'pool': 'POOL_ID',
+            'healthmonitor': 'HM_ID',
+            'preexisting': True,
+        }
+        cp_data = {
+            'LoadBalancingPolicy': {
+                'version': '1.0',
+                'data': policy_data
+            }
+        }
+        cp.data = cp_data
+        m_load.return_value = cp
+        m_extract.return_value = policy_data
+
+        policy = lb_policy.LoadBalancingPolicy('test-policy', self.spec)
+        policy._lbaasclient = self.lb_driver
+        cluster = mock.Mock(
+            id='CLUSTER_ID',
+            data={
+                'loadbalancers': {
+                    policy.id: {'vip_address': '192.168.1.100'}
+
+                }
+            })
+        node1 = mock.Mock(id='node1', data={})
+        node2 = mock.Mock(id='node2', data={})
+        cluster.nodes = [node1, node2]
+        m_remove.return_value = [node2.id]
+
+        res, data = policy.detach(cluster)
+
+        self.assertFalse(res)
+        self.assertEqual('Failed to remove servers from existed LB.', data)
+        m_load.assert_called_once_with(mock.ANY, cluster.id, policy.id)
+        m_extract.assert_called_once_with(cp_data)
+        m_remove.assert_called_with(mock.ANY, ['node1', 'node2'], cp,
+                                    self.lb_driver)
+        self.assertEqual({
+            'loadbalancers': {
+                None: {'vip_address': '192.168.1.100'}
+            }},
+            cluster.data)
+
     def test_detach_failed_lb_delete(self, m_extract, m_load):
         cluster = mock.Mock()
+        policy_data = {
+            'preexisting': False,
+        }
+        m_extract.return_value = policy_data
         self.lb_driver.lb_delete.return_value = (False, 'lb_delete failed.')
 
         policy = lb_policy.LoadBalancingPolicy('test-policy', self.spec)
@@ -579,7 +730,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
 
         # do it
         candidates = ['NODE1_ID', 'NODE2_ID']
-        res = policy._add_member(candidates, cp, action, self.lb_driver)
+        res = policy._add_member(action.context, candidates,
+                                 cp, self.lb_driver)
 
         # assertions
         self.assertEqual([], res)
@@ -634,7 +786,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
 
         # do it
         candidates = ['NODE1_ID']
-        res = policy._add_member(candidates, cp, action, self.lb_driver)
+        res = policy._add_member(action.context, candidates,
+                                 cp, self.lb_driver)
 
         # assertions
         self.assertEqual(['NODE1_ID'], res)
@@ -671,7 +824,7 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         self.assertIsNone(res)
         m_get.assert_called_once_with(action)
         m_load.assert_called_once_with(ctx, cid, policy.id)
-        m_add.assert_called_once_with(['NODE_ID'], cp, action, self.lb_driver)
+        m_add.assert_called_once_with(ctx, ['NODE_ID'], cp, self.lb_driver)
         self.assertFalse(m_remove.called)
 
     @mock.patch.object(lb_policy.LoadBalancingPolicy, '_add_member')
@@ -703,7 +856,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         self.assertIsNone(res)
         m_get.assert_called_once_with(action)
         m_load.assert_called_once_with('action_context', cid, policy.id)
-        m_add.assert_called_once_with(candidates, cp, action, self.lb_driver)
+        m_add.assert_called_once_with(action.context, candidates,
+                                      cp, self.lb_driver)
         self.assertFalse(m_remove.called)
 
     @mock.patch.object(lb_policy.LoadBalancingPolicy, '_add_member')
@@ -732,7 +886,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         self.assertIsNone(res)
         m_get.assert_called_once_with(action)
         m_load.assert_called_once_with('action_context', cid, policy.id)
-        m_add.assert_called_once_with(['NODE1'], cp, action, self.lb_driver)
+        m_add.assert_called_once_with(action.context, ['NODE1'],
+                                      cp, self.lb_driver)
         m_recovery.assert_called_once_with(['NODE1'], cp, self.lb_driver,
                                            action)
 
@@ -760,7 +915,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         self.assertEqual("Failed in adding nodes into lb pool: "
                          "['NODE1_ID']", action.data['reason'])
         m_get.assert_called_once_with(action)
-        m_add.assert_called_once_with(['NODE1_ID'], cp, action, self.lb_driver)
+        m_add.assert_called_once_with(action.context, ['NODE1_ID'],
+                                      cp, self.lb_driver)
         self.assertFalse(m_remove.called)
 
     @mock.patch.object(no.Node, 'get')
@@ -797,8 +953,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         policy._lbaasclient = self.lb_driver
 
         candidates = [node1.id, node2.id]
-        res = policy._remove_member(
-            candidates, cp, action, self.lb_driver)
+        res = policy._remove_member(action.context, candidates,
+                                    cp, self.lb_driver)
 
         m_extract.assert_called_once_with(cp_data)
         calls_node_get = [
@@ -852,7 +1008,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         policy._lbaasclient = self.lb_driver
 
         candidates = [node1.id, node2.id]
-        res = policy._remove_member(candidates, cp, action, self.lb_driver)
+        res = policy._remove_member(action.context, candidates,
+                                    cp, self.lb_driver)
 
         m_extract.assert_called_once_with(cp_data)
         calls_node_get = [
@@ -899,7 +1056,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         policy._lbaasclient = self.lb_driver
 
         candidates = [node1.id]
-        res = policy._remove_member(candidates, cp, action, self.lb_driver)
+        res = policy._remove_member(action.context, candidates,
+                                    cp, self.lb_driver)
 
         m_extract.assert_called_once_with(cp_data)
         m_node_get.assert_called_once_with(action.context, node_id='NODE1')
@@ -958,8 +1116,8 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         self.assertEqual("Failed in removing deleted node(s) from lb pool: "
                          "['NODE1_ID']", action.data['reason'])
 
-        m_remove.assert_called_once_with(
-            ['NODE1_ID'], mock.ANY, action, self.lb_driver)
+        m_remove.assert_called_once_with(action.context, ['NODE1_ID'],
+                                         mock.ANY, self.lb_driver)
 
     @mock.patch.object(no.Node, 'update')
     def test__process_recovery_not_lb_member(self, m_update, m1, m2):
@@ -1012,6 +1170,6 @@ class TestLoadBalancingPolicyOperations(base.SenlinTestCase):
         res = policy._process_recovery(['NODE'], cp, self.lb_driver, action)
 
         self.assertEqual(['NODE'], res)
-        m_remove.assert_called_once_with(['NODE'], cp, action, self.lb_driver,
-                                         handle_err=False)
+        m_remove.assert_called_once_with(action.context, ['NODE'], cp,
+                                         self.lb_driver, handle_err=False)
         m_update.assert_called_once_with(action.context, {'data': {}})
