@@ -178,7 +178,7 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         self.assertEqual(msg, ex.message)
 
     @mock.patch.object(node.Node, 'load')
-    def test__get_host_node_found(self, mock_load):
+    def test__get_host_node_found_by_node(self, mock_load):
         node = mock.Mock()
         mock_load.return_value = node
         ctx = mock.Mock()
@@ -189,6 +189,18 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         self.assertEqual(node, res)
         mock_load.assert_called_once_with(ctx, node_id='host_node')
 
+    @mock.patch.object(dp.DockerProfile, '_get_random_node')
+    def test__get_host_node_found_by_cluster(self, mock_get):
+        node = mock.Mock()
+        mock_get.return_value = node
+        ctx = mock.Mock()
+        profile = dp.DockerProfile('container', self.spec)
+
+        res = profile._get_host(ctx, None, 'host_cluster')
+
+        self.assertEqual(node, res)
+        mock_get.assert_called_once_with(ctx, 'host_cluster')
+
     @mock.patch.object(node.Node, 'load')
     def test__get_host_node_not_found(self, mock_load):
         mock_load.side_effect = exc.ResourceNotFound(type='node',
@@ -203,18 +215,29 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         msg = _("The host node 'fake_node' could not be found.")
         self.assertEqual(msg, ex.message)
 
+    @mock.patch.object(node.Node, 'load')
+    @mock.patch.object(no.Node, 'get_all_by_cluster')
     @mock.patch.object(cluster.Cluster, 'load')
-    def test__get_random_node(self, mock_cluster):
-        node1 = mock.Mock(status='ERROR')
-        node2 = mock.Mock(status='ACTIVE')
-        node3 = mock.Mock(status='ACTIVE')
-        cluster = mock.Mock(rt={'nodes': [node1, node2, node3]})
+    def test__get_random_node(self, mock_cluster, mock_nodes, mock_load):
+        cluster = mock.Mock()
         mock_cluster.return_value = cluster
-        active_nodes = [node2, node3]
+        node1 = mock.Mock()
+        node2 = mock.Mock()
+        mock_nodes.return_value = [node1, node2]
         profile = dp.DockerProfile('container', self.spec)
         ctx = mock.Mock()
-        node = profile._get_random_node(ctx, 'host_cluster')
-        self.assertIn(node, active_nodes)
+        x_node = mock.Mock()
+        mock_load.return_value = x_node
+
+        res = profile._get_random_node(ctx, 'host_cluster')
+
+        self.assertEqual(x_node, res)
+        mock_cluster.assert_called_once_with(ctx, cluster_id='host_cluster')
+        mock_nodes.assert_called_once_with(ctx, cluster_id='host_cluster',
+                                           filters={'status': 'ACTIVE'})
+        mock_load.assert_called_once_with(ctx, db_node=mock.ANY)
+        n = mock_load.call_args[1]['db_node']
+        self.assertIn(n, [node1, node2])
 
     @mock.patch.object(cluster.Cluster, 'load')
     def test__get_random_node_cluster_not_found(self, mock_load):
@@ -230,34 +253,23 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         msg = _("The host cluster 'host_cluster' could not be found.")
         self.assertEqual(msg, ex.message)
 
+    @mock.patch.object(no.Node, 'get_all_by_cluster')
     @mock.patch.object(cluster.Cluster, 'load')
-    def test__get_random_node_empty_cluster(self, mock_cluster):
-        cluster = mock.Mock(rt={'nodes': []})
+    def test__get_random_node_empty_cluster(self, mock_cluster, mock_nodes):
+        cluster = mock.Mock()
         mock_cluster.return_value = cluster
+        mock_nodes.return_value = []
         profile = dp.DockerProfile('container', self.spec)
         ctx = mock.Mock()
+
         ex = self.assertRaises(exc.InternalError,
                                profile._get_random_node,
                                ctx, 'host_cluster')
-        msg = _('The cluster (host_cluster) contains no nodes')
 
+        msg = _('The cluster (host_cluster) contains no active nodes')
         self.assertEqual(msg, ex.message)
-
-    @mock.patch.object(cluster.Cluster, 'load')
-    def test__get_random_node_no_active_nodes(self, mock_cluster):
-        node1 = mock.Mock(status='ERROR')
-        node2 = mock.Mock(status='ERROR')
-        node3 = mock.Mock(status='ERROR')
-        cluster = mock.Mock(rt={'nodes': [node1, node2, node3]})
-        mock_cluster.return_value = cluster
-        profile = dp.DockerProfile('container', self.spec)
-        ctx = mock.Mock()
-        ex = self.assertRaises(exc.InternalError,
-                               profile._get_random_node,
-                               ctx, 'host_cluster')
-        msg = _('There is no active nodes running in the cluster '
-                '(host_cluster)')
-        self.assertEqual(msg, ex.message)
+        mock_nodes.assert_called_once_with(ctx, cluster_id='host_cluster',
+                                           filters={'status': 'ACTIVE'})
 
     def test_get_host_ip_nova_server(self):
         addresses = {
@@ -314,7 +326,7 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         ex = self.assertRaises(exc.InvalidSpec,
                                profile.do_validate, obj)
 
-        self.assertEqual("Either 'host_cluster' or 'host_node' should be "
+        self.assertEqual("Either 'host_cluster' or 'host_node' must be "
                          "specified, but not both.", six.text_type(ex))
 
     def test_do_validate_with_neither_cluster_or_node(self):
@@ -326,7 +338,7 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         ex = self.assertRaises(exc.InvalidSpec,
                                profile.do_validate, obj)
 
-        self.assertEqual("Either 'host_cluster' or 'host_node' should be "
+        self.assertEqual("Either 'host_cluster' or 'host_node' must be "
                          "specified.", six.text_type(ex))
 
     @mock.patch.object(no.Node, 'find')
@@ -386,7 +398,7 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         mock_find.assert_called_once_with(profile.context, 'fake_cluster')
 
     @mock.patch.object(db_api, 'node_add_dependents')
-    @mock.patch.object(context, 'get_admin_context')
+    @mock.patch.object(context, 'get_service_context')
     @mock.patch.object(dp.DockerProfile, 'docker')
     def test_do_create(self, mock_docker, mock_ctx, mock_add):
         ctx = mock.Mock()
@@ -402,7 +414,9 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         profile.cluster = cluster
         profile.id = 'profile_id'
         obj = mock.Mock(id='fake_con_id')
+
         ret_container_id = profile.do_create(obj)
+
         mock_add.assert_called_once_with(ctx, 'node_id', 'fake_con_id')
         self.assertEqual(container_id, ret_container_id)
         params = {
@@ -412,14 +426,20 @@ class TestContainerDockerProfile(base.SenlinTestCase):
         }
         dockerclient.container_create.assert_called_once_with(**params)
 
+    @mock.patch.object(context, 'get_service_context')
     @mock.patch.object(dp.DockerProfile, 'docker')
-    def test_do_create_failed(self, mock_docker):
+    def test_do_create_failed(self, mock_docker, mock_ctx):
+        mock_ctx.return_value = mock.Mock()
         mock_docker.side_effect = exc.InternalError
+
         profile = dp.DockerProfile('container', self.spec)
+
         obj = mock.Mock()
         self.assertRaises(exc.EResourceCreation,
                           profile.do_create, obj)
 
+        mock_ctx.assert_called_once_with(project=obj.project, user=obj.user)
+
     @mock.patch.object(context, 'get_admin_context')
     @mock.patch.object(db_api, 'node_remove_dependents')
     @mock.patch.object(dp.DockerProfile, 'docker')
