@@ -491,7 +491,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
         cc.wait_for_server.side_effect = err
         ex = self.assertRaises(exc.EResourceCreation, profile.do_create,
                                node_obj)
-
+        self.assertEqual('FAKE_ID', ex.resource_id)
         self.assertEqual('Failed in creating server: TIMEOUT.',
                          six.text_type(ex))
         cc.wait_for_server.assert_called_once_with('FAKE_ID')
@@ -522,6 +522,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
         # assertions
         self.assertEqual('Failed in creating server: creation failed.',
                          six.text_type(ex))
+        self.assertIsNone(ex.resource_id)
         self.assertEqual(0, cc.wait_for_server.call_count)
         self.assertEqual(0, mock_zone_info.call_count)
 
@@ -541,6 +542,22 @@ class TestNovaServerBasic(base.SenlinTestCase):
         cc.server_delete.assert_called_once_with('FAKE_ID', True)
         cc.wait_for_server_delete.assert_called_once_with('FAKE_ID')
 
+    def test_do_delete_no_physical_id(self):
+        profile = server.ServerProfile('t', self.spec)
+
+        cc = mock.Mock()
+        profile._computeclient = cc
+
+        test_server = mock.Mock(physical_id=None)
+
+        # do it
+        res = profile.do_delete(test_server)
+
+        # assertions
+        self.assertTrue(res)
+        self.assertFalse(cc.server_delete.called)
+        self.assertFalse(cc.wait_for_server_delete.called)
+
     @mock.patch.object(node_ob.Node, 'update')
     def test_do_delete_ports_ok(self, mock_node_obj):
         profile = server.ServerProfile('t', self.spec)
@@ -589,14 +606,6 @@ class TestNovaServerBasic(base.SenlinTestCase):
         cc.server_force_delete.assert_called_once_with('FAKE_ID', False)
         cc.wait_for_server_delete.assert_called_once_with('FAKE_ID')
 
-    def test_do_delete_no_physical_id(self):
-        profile = server.ServerProfile('t', self.spec)
-        test_server = mock.Mock(physical_id=None)
-
-        res = profile.do_delete(test_server)
-
-        self.assertTrue(res)
-
     def test_do_delete_with_delete_failure(self):
         cc = mock.Mock()
         profile = server.ServerProfile('t', self.spec)
@@ -695,6 +704,9 @@ class TestNovaServerBasic(base.SenlinTestCase):
                     'rel': 'bookmark'
                 }],
             },
+            'attached_volumes': [{
+                'id': 'FAKE_VOLUME',
+            }],
             'key_name': 'FAKE_KEY',
             'links': [{
                 'href': 'http://url1',
@@ -705,7 +717,6 @@ class TestNovaServerBasic(base.SenlinTestCase):
             }],
             'metadata': {},
             'name': 'FAKE_NAME',
-            'os-extended-volumes:volumes_attached': [],
             'progress': 0,
             'security_groups': [{'name': 'default'}],
             'status': 'FAKE_STATUS',
@@ -731,10 +742,10 @@ class TestNovaServerBasic(base.SenlinTestCase):
             'hostId': 'FAKE_HOST_ID',
             'id': 'FAKE_ID',
             'image': 'FAKE_IMAGE',
+            'attached_volumes': ['FAKE_VOLUME'],
             'key_name': 'FAKE_KEY',
             'metadata': {},
             'name': 'FAKE_NAME',
-            'os-extended-volumes:volumes_attached': [],
             'addresses': {
                 'private': [{
                     'OS-EXT-IPS-MAC:mac_addr': 'fa:16:3e:5e:00:81',
@@ -768,6 +779,9 @@ class TestNovaServerBasic(base.SenlinTestCase):
             'image': {
                 'id': 'FAKE_IMAGE',
             },
+            'attached_volumes': [{
+                'id': 'FAKE_VOLUME',
+            }],
             'security_groups': [],
         }
         cc.server_get.return_value = nova_server
@@ -776,6 +790,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
             'flavor': 'FAKE_FLAVOR',
             'id': 'FAKE_ID',
             'image': 'FAKE_IMAGE',
+            'attached_volumes': ['FAKE_VOLUME'],
             'addresses': {},
             'security_groups': '',
         }
@@ -802,6 +817,9 @@ class TestNovaServerBasic(base.SenlinTestCase):
             },
             'id': 'FAKE_ID',
             'image': {},
+            'attached_volumes': [{
+                'id': 'FAKE_VOLUME',
+            }],
             'security_groups': [{'name': 'default'}],
         }
         cc.server_get.return_value = nova_server
@@ -810,6 +828,48 @@ class TestNovaServerBasic(base.SenlinTestCase):
             'flavor': 'FAKE_FLAVOR',
             'id': 'FAKE_ID',
             'image': {},
+            'attached_volumes': ['FAKE_VOLUME'],
+            'addresses': {
+                'private': [{
+                    'version': 4,
+                    'addr': '10.0.0.3',
+                }]
+            },
+            'security_groups': 'default',
+        }
+        self.assertEqual(expected, res)
+        cc.server_get.assert_called_once_with('FAKE_ID')
+
+    def test_do_get_details_bdm_no_id_key(self):
+        cc = mock.Mock()
+        profile = server.ServerProfile('t', self.spec)
+        profile._computeclient = cc
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        # Test normal path
+        nova_server = mock.Mock()
+        nova_server.to_dict.return_value = {
+            'addresses': {
+                'private': [{
+                    'version': 4,
+                    'addr': '10.0.0.3',
+                }]
+            },
+            'flavor': {
+                'id': 'FAKE_FLAVOR',
+            },
+            'id': 'FAKE_ID',
+            'image': {},
+            'attached_volumes': [],
+            'security_groups': [{'name': 'default'}],
+        }
+        cc.server_get.return_value = nova_server
+        res = profile.do_get_details(node_obj)
+        expected = {
+            'flavor': 'FAKE_FLAVOR',
+            'id': 'FAKE_ID',
+            'image': {},
+            'attached_volumes': [],
             'addresses': {
                 'private': [{
                     'version': 4,
@@ -850,6 +910,9 @@ class TestNovaServerBasic(base.SenlinTestCase):
             'image': {
                 'id': 'FAKE_IMAGE',
             },
+            'attached_volumes': [{
+                'id': 'FAKE_VOLUME',
+            }],
             'security_groups': [{
                 'name': 'default',
             }, {
@@ -862,6 +925,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
             'flavor': 'FAKE_FLAVOR',
             'id': 'FAKE_ID',
             'image': 'FAKE_IMAGE',
+            'attached_volumes': ['FAKE_VOLUME'],
             'addresses': {
                 'private': [{
                     'version': 4,
@@ -926,12 +990,26 @@ class TestNovaServerBasic(base.SenlinTestCase):
             },
             addresses={
                 "NET1": [{
+                    "OS-EXT-IPS-MAC:mac_addr": "fa:16:3e:09:6f:d8",
                     "OS-EXT-IPS:type": "fixed",
-                    "addr": "ADDR1"
+                    "addr": "ADDR1_IPv4",
+                    "version": 4
+                }, {
+                    "OS-EXT-IPS-MAC:mac_addr": "fa:16:3e:09:6f:d8",
+                    "OS-EXT-IPS:type": "fixed",
+                    "addr": "ADDR1_IPv6",
+                    "version": 6
                 }],
                 "NET2": [{
+                    "OS-EXT-IPS-MAC:mac_addr": "aa:e6:3e:09:6f:db",
                     "OS-EXT-IPS:type": "fixed",
-                    "addr": "ADDR2"
+                    "addr": "ADDR2_IPv4",
+                    "version": 4
+                }, {
+                    "OS-EXT-IPS-MAC:mac_addr": "aa:e6:3e:09:6f:db",
+                    "OS-EXT-IPS:type": "fixed",
+                    "addr": "ADDR2_IPv6",
+                    "version": 6
                 }],
             },
             security_groups=[{'name': 'GROUP1'}, {'name': 'GROUP2'}]
@@ -952,6 +1030,8 @@ class TestNovaServerBasic(base.SenlinTestCase):
         self.assertEqual('IMAGE_ID', res['image'])
         self.assertEqual('FAKE_KEY', res['key_name'])
         self.assertEqual({'mkey': 'mvalue'}, res['metadata'])
+
+        self.assertEqual(2, len(res['networks']))
         self.assertIn({'network': 'NET1'}, res['networks'])
         self.assertIn({'network': 'NET2'}, res['networks'])
         self.assertIn('GROUP1', res['security_groups'])
@@ -1080,6 +1160,32 @@ class TestNovaServerBasic(base.SenlinTestCase):
         cc.server_get.assert_called_with('FAKE_ID')
         self.assertTrue(res)
 
+    def test_do_check_no_physical_id(self):
+        obj = mock.Mock(physical_id=None)
+        profile = server.ServerProfile('t', self.spec)
+
+        # do it
+        res = profile.do_check(obj)
+
+        self.assertFalse(res)
+
+    def test_do_check_no_server(self):
+        profile = server.ServerProfile('t', self.spec)
+        cc = mock.Mock()
+        err = exc.InternalError(code=404, message='No Server found')
+        cc.server_get.side_effect = err
+        profile._computeclient = cc
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        ex = self.assertRaises(exc.EServerNotFound,
+                               profile.do_check,
+                               node_obj)
+
+        self.assertEqual("Failed in found server 'FAKE_ID': "
+                         "No Server found.",
+                         six.text_type(ex))
+        cc.server_get.assert_called_once_with('FAKE_ID')
+
     @mock.patch.object(server.ServerProfile, 'do_delete')
     @mock.patch.object(server.ServerProfile, 'do_create')
     def test_do_recover_operation_is_none(self, mock_create, mock_delete):
@@ -1186,9 +1292,9 @@ class TestNovaServerBasic(base.SenlinTestCase):
         res = profile.handle_reboot(obj, type='foo')
         self.assertFalse(res)
 
-    def test_handle_rebuild(self):
+    def test_handle_rebuild_with_image(self):
         profile = server.ServerProfile('t', self.spec)
-        x_image = {'id': '123'}
+        x_image = '123'
         x_server = mock.Mock(image=x_image)
         cc = mock.Mock()
         cc.server_get.return_value = x_server
@@ -1198,7 +1304,46 @@ class TestNovaServerBasic(base.SenlinTestCase):
 
         res = profile.handle_rebuild(node_obj)
 
-        self.assertTrue(res)
+        self.assertEqual('FAKE_ID', res)
+        cc.server_get.assert_called_with('FAKE_ID')
+        cc.server_rebuild.assert_called_once_with('FAKE_ID', '123',
+                                                  'FAKE_SERVER_NAME',
+                                                  'adminpass')
+        cc.wait_for_server.assert_called_once_with('FAKE_ID', 'ACTIVE')
+
+    def test_handle_rebuild_with_bdm(self):
+        bdm_v2 = [
+            {
+                'volume_size': 1,
+                'uuid': '123',
+                'source_type': 'image',
+                'destination_type': 'volume',
+                'boot_index': 0,
+            }
+        ]
+        spec = {
+            'type': 'os.nova.server',
+            'version': '1.0',
+            'properties': {
+                'flavor': 'FLAV',
+                'admin_pass': 'adminpass',
+                'name': 'FAKE_SERVER_NAME',
+                'security_groups': ['HIGH_SECURITY_GROUP'],
+                'block_device_mapping_v2': bdm_v2,
+            }
+        }
+        profile = server.ServerProfile('t', spec)
+        x_image = '123'
+        x_server = mock.Mock(image=x_image)
+        cc = mock.Mock()
+        cc.server_get.return_value = x_server
+        cc.server_rebuild.return_value = True
+        profile._computeclient = cc
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+
+        res = profile.handle_rebuild(node_obj)
+
+        self.assertEqual('FAKE_ID', res)
         cc.server_get.assert_called_with('FAKE_ID')
         cc.server_rebuild.assert_called_once_with('FAKE_ID', '123',
                                                   'FAKE_SERVER_NAME',
@@ -1224,7 +1369,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
 
     def test_handle_rebuild_failed_rebuild(self):
         profile = server.ServerProfile('t', self.spec)
-        x_image = {'id': '123'}
+        x_image = '123'
         x_server = mock.Mock(image=x_image)
         cc = mock.Mock()
         cc.server_get.return_value = x_server
@@ -1248,7 +1393,7 @@ class TestNovaServerBasic(base.SenlinTestCase):
 
     def test_handle_rebuild_failed_waiting(self):
         profile = server.ServerProfile('t', self.spec)
-        x_image = {'id': '123'}
+        x_image = '123'
         x_server = mock.Mock(image=x_image)
         cc = mock.Mock()
         cc.server_get.return_value = x_server
@@ -1294,6 +1439,34 @@ class TestNovaServerBasic(base.SenlinTestCase):
 
         self.assertFalse(res)
 
+    def test_handle_rebuild_failed_with_name(self):
+        self.spec['properties']['name'] = None
+        profile = server.ServerProfile('t', self.spec)
+        x_image = '123'
+        x_server = mock.Mock(image=x_image)
+        cc = mock.Mock()
+        cc.server_get.return_value = x_server
+        ex = exc.InternalError(code=400,
+                               message='Server name is not '
+                                       'a string or unicode.')
+        cc.server_rebuild.side_effect = ex
+        profile._computeclient = cc
+        node_obj = mock.Mock(physical_id='FAKE_ID')
+        node_obj.name = None
+
+        ex = self.assertRaises(exc.EResourceOperation,
+                               profile.handle_rebuild,
+                               node_obj)
+
+        self.assertEqual("Failed in rebuilding server 'FAKE_ID': "
+                         "Server name is not a string or unicode.",
+                         six.text_type(ex))
+        cc.server_get.assert_called_once_with('FAKE_ID')
+        cc.server_rebuild.assert_called_once_with('FAKE_ID', '123',
+                                                  None,
+                                                  'adminpass')
+        self.assertEqual(0, cc.wait_for_server.call_count)
+
     def test_handle_change_password(self):
         obj = mock.Mock(physical_id='FAKE_ID')
         profile = server.ServerProfile('t', self.spec)
