@@ -25,6 +25,7 @@ from senlin.engine import parser
 from senlin.objects import credential as co
 from senlin.objects import profile as po
 from senlin.profiles import base as pb
+from senlin.profiles.os.nova import server as nova_server
 from senlin.tests.unit.common import base
 from senlin.tests.unit.common import utils
 
@@ -87,9 +88,9 @@ class TestProfileBase(base.SenlinTestCase):
 
     def _create_profile(self, name, pid=None, context=None):
         profile = pb.Profile(name, self.spec,
-                             user=self.ctx.user,
-                             project=self.ctx.project,
-                             domain=self.ctx.domain,
+                             user=self.ctx.user_id,
+                             project=self.ctx.project_id,
+                             domain=self.ctx.domain_id,
                              context=context)
         if pid:
             profile.id = pid
@@ -108,9 +109,9 @@ class TestProfileBase(base.SenlinTestCase):
         self.assertEqual('os.dummy', profile.type_name)
         self.assertEqual('1.0', profile.version)
         self.assertEqual('os.dummy-1.0', profile.type)
-        self.assertEqual(self.ctx.user, profile.user)
-        self.assertEqual(self.ctx.project, profile.project)
-        self.assertEqual(self.ctx.domain, profile.domain)
+        self.assertEqual(self.ctx.user_id, profile.user)
+        self.assertEqual(self.ctx.project_id, profile.project)
+        self.assertEqual(self.ctx.domain_id, profile.domain)
         self.assertEqual({}, profile.metadata)
         self.assertIsNone(profile.created_at)
         self.assertIsNone(profile.updated_at)
@@ -127,6 +128,7 @@ class TestProfileBase(base.SenlinTestCase):
         self.assertIsNone(profile._computeclient)
         self.assertIsNone(profile._networkclient)
         self.assertIsNone(profile._orchestrationclient)
+        self.assertIsNone(profile._block_storageclient)
 
     @mock.patch.object(senlin_ctx, 'get_service_credentials')
     def test_init_with_context(self, mock_creds):
@@ -365,6 +367,18 @@ class TestProfileBase(base.SenlinTestCase):
         res_obj = profile.do_delete.return_value
         self.assertEqual(res_obj, res)
 
+    @mock.patch.object(pb.Profile, 'load')
+    def test_check_object_exception_return_value(self, mock_load):
+        profile = pb.Profile
+        profile.load(self.ctx).do_check = mock.Mock(
+            side_effect=exception.InternalError(code=400, message='BAD'))
+        obj = mock_load
+
+        res = profile.check_object(self.ctx, obj)
+
+        profile.load(self.ctx).do_check.assert_called_once_with(obj)
+        self.assertFalse(res)
+
     @mock.patch.object(pb.Profile, 'load')
     def test_update_object_with_profile(self, mock_load):
         old_profile = mock.Mock()
@@ -480,6 +494,17 @@ class TestProfileBase(base.SenlinTestCase):
         actual = DummyProfile.get_ops()
         self.assertEqual(expected, actual)
 
+    @mock.patch.object(nova_server.ServerProfile, 'do_adopt')
+    def test_adopt_node(self, mock_adopt):
+        obj = mock.Mock()
+
+        res = pb.Profile.adopt_node(self.ctx, obj, "os.nova.server-1.0",
+                                    overrides=None, snapshot=False)
+
+        mock_adopt.assert_called_once_with(obj, overrides=None, snapshot=False)
+        res_obj = mock_adopt.return_value
+        self.assertEqual(res_obj, res)
+
     @mock.patch.object(pb.Profile, 'load')
     def test_join_cluster(self, mock_load):
         profile = mock.Mock()
@@ -538,9 +563,9 @@ class TestProfileBase(base.SenlinTestCase):
                 "key2": 2,
             }
         }
-        profile = DummyProfile("p-bad-ctx", spec, user=self.ctx.user,
-                               project=self.ctx.project,
-                               domain=self.ctx.domain)
+        profile = DummyProfile("p-bad-ctx", spec, user=self.ctx.user_id,
+                               project=self.ctx.project_id,
+                               domain=self.ctx.domain_id)
 
         self.assertRaises(exception.ESchema, profile.validate)
 
@@ -690,6 +715,25 @@ class TestProfileBase(base.SenlinTestCase):
         mock_params.assert_called_once_with(obj.user, obj.project)
         sd.network.assert_called_once_with(fake_params)
 
+    @mock.patch.object(pb.Profile, '_build_conn_params')
+    @mock.patch("senlin.drivers.base.SenlinDriver")
+    def test_cinder_client(self, mock_senlindriver, mock_params):
+        obj = mock.Mock()
+        sd = mock.Mock()
+        nc = mock.Mock()
+        sd.block_storage.return_value = nc
+        mock_senlindriver.return_value = sd
+        fake_params = mock.Mock()
+        mock_params.return_value = fake_params
+        profile = self._create_profile('test-profile')
+
+        res = profile.block_storage(obj)
+
+        self.assertEqual(nc, res)
+        self.assertEqual(nc, profile._block_storageclient)
+        mock_params.assert_called_once_with(obj.user, obj.project)
+        sd.block_storage.assert_called_once_with(fake_params)
+
     def test_interface_methods(self):
         profile = self._create_profile('test-profile')
 
@@ -710,10 +754,7 @@ class TestProfileBase(base.SenlinTestCase):
         res = profile.do_recover(mock.Mock())
         self.assertTrue(res)
 
-        res = profile.do_recover(mock.Mock(), foo='bar')
-        self.assertTrue(res)
-
-        res = profile.do_recover(mock.Mock(), operation='bar')
+        res = profile.do_recover(mock.Mock(), operation=[{'name': 'bar'}])
         self.assertFalse(res)
 
     def test_do_recover_with_fencing(self):
@@ -722,19 +763,20 @@ class TestProfileBase(base.SenlinTestCase):
         self.patchobject(profile, 'do_delete', return_value=True)
         obj = mock.Mock()
 
-        res = profile.do_recover(obj, ignore_missing=True, force=True)
-        self.assertTrue(res)
+        res = profile.do_recover(obj, ignore_missing=True,
+                                 params={"fence_compute": True})
 
-        profile.do_delete.assert_called_once_with(
-            obj, ignore_missing=True, force=True)
+        self.assertTrue(res)
+        profile.do_delete.assert_called_once_with(obj, force=True)
         profile.do_create.assert_called_once_with(obj)
 
     def test_do_recover_with_recreate_succeeded(self):
         profile = self._create_profile('test-profile')
-
         self.patchobject(profile, 'do_delete', return_value=True)
         self.patchobject(profile, 'do_create', return_value=True)
-        res = profile.do_recover(mock.Mock(), operation='RECREATE')
+        operation = [{"name": "RECREATE"}]
+        res = profile.do_recover(mock.Mock(), operation=operation)
+
         self.assertTrue(res)
 
     def test_do_recover_with_recreate_failed_delete(self):
@@ -742,10 +784,11 @@ class TestProfileBase(base.SenlinTestCase):
         err = exception.EResourceDeletion(type='STACK', id='ID',
                                           message='BANG')
         self.patchobject(profile, 'do_delete', side_effect=err)
+        operation = [{"name": "RECREATE"}]
 
         ex = self.assertRaises(exception.EResourceOperation,
                                profile.do_recover,
-                               mock.Mock(id='NODE_ID'), operation='RECREATE')
+                               mock.Mock(id='NODE_ID'), operation=operation)
         self.assertEqual("Failed in recovering node 'NODE_ID': "
                          "Failed in deleting STACK 'ID': BANG.",
                          six.text_type(ex))
@@ -755,10 +798,12 @@ class TestProfileBase(base.SenlinTestCase):
         self.patchobject(profile, 'do_delete', return_value=True)
         err = exception.EResourceCreation(type='STACK', message='BANNG')
         self.patchobject(profile, 'do_create', side_effect=err)
+        operation = [{"name": "RECREATE"}]
 
         ex = self.assertRaises(exception.EResourceOperation,
                                profile.do_recover,
-                               mock.Mock(id='NODE_ID'), operation='RECREATE')
+                               mock.Mock(id='NODE_ID'), operation=operation)
+
         msg = ("Failed in recovering node 'NODE_ID': Failed in creating "
                "STACK: BANNG.")
         self.assertEqual(msg, six.text_type(ex))
@@ -791,9 +836,9 @@ class TestProfileBase(base.SenlinTestCase):
         new_spec['properties']['key1'] = 'new_v1'
         new_spec['properties']['key2'] = 3
         new_profile = pb.Profile('new-profile', new_spec,
-                                 user=self.ctx.user,
-                                 project=self.ctx.project,
-                                 domain=self.ctx.domain,
+                                 user=self.ctx.user_id,
+                                 project=self.ctx.project_id,
+                                 domain=self.ctx.domain_id,
                                  context=None)
         res = profile.validate_for_update(new_profile)
         self.assertTrue(res)
@@ -805,9 +850,9 @@ class TestProfileBase(base.SenlinTestCase):
         new_spec = copy.deepcopy(self.spec)
         new_spec['properties']['key3'] = 'new_v3'
         new_profile = pb.Profile('new-profile', new_spec,
-                                 user=self.ctx.user,
-                                 project=self.ctx.project,
-                                 domain=self.ctx.domain,
+                                 user=self.ctx.user_id,
+                                 project=self.ctx.project_id,
+                                 domain=self.ctx.domain_id,
                                  context=None)
 
         res = profile.validate_for_update(new_profile)
