@@ -15,8 +15,7 @@ import six
 import testtools
 
 from senlin.common import constraints
-from senlin.common import exception
-from senlin.common.i18n import _
+from senlin.common import exception as exc
 from senlin.common import schema
 
 
@@ -45,15 +44,13 @@ class TestConstraintsSchema(testtools.TestCase):
         # ... and value as number or string
         self.assertIsNone(s.validate(1))
 
-        err = self.assertRaises(exception.SpecValidationFailed,
-                                s.validate, 3)
-        self.assertEqual('"3" must be one of the allowed values: 1, 2, 4',
+        err = self.assertRaises(exc.ESchema, s.validate, 3)
+        self.assertEqual("'3' must be one of the allowed values: 1, 2, 4",
                          six.text_type(err))
 
         self.assertIsNone(s.validate('1'))
-        err = self.assertRaises(exception.SpecValidationFailed,
-                                s.validate, '3')
-        self.assertEqual('"3" must be one of the allowed values: 1, 2, 4',
+        err = self.assertRaises(exc.ESchema, s.validate, '3')
+        self.assertEqual("'3' must be one of the allowed values: 1, 2, 4",
                          six.text_type(err))
 
         # Allowed values defined as integer strings
@@ -62,15 +59,13 @@ class TestConstraintsSchema(testtools.TestCase):
         )
         # ... and value as number or string
         self.assertIsNone(s.validate(1))
-        err = self.assertRaises(exception.SpecValidationFailed,
-                                s.validate, 3)
-        self.assertEqual('"3" must be one of the allowed values: 1, 2, 4',
+        err = self.assertRaises(exc.ESchema, s.validate, 3)
+        self.assertEqual("'3' must be one of the allowed values: 1, 2, 4",
                          six.text_type(err))
 
         self.assertIsNone(s.validate('1'))
-        err = self.assertRaises(exception.SpecValidationFailed,
-                                s.validate, '3')
-        self.assertEqual('"3" must be one of the allowed values: 1, 2, 4',
+        err = self.assertRaises(exc.ESchema, s.validate, '3')
+        self.assertEqual("'3' must be one of the allowed values: 1, 2, 4",
                          six.text_type(err))
 
     def test_allowed_values_numeric_float(self):
@@ -87,15 +82,13 @@ class TestConstraintsSchema(testtools.TestCase):
         )
         # ... and value as number or string
         self.assertIsNone(s.validate_constraints(1.1))
-        err = self.assertRaises(exception.SpecValidationFailed,
-                                s.validate_constraints, 3.3)
-        self.assertEqual('"3.3" must be one of the allowed values: '
-                         '1.1, 2.2, 4.4', six.text_type(err))
+        err = self.assertRaises(exc.ESchema, s.validate_constraints, 3.3)
+        self.assertEqual("'3.3' must be one of the allowed values: "
+                         "1.1, 2.2, 4.4", six.text_type(err))
         self.assertIsNone(s.validate_constraints('1.1', s))
-        err = self.assertRaises(exception.SpecValidationFailed,
-                                s.validate_constraints, '3.3')
-        self.assertEqual('"3.3" must be one of the allowed values: '
-                         '1.1, 2.2, 4.4', six.text_type(err))
+        err = self.assertRaises(exc.ESchema, s.validate_constraints, '3.3')
+        self.assertEqual("'3.3' must be one of the allowed values: "
+                         "1.1, 2.2, 4.4", six.text_type(err))
 
         # Allowed values defined as strings
         s = schema.Number(
@@ -103,23 +96,21 @@ class TestConstraintsSchema(testtools.TestCase):
         )
         # ... and value as number or string
         self.assertIsNone(s.validate_constraints(1.1, s))
-        err = self.assertRaises(exception.SpecValidationFailed,
-                                s.validate_constraints, 3.3, s)
-        self.assertEqual('"3.3" must be one of the allowed values: '
-                         '1.1, 2.2, 4.4', six.text_type(err))
+        err = self.assertRaises(exc.ESchema, s.validate_constraints, 3.3, s)
+        self.assertEqual("'3.3' must be one of the allowed values: "
+                         "1.1, 2.2, 4.4", six.text_type(err))
         self.assertIsNone(s.validate_constraints('1.1', s))
-        err = self.assertRaises(exception.SpecValidationFailed,
-                                s.validate_constraints, '3.3', s)
-        self.assertEqual('"3.3" must be one of the allowed values: '
-                         '1.1, 2.2, 4.4', six.text_type(err))
+        err = self.assertRaises(exc.ESchema, s.validate_constraints, '3.3', s)
+        self.assertEqual("'3.3' must be one of the allowed values: "
+                         "1.1, 2.2, 4.4", six.text_type(err))
 
     def test_schema_all(self):
         d = {
             'type': 'String',
             'description': 'A string',
             'default': 'wibble',
-            'readonly': False,
             'required': True,
+            'updatable': False,
             'constraints': [{
                 'constraint': ['foo', 'bar'],
                 'type': 'AllowedValues'
@@ -139,16 +130,16 @@ class TestConstraintsSchema(testtools.TestCase):
                     'type': 'String',
                     'description': 'A string',
                     'default': 'wibble',
-                    'readonly': False,
                     'required': True,
+                    'updatable': False,
                     'constraints': [{
                         'constraint': ['foo', 'bar'],
                         'type': 'AllowedValues'
                     }]
                 }
             },
-            'readonly': False,
             'required': False,
+            'updatable': False,
         }
         c = constraints.AllowedValues(['foo', 'bar'])
         s = schema.String('A string', default='wibble', required=True,
@@ -165,16 +156,16 @@ class TestConstraintsSchema(testtools.TestCase):
                     'type': 'String',
                     'description': 'A string',
                     'default': 'wibble',
-                    'readonly': False,
                     'required': True,
+                    'updatable': False,
                     'constraints': [{
                         'type': 'AllowedValues',
                         'constraint': ['foo', 'bar']
                     }]
                 }
             },
-            'readonly': False,
             'required': False,
+            'updatable': False,
         }
         c = constraints.AllowedValues(['foo', 'bar'])
         s = schema.String('A string', default='wibble', required=True,
@@ -195,20 +186,20 @@ class TestConstraintsSchema(testtools.TestCase):
                             'type': 'String',
                             'description': 'A string',
                             'default': 'wibble',
-                            'readonly': False,
                             'required': True,
+                            'updatable': False,
                             'constraints': [{
                                 'type': 'AllowedValues',
                                 'constraint': ['foo', 'bar']
                             }]
                         }
                     },
-                    'readonly': False,
                     'required': False,
+                    'updatable': False,
                 }
             },
-            'readonly': False,
             'required': False,
+            'updatable': False,
         }
         c = constraints.AllowedValues(['foo', 'bar'])
         s = schema.String('A string', default='wibble', required=True,
@@ -217,11 +208,6 @@ class TestConstraintsSchema(testtools.TestCase):
         l = schema.List('A list', schema=m)
         self.assertEqual(d, dict(l))
 
-    def test_schema_invalid_type(self):
-        self.assertRaises(exception.InvalidSchemaError,
-                          schema.String,
-                          schema=schema.String('String'))
-
     def test_schema_validate_good(self):
         c = constraints.AllowedValues(['foo', 'bar'])
         s = schema.String('A string', default='wibble', required=True,
@@ -232,10 +218,8 @@ class TestConstraintsSchema(testtools.TestCase):
         c = constraints.AllowedValues(['foo', 'bar'])
         s = schema.String('A string', default='wibble', required=True,
                           constraints=[c])
-        err = self.assertRaises(exception.SpecValidationFailed,
-                                s.validate,
-                                'zoo')
-        self.assertIn('"zoo" must be one of the allowed values: foo, bar',
+        err = self.assertRaises(exc.ESchema, s.validate, 'zoo')
+        self.assertIn("'zoo' must be one of the allowed values: foo, bar",
                       six.text_type(err))
 
     def test_schema_nested_validate_good(self):
@@ -250,94 +234,7 @@ class TestConstraintsSchema(testtools.TestCase):
         nested = schema.String('A string', default='wibble', required=True,
                                constraints=[c])
         s = schema.Map('A map', schema={'Foo': nested})
-        err = self.assertRaises(exception.SpecValidationFailed, s.validate,
-                                {'Foo': 'zoo'})
+        err = self.assertRaises(exc.ESchema, s.validate, {'Foo': 'zoo'})
 
-        self.assertIn('"zoo" must be one of the allowed values: foo, bar',
+        self.assertIn("'zoo' must be one of the allowed values: foo, bar",
                       six.text_type(err))
-
-    def test_spec_validate_good(self):
-        spec_schema = {
-            'key1': schema.String('first key', default='value1'),
-            'key2': schema.Integer('second key', required=True),
-        }
-
-        data = {'key1': 'value1', 'key2': 2}
-        spec = schema.Spec(spec_schema, data)
-        self.assertEqual(None, spec.validate())
-
-        data = {'key2': 2}
-        spec = schema.Spec(spec_schema, data)
-        self.assertEqual(None, spec.validate())
-
-    def test_spec_validate_fail_value_type_incorrect(self):
-        spec_schema = {
-            'key1': schema.String('first key', default='value1'),
-            'key2': schema.Integer('second key', required=True),
-        }
-
-        data = {'key1': 'value1', 'key2': 'abc'}
-        spec = schema.Spec(spec_schema, data)
-        ex = self.assertRaises(exception.SpecValidationFailed,
-                               spec.validate)
-        msg = _('The value "%s" cannot be converted into an '
-                'integer.') % data['key2']
-        self.assertTrue(six.text_type(ex.message).find(msg) != -1)
-
-    def test_policy_validate_fail_unrecognizable_key(self):
-        spec_schema = {
-            'key1': schema.String('first key', default='value1'),
-        }
-
-        data = {'key1': 'value1', 'key2': 2}
-        spec = schema.Spec(spec_schema, data)
-        ex = self.assertRaises(exception.SpecValidationFailed,
-                               spec.validate)
-        msg = _('Unrecognizable spec item "%s"') % 'key2'
-        self.assertTrue(six.text_type(ex.message).find(msg) != -1)
-
-    def test_policy_validate_fail_required_key_missing(self):
-        spec_schema = {
-            'key1': schema.String('first key', default='value1'),
-            'key2': schema.Integer('second key', required=True),
-        }
-
-        data = {'key1': 'value1'}
-        spec = schema.Spec(spec_schema, data)
-        ex = self.assertRaises(exception.SpecValidationFailed,
-                               spec.validate)
-        msg = _('Required spec item "%s" not assigned') % 'key2'
-        self.assertTrue(six.text_type(ex.message).find(msg) != -1)
-
-
-class TestSpecVersionChecking(testtools.TestCase):
-
-    def test_spec_version_okay(self):
-        spec = {'type': 'Foo', 'version': 'version string'}
-        res = schema.get_spec_version(spec)
-        self.assertEqual(('Foo', 'version string'), res)
-
-        spec = {'type': 'Foo', 'version': 1.5}
-        res = schema.get_spec_version(spec)
-        self.assertEqual(('Foo', 1.5), res)
-
-    def test_spec_version_not_dict(self):
-        spec = 'a string'
-        ex = self.assertRaises(exception.SpecValidationFailed,
-                               schema.get_spec_version, spec)
-        self.assertEqual('The provided spec is not a map.',
-                         six.text_type(ex))
-
-    def test_spec_version_no_type_key(self):
-        spec = {'tpye': 'a string'}
-        ex = self.assertRaises(exception.SpecValidationFailed,
-                               schema.get_spec_version, spec)
-        self.assertEqual("The 'type' key is missing from the provided "
-                         "spec map.", six.text_type(ex))
-
-    def test_spec_version_no_version_key(self):
-        spec = {'type': 'a string', 'ver': '123'}
-        ex = self.assertRaises(exception.SpecValidationFailed,
-                               schema.get_spec_version, spec)
-        self.assertEqual("The 'version' key is missing from the provided "
-                         "spec map.", six.text_type(ex))
