@@ -201,11 +201,14 @@ class ScaleUtilsTest(base.SenlinTestCase):
             mock.Mock(id='N4', status='ACTIVE'),
             mock.Mock(id='N5', status='WARNING'),
             mock.Mock(id='N6', status='ERROR'),
+            mock.Mock(id='N7', created_at=None)
         ]
         res = su.filter_error_nodes(nodes)
         self.assertIn('N3', res[0])
+        self.assertIn('N5', res[0])
         self.assertIn('N6', res[0])
-        self.assertEqual(4, len(res[1]))
+        self.assertIn('N7', res[0])
+        self.assertEqual(3, len(res[1]))
 
     @mock.patch.object(su, 'filter_error_nodes')
     def test_nodes_by_random(self, mock_filter):
@@ -278,11 +281,11 @@ class ScaleUtilsTest(base.SenlinTestCase):
     @mock.patch.object(su, 'filter_error_nodes')
     def test__victims_by_profile_age_oldest(self, mock_filter):
         good_nodes = [
-            mock.Mock(id='N11', rt={'profile': mock.Mock(created_at=110)}),
-            mock.Mock(id='N15', rt={'profile': mock.Mock(created_at=150)}),
-            mock.Mock(id='N12', rt={'profile': mock.Mock(created_at=120)}),
-            mock.Mock(id='N13', rt={'profile': mock.Mock(created_at=130)}),
-            mock.Mock(id='N14', rt={'profile': mock.Mock(created_at=140)}),
+            mock.Mock(id='N11', profile_created_at=110),
+            mock.Mock(id='N15', profile_created_at=150),
+            mock.Mock(id='N12', profile_created_at=120),
+            mock.Mock(id='N13', profile_created_at=130),
+            mock.Mock(id='N14', profile_created_at=140),
         ]
         mock_filter.return_value = (['N1', 'N2'], good_nodes)
 
@@ -394,7 +397,7 @@ class CheckSizeParamsTest(base.SenlinTestCase):
 
     def setUp(self):
         super(CheckSizeParamsTest, self).setUp()
-        cfg.CONF.set_override('max_nodes_per_cluster', 100, enforce_type=True)
+        cfg.CONF.set_override('max_nodes_per_cluster', 100)
 
     def test_check_size_params(self):
         cluster = mock.Mock()
